@@ -7,10 +7,18 @@ module Path.Extra
   ,dropRoot
   ,parseCollapsedAbsDir
   ,parseCollapsedAbsFile
+  ,concatAndColapseAbsDir
   ,rejectMissingFile
   ,rejectMissingDir
+  ,pathToByteString
+  ,pathToLazyByteString
+  ,pathToText
   ) where
 
+import qualified Data.ByteString.Lazy.Char8 as BSL
+import qualified Data.ByteString.Char8 as BS
+import qualified Data.Text as T
+import qualified Data.Text.Encoding as T
 import           Control.Monad (liftM)
 import           Control.Monad.Catch
 import           Control.Monad.IO.Class
@@ -36,6 +44,12 @@ parseCollapsedAbsDir = parseAbsDir . collapseFilePath
 parseCollapsedAbsFile :: MonadThrow m => FilePath -> m (Path Abs File)
 parseCollapsedAbsFile = parseAbsFile . collapseFilePath
 
+-- | Add a relative FilePath to the end of a Path
+-- We can't parse the FilePath first because we need to account for ".."
+-- in the FilePath (#2895)
+concatAndColapseAbsDir :: MonadThrow m => Path Abs Dir -> FilePath -> m (Path Abs Dir)
+concatAndColapseAbsDir base rel = parseCollapsedAbsDir (toFilePath base FP.</> rel)
+
 -- | Collapse intermediate "." and ".." directories from a path.
 --
 -- > collapseFilePath "./foo" == "foo"
@@ -46,21 +60,19 @@ parseCollapsedAbsFile = parseAbsFile . collapseFilePath
 -- > collapseFilePath "parent/foo/.." ==  "parent"
 -- > collapseFilePath "/parent/foo/../../bar" ==  "/bar"
 --
--- (borrowed from @Text.Pandoc.Shared@)
+-- (adapted from @Text.Pandoc.Shared@)
 collapseFilePath :: FilePath -> FilePath
 collapseFilePath = FP.joinPath . reverse . foldl go [] . FP.splitDirectories
   where
     go rs "." = rs
     go r@(p:rs) ".." = case p of
                             ".." -> "..":r
-                            (checkPathSeperator -> Just True) -> "..":r
+                            (checkPathSeparator -> True) -> "..":r
                             _ -> rs
-    go _ (checkPathSeperator -> Just True) = [[FP.pathSeparator]]
+    go _ (checkPathSeparator -> True) = [[FP.pathSeparator]]
     go rs x = x:rs
-    isSingleton [] = Nothing
-    isSingleton [x] = Just x
-    isSingleton _ = Nothing
-    checkPathSeperator = fmap FP.isPathSeparator . isSingleton
+    checkPathSeparator [x] = FP.isPathSeparator x
+    checkPathSeparator _ = False
 
 -- | Drop the root (either @\/@ on POSIX or @C:\\@, @D:\\@, etc. on
 -- Windows).
@@ -94,3 +106,14 @@ rejectMissingDir :: MonadIO m
   -> m (Maybe (Path Abs Dir))
 rejectMissingDir Nothing = return Nothing
 rejectMissingDir (Just p) = bool Nothing (Just p) `liftM` doesDirExist p
+
+-- | Convert to a lazy ByteString using toFilePath and UTF8.
+pathToLazyByteString :: Path b t -> BSL.ByteString
+pathToLazyByteString = BSL.fromStrict . pathToByteString
+
+-- | Convert to a ByteString using toFilePath and UTF8.
+pathToByteString :: Path b t -> BS.ByteString
+pathToByteString = T.encodeUtf8 . pathToText
+
+pathToText :: Path b t -> T.Text
+pathToText = T.pack . toFilePath
