@@ -42,14 +42,23 @@ import           Path.Extra
 import           Path.IO
 import           Prelude
 import           Stack.PackageDump
-import           Stack.Types
+import           Stack.PrettyPrint
+import           Stack.Types.Build
+import           Stack.Types.Compiler
+import           Stack.Types.Config
+import           Stack.Types.GhcPkgId
+import           Stack.Types.Package
+import           Stack.Types.PackageIdentifier
+import           Stack.Types.PackageName
+import           Stack.Types.StackT (StackM)
+import           Stack.Types.StringError
 import qualified System.FilePath as FP
 import           System.IO.Error (isDoesNotExistError)
 import           System.Process.Read
 import           Web.Browser (openBrowser)
 
 openHaddocksInBrowser
-    :: (MonadIO m, MonadThrow m, MonadLogger m)
+    :: StackM env m
     => BaseConfigOpts
     -> Map PackageName (PackageIdentifier, InstallLocation)
     -- ^ Available packages and their locations for the current project
@@ -68,7 +77,7 @@ openHaddocksInBrowser bco pkgLocations buildTargets = do
                     snapExists <- doesFileExist snapDocs
                     if snapExists
                         then return snapDocs
-                        else fail "No local or snapshot doc index found to open."
+                        else throwString "No local or snapshot doc index found to open."
     docFile <-
         case (cliTargets, map (`Map.lookup` pkgLocations) (Set.toList buildTargets)) of
             ([_], [Just (pkgId, iloc)]) -> do
@@ -88,7 +97,7 @@ openHaddocksInBrowser bco pkgLocations buildTargets = do
                             ", but that file is missing.  Opening doc index instead."
                         getDocIndex
             _ -> getDocIndex
-    $logInfo ("Opening " <> T.pack (toFilePath docFile) <> " in the browser.")
+    $prettyInfo $ "Opening" <+> display docFile <+> "in the browser."
     _ <- liftIO $ openBrowser (toFilePath docFile)
     return ()
 
@@ -114,7 +123,7 @@ generateLocalHaddockIndex
     => EnvOverride
     -> WhichCompiler
     -> BaseConfigOpts
-    -> Map GhcPkgId (DumpPackage () ())  -- ^ Local package dump
+    -> Map GhcPkgId (DumpPackage () () ())  -- ^ Local package dump
     -> [LocalPackage]
     -> m ()
 generateLocalHaddockIndex envOverride wc bco localDumpPkgs locals = do
@@ -140,9 +149,9 @@ generateDepsHaddockIndex
     => EnvOverride
     -> WhichCompiler
     -> BaseConfigOpts
-    -> Map GhcPkgId (DumpPackage () ())  -- ^ Global dump information
-    -> Map GhcPkgId (DumpPackage () ())  -- ^ Snapshot dump information
-    -> Map GhcPkgId (DumpPackage () ())  -- ^ Local dump information
+    -> Map GhcPkgId (DumpPackage () () ())  -- ^ Global dump information
+    -> Map GhcPkgId (DumpPackage () () ())  -- ^ Snapshot dump information
+    -> Map GhcPkgId (DumpPackage () () ())  -- ^ Local dump information
     -> [LocalPackage]
     -> m ()
 generateDepsHaddockIndex envOverride wc bco globalDumpPkgs snapshotDumpPkgs localDumpPkgs locals = do
@@ -185,8 +194,8 @@ generateSnapHaddockIndex
     => EnvOverride
     -> WhichCompiler
     -> BaseConfigOpts
-    -> Map GhcPkgId (DumpPackage () ())  -- ^ Global package dump
-    -> Map GhcPkgId (DumpPackage () ())  -- ^ Snapshot package dump
+    -> Map GhcPkgId (DumpPackage () () ())  -- ^ Global package dump
+    -> Map GhcPkgId (DumpPackage () () ())  -- ^ Snapshot package dump
     -> m ()
 generateSnapHaddockIndex envOverride wc bco globalDumpPkgs snapshotDumpPkgs =
     generateHaddockIndex
@@ -205,7 +214,7 @@ generateHaddockIndex
     -> EnvOverride
     -> WhichCompiler
     -> HaddockOpts
-    -> [DumpPackage () ()]
+    -> [DumpPackage () () ()]
     -> FilePath
     -> Path Abs Dir
     -> m ()
@@ -220,20 +229,25 @@ generateHaddockIndex descr envOverride wc hdopts dumpPackages docRelFP destDir =
                     Left _ -> True
                     Right indexModTime ->
                         or [mt > indexModTime | (_,mt,_,_) <- interfaceOpts]
-        when needUpdate $ do
-            $logInfo
-                (T.concat ["Updating Haddock index for ", descr, " in\n",
-                           T.pack (toFilePath destIndexFile)])
-            liftIO (mapM_ copyPkgDocs interfaceOpts)
-            readProcessNull
-                (Just destDir)
-                envOverride
-                (haddockExeName wc)
-                (toHaddockArgs hdopts ++
-                 ["--gen-contents", "--gen-index"] ++
-                 [x | (xs,_,_,_) <- interfaceOpts, x <- xs])
+        if needUpdate
+            then do
+                $logInfo
+                    (T.concat ["Updating Haddock index for ", descr, " in\n",
+                               T.pack (toFilePath destIndexFile)])
+                liftIO (mapM_ copyPkgDocs interfaceOpts)
+                readProcessNull
+                    (Just destDir)
+                    envOverride
+                    (haddockExeName wc)
+                    (hoAdditionalArgs hdopts ++
+                     ["--gen-contents", "--gen-index"] ++
+                     [x | (xs,_,_,_) <- interfaceOpts, x <- xs])
+            else
+              $logInfo
+                    (T.concat ["Haddock index for ", descr, " already up to date at:\n",
+                               T.pack (toFilePath destIndexFile)])
   where
-    toInterfaceOpt :: DumpPackage a b -> IO (Maybe ([String], UTCTime, Path Abs File, Path Abs File))
+    toInterfaceOpt :: DumpPackage a b c -> IO (Maybe ([String], UTCTime, Path Abs File, Path Abs File))
     toInterfaceOpt DumpPackage {..} =
         case dpHaddockInterfaces of
             [] -> return Nothing
@@ -286,8 +300,8 @@ generateHaddockIndex descr envOverride wc hdopts dumpPackages docRelFP destDir =
 
 -- | Find first DumpPackage matching the GhcPkgId
 lookupDumpPackage :: GhcPkgId
-                  -> [Map GhcPkgId (DumpPackage () ())]
-                  -> Maybe (DumpPackage () ())
+                  -> [Map GhcPkgId (DumpPackage () () ())]
+                  -> Maybe (DumpPackage () () ())
 lookupDumpPackage ghcPkgId dumpPkgs =
     listToMaybe $ mapMaybe (Map.lookup ghcPkgId) dumpPkgs
 
