@@ -1,7 +1,7 @@
 {-# LANGUAGE ConstraintKinds       #-}
 {-# LANGUAGE FlexibleContexts      #-}
 {-# LANGUAGE MultiParamTypeClasses #-}
-{-# LANGUAGE OverloadedStrings #-}
+{-# LANGUAGE OverloadedStrings     #-}
 {-# LANGUAGE TemplateHaskell       #-}
 -- Determine which packages are already installed
 module Stack.Build.Installed
@@ -12,38 +12,38 @@ module Stack.Build.Installed
     ) where
 
 import           Control.Applicative
+import           Control.Arrow
 import           Control.Monad
-import           Control.Monad.Catch          (MonadMask)
-import           Control.Monad.IO.Class
 import           Control.Monad.Logger
-import           Control.Monad.Reader         (MonadReader, asks)
-import           Control.Monad.Trans.Resource
 import           Data.Conduit
-import qualified Data.Conduit.List            as CL
+import qualified Data.Conduit.List as CL
+import qualified Data.Foldable as F
 import           Data.Function
-import qualified Data.Foldable                as F
-import qualified Data.HashSet                 as HashSet
+import qualified Data.HashSet as HashSet
 import           Data.List
-import           Data.Map.Strict              (Map)
-import qualified Data.Map.Strict              as M
-import qualified Data.Map.Strict              as Map
+import           Data.Map.Strict (Map)
+import qualified Data.Map.Strict as M
+import qualified Data.Map.Strict as Map
 import           Data.Maybe
-import           Data.Maybe.Extra             (mapMaybeM)
+import           Data.Maybe.Extra (mapMaybeM)
 import           Data.Monoid
-import qualified Data.Text                    as T
-import           Network.HTTP.Client.Conduit  (HasHttpManager)
+import qualified Data.Text as T
 import           Path
-import           Prelude                      hiding (FilePath, writeFile)
+import           Prelude hiding (FilePath, writeFile)
 import           Stack.Build.Cache
-import           Stack.Types.Build
-import           Stack.Types.Version
 import           Stack.Constants
 import           Stack.GhcPkg
 import           Stack.PackageDump
-import           Stack.Types
-import           Stack.Types.Internal
-
-type M env m = (MonadIO m,MonadReader env m,HasHttpManager env,HasEnvConfig env,MonadLogger m,MonadBaseControl IO m,MonadMask m,HasLogLevel env)
+import           Stack.Types.Build
+import           Stack.Types.Compiler
+import           Stack.Types.Config
+import           Stack.Types.GhcPkgId
+import           Stack.Types.Package
+import           Stack.Types.PackageDump
+import           Stack.Types.PackageIdentifier
+import           Stack.Types.PackageName
+import           Stack.Types.StackT
+import           Stack.Types.Version
 
 -- | Options for 'getInstalled'.
 data GetInstalledOpts = GetInstalledOpts
@@ -51,44 +51,47 @@ data GetInstalledOpts = GetInstalledOpts
       -- ^ Require profiling libraries?
     , getInstalledHaddock   :: !Bool
       -- ^ Require haddocks?
+    , getInstalledSymbols   :: !Bool
+      -- ^ Require debugging symbols?
     }
 
 -- | Returns the new InstalledMap and all of the locally registered packages.
-getInstalled :: (M env m, PackageInstallInfo pii)
+getInstalled :: (StackM env m, HasEnvConfig env, PackageInstallInfo pii)
              => EnvOverride
              -> GetInstalledOpts
              -> Map PackageName pii -- ^ does not contain any installed information
              -> m ( InstalledMap
-                  , [DumpPackage () ()] -- globally installed
-                  , [DumpPackage () ()] -- snapshot installed
-                  , [DumpPackage () ()] -- locally installed
+                  , [DumpPackage () () ()] -- globally installed
+                  , [DumpPackage () () ()] -- snapshot installed
+                  , [DumpPackage () () ()] -- locally installed
                   )
 getInstalled menv opts sourceMap = do
+    $logDebug "Finding out which packages are already installed"
     snapDBPath <- packageDatabaseDeps
     localDBPath <- packageDatabaseLocal
     extraDBPaths <- packageDatabaseExtra
 
-    bconfig <- asks getBuildConfig
-
     mcache <-
         if getInstalledProfiling opts || getInstalledHaddock opts
-            then liftM Just $ loadInstalledCache $ configInstalledCache bconfig
+            then configInstalledCache >>= liftM Just . loadInstalledCache
             else return Nothing
 
     let loadDatabase' = loadDatabase menv opts mcache sourceMap
 
     (installedLibs0, globalDumpPkgs) <- loadDatabase' Nothing []
     (installedLibs1, _extraInstalled) <-
-      (foldM (\lhs' pkgdb ->
+      foldM (\lhs' pkgdb ->
         loadDatabase' (Just (ExtraGlobal, pkgdb)) (fst lhs')
-        ) (installedLibs0, globalDumpPkgs) extraDBPaths)
+        ) (installedLibs0, globalDumpPkgs) extraDBPaths
     (installedLibs2, snapshotDumpPkgs) <-
         loadDatabase' (Just (InstalledTo Snap, snapDBPath)) installedLibs1
     (installedLibs3, localDumpPkgs) <-
         loadDatabase' (Just (InstalledTo Local, localDBPath)) installedLibs2
     let installedLibs = M.fromList $ map lhPair installedLibs3
 
-    F.forM_ mcache (saveInstalledCache (configInstalledCache bconfig))
+    F.forM_ mcache $ \cache -> do
+        icache <- configInstalledCache
+        saveInstalledCache icache cache
 
     -- Add in the executables that are installed, making sure to only trust a
     -- listed installation under the right circumstances (see below)
@@ -123,16 +126,16 @@ getInstalled menv opts sourceMap = do
 -- The goal is to ascertain that the dependencies for a package are present,
 -- that it has profiling if necessary, and that it matches the version and
 -- location needed by the SourceMap
-loadDatabase :: (M env m, PackageInstallInfo pii)
+loadDatabase :: (StackM env m, HasEnvConfig env, PackageInstallInfo pii)
              => EnvOverride
              -> GetInstalledOpts
              -> Maybe InstalledCache -- ^ if Just, profiling or haddock is required
              -> Map PackageName pii -- ^ to determine which installed things we should include
              -> Maybe (InstalledPackageLocation, Path Abs Dir) -- ^ package database, Nothing for global
              -> [LoadHelper] -- ^ from parent databases
-             -> m ([LoadHelper], [DumpPackage () ()])
+             -> m ([LoadHelper], [DumpPackage () () ()])
 loadDatabase menv opts mcache sourceMap mdb lhs0 = do
-    wc <- getWhichCompiler
+    wc <- view $ actualCompilerVersionL.to whichCompiler
     (lhs1', dps) <- ghcPkgDump menv wc (fmap snd (maybeToList mdb))
                 $ conduitDumpPackage =$ sink
     let ghcjsHack = wc == Ghcjs && isNothing mdb
@@ -157,11 +160,18 @@ loadDatabase menv opts mcache sourceMap mdb lhs0 = do
             -- Just an optimization to avoid calculating the haddock
             -- values when they aren't necessary
             _ -> CL.map (\dp -> dp { dpHaddock = False })
+    conduitSymbolsCache =
+        case mcache of
+            Just cache | getInstalledSymbols opts -> addSymbols cache
+            -- Just an optimization to avoid calculating the debugging
+            -- symbol values when they aren't necessary
+            _ -> CL.map (\dp -> dp { dpSymbols = False })
     mloc = fmap fst mdb
     sinkDP = conduitProfilingCache
-          =$ conduitHaddockCache
-          =$ CL.map (\dp -> (isAllowed opts mcache sourceMap mloc dp, toLoadHelper mloc dp))
-          =$ CL.consume
+           =$ conduitHaddockCache
+           =$ conduitSymbolsCache
+           =$ CL.map (isAllowed opts mcache sourceMap mloc &&& toLoadHelper mloc)
+           =$ CL.consume
     sink = getZipSink $ (,)
         <$> ZipSink sinkDP
         <*> ZipSink CL.consume
@@ -196,6 +206,7 @@ processLoadResult mdb _ (reason, lh) = do
             Allowed -> " the impossible?!?!"
             NeedsProfiling -> " it needing profiling."
             NeedsHaddock -> " it needing haddocks."
+            NeedsSymbols -> " it needing debugging symbols."
             UnknownPkg -> " it being unknown to the resolver / extra-deps."
             WrongLocation mloc loc -> " wrong location: " <> T.pack (show (mloc, loc))
             WrongVersion actual wanted -> T.concat
@@ -211,6 +222,7 @@ data Allowed
     = Allowed
     | NeedsProfiling
     | NeedsHaddock
+    | NeedsSymbols
     | UnknownPkg
     | WrongLocation (Maybe InstalledPackageLocation) InstallLocation
     | WrongVersion Version Version
@@ -224,13 +236,15 @@ isAllowed :: PackageInstallInfo pii
           -> Maybe InstalledCache
           -> Map PackageName pii
           -> Maybe InstalledPackageLocation
-          -> DumpPackage Bool Bool
+          -> DumpPackage Bool Bool Bool
           -> Allowed
 isAllowed opts mcache sourceMap mloc dp
     -- Check that it can do profiling if necessary
     | getInstalledProfiling opts && isJust mcache && not (dpProfiling dp) = NeedsProfiling
     -- Check that it has haddocks if necessary
     | getInstalledHaddock opts && isJust mcache && not (dpHaddock dp) = NeedsHaddock
+    -- Check that it has haddocks if necessary
+    | getInstalledSymbols opts && isJust mcache && not (dpSymbols dp) = NeedsSymbols
     | otherwise =
         case Map.lookup name sourceMap of
             Nothing ->
@@ -261,7 +275,7 @@ data LoadHelper = LoadHelper
     }
     deriving Show
 
-toLoadHelper :: Maybe InstalledPackageLocation -> DumpPackage Bool Bool -> LoadHelper
+toLoadHelper :: Maybe InstalledPackageLocation -> DumpPackage Bool Bool Bool -> LoadHelper
 toLoadHelper mloc dp = LoadHelper
     { lhId = gid
     , lhDeps =
