@@ -6,15 +6,18 @@
 {-# LANGUAGE TemplateHaskell       #-}
 {-# LANGUAGE TupleSections         #-}
 {-# LANGUAGE ScopedTypeVariables   #-}
+{-# LANGUAGE ConstraintKinds #-}
 -- Load information on package sources
 module Stack.Build.Source
     ( loadSourceMap
+    , loadSourceMapFull
     , SourceMap
     , PackageSource (..)
     , getLocalFlags
     , getGhcOptions
     , getLocalPackageViews
     , parseTargetsFromBuildOpts
+    , parseTargetsFromBuildOptsWith
     , addUnlistedToBuildCache
     , getDefaultPackageConfig
     , getPackageConfig
@@ -24,15 +27,14 @@ import              Control.Applicative
 import              Control.Arrow ((&&&))
 import              Control.Exception (assert, catch)
 import              Control.Monad hiding (sequence)
-import              Control.Monad.Catch (MonadMask)
 import              Control.Monad.IO.Class
 import              Control.Monad.Logger
-import              Control.Monad.Reader (MonadReader, asks)
+import              Control.Monad.Reader (MonadReader)
 import              Control.Monad.Trans.Resource
-import "cryptohash" Crypto.Hash (Digest, SHA256)
+import              Crypto.Hash (Digest, SHA256(..))
 import              Crypto.Hash.Conduit (sinkHash)
+import qualified    Data.ByteArray as Mem (convert)
 import qualified    Data.ByteString as S
-import              Data.Byteable (toBytes)
 import              Data.Conduit (($$), ZipSink (..))
 import qualified    Data.Conduit.Binary as CB
 import qualified    Data.Conduit.List as CL
@@ -53,35 +55,68 @@ import              Data.Traversable (sequence)
 import              Distribution.Package (pkgName, pkgVersion)
 import              Distribution.PackageDescription (GenericPackageDescription, package, packageDescription)
 import qualified    Distribution.PackageDescription as C
-import              Network.HTTP.Client.Conduit (HasHttpManager)
 import              Path
 import              Path.IO
 import              Prelude hiding (sequence)
 import              Stack.Build.Cache
 import              Stack.Build.Target
 import              Stack.BuildPlan (shadowMiniBuildPlan)
+import              Stack.Config (getLocalPackages)
 import              Stack.Constants (wiredInPackages)
 import              Stack.Package
 import              Stack.PackageIndex (getPackageVersions)
-import              Stack.Types
+import              Stack.Types.Build
+import              Stack.Types.BuildPlan
+import              Stack.Types.Config
+import              Stack.Types.FlagName
+import              Stack.Types.Package
+import              Stack.Types.PackageName
+import              Stack.Types.Resolver
+import              Stack.Types.StackT
+import              Stack.Types.Version
 import qualified    System.Directory as D
 import              System.FilePath (takeFileName)
 import              System.IO (withBinaryFile, IOMode (ReadMode))
 import              System.IO.Error (isDoesNotExistError)
 
-loadSourceMap :: (MonadIO m, MonadMask m, MonadReader env m, MonadBaseControl IO m, HasHttpManager env, MonadLogger m, HasEnvConfig env)
+-- | Like 'loadSourceMapFull', but doesn't return values that aren't as
+-- commonly needed.
+loadSourceMap :: (StackM env m, HasEnvConfig env)
               => NeedTargets
               -> BuildOptsCLI
-              -> m ( Map PackageName SimpleTarget
-                   , MiniBuildPlan
-                   , [LocalPackage]
-                   , Set PackageName -- non-local targets
+              -> m ( [LocalPackage]
                    , SourceMap
                    )
 loadSourceMap needTargets boptsCli = do
-    bconfig <- asks getBuildConfig
+    (_, _, locals, _, _, sourceMap) <- loadSourceMapFull needTargets boptsCli
+    return (locals, sourceMap)
+
+-- | Given the build commandline options, does the following:
+--
+-- * Parses the build targets.
+--
+-- * Loads the 'MiniBuildPlan' from the resolver, with extra-deps
+--   shadowing any packages that should be built locally.
+--
+-- * Loads up the 'LocalPackage' info.
+--
+-- * Builds a 'SourceMap', which contains info for all the packages that
+--   will be involved in the build.
+loadSourceMapFull :: (StackM env m, HasEnvConfig env)
+                  => NeedTargets
+                  -> BuildOptsCLI
+                  -> m ( Map PackageName SimpleTarget
+                       , MiniBuildPlan
+                       , [LocalPackage]
+                       , Set PackageName -- non-local targets
+                       , Map PackageName Version -- extra-deps from configuration and cli
+                       , SourceMap
+                       )
+loadSourceMapFull needTargets boptsCli = do
+    bconfig <- view buildConfigL
     rawLocals <- getLocalPackageViews
-    (mbp0, cliExtraDeps, targets) <- parseTargetsFromBuildOpts needTargets boptsCli
+    (mbp0, cliExtraDeps, targets) <- parseTargetsFromBuildOptsWith rawLocals needTargets boptsCli
+
     -- Extend extra-deps to encompass targets requested on the command line
     -- that are not in the snapshot.
     extraDeps0 <- extendExtraDeps
@@ -107,10 +142,14 @@ loadSourceMap needTargets boptsCli = do
             isLocal STNonLocal = False
 
         shadowed = Map.keysSet rawLocals <> Map.keysSet extraDeps0
+
+        -- Ignores all packages in the MiniBuildPlan that depend on any
+        -- local packages or extra-deps. All packages that have
+        -- transitive dependenceis on these packages are treated as
+        -- extra-deps (extraDeps1).
         (mbp, extraDeps1) = shadowMiniBuildPlan mbp0 shadowed
 
-        -- Add the extra deps from the stack.yaml file to the deps grabbed from
-        -- the snapshot
+        -- Combine the extra-deps with the ones implicitly shadowed.
         extraDeps2 = Map.union
             (Map.map (\v -> (v, Map.empty, [])) extraDeps0)
             (Map.map (\mpi -> (mpiVersion mpi, mpiFlags mpi, mpiGhcOptions mpi)) extraDeps1)
@@ -142,18 +181,22 @@ loadSourceMap needTargets boptsCli = do
                 in PSUpstream v Local flags ghcOptions Nothing)
             extraDeps2
 
+    -- Combine the local packages, extra-deps, and MiniBuildPlan into
+    -- one unified source map.
     let sourceMap = Map.unions
             [ Map.fromList $ flip map locals $ \lp ->
                 let p = lpPackage lp
                  in (packageName p, PSLocal lp)
             , extraDeps3
-            , flip fmap (mbpPackages mbp) $ \mpi ->
-                PSUpstream (mpiVersion mpi) Snap (mpiFlags mpi) (mpiGhcOptions mpi) (mpiGitSHA1 mpi)
-            ] `Map.difference` Map.fromList (map (, ()) (HashSet.toList wiredInPackages))
+            , flip Map.mapWithKey (mbpPackages mbp) $ \n mpi ->
+                let configOpts = getGhcOptions bconfig boptsCli n False False
+                 in PSUpstream (mpiVersion mpi) Snap (mpiFlags mpi) (mpiGhcOptions mpi ++ configOpts) (mpiGitSHA1 mpi)
+            ]
+            `Map.difference` Map.fromList (map (, ()) (HashSet.toList wiredInPackages))
 
-    return (targets, mbp, locals, nonLocalTargets, sourceMap)
+    return (targets, mbp, locals, nonLocalTargets, extraDeps0, sourceMap)
 
--- | All flags for a local package
+-- | All flags for a local package.
 getLocalFlags
     :: BuildConfig
     -> BuildOptsCLI
@@ -167,20 +210,25 @@ getLocalFlags bconfig boptsCli name = Map.unions
   where
     cliFlags = boptsCLIFlags boptsCli
 
+-- | Get the configured options to pass from GHC, based on the build
+-- configuration and commandline.
 getGhcOptions :: BuildConfig -> BuildOptsCLI -> PackageName -> Bool -> Bool -> [Text]
 getGhcOptions bconfig boptsCli name isTarget isLocal = concat
     [ ghcOptionsFor name (configGhcOptions config)
     , concat [["-fhpc"] | isLocal && toCoverage (boptsTestOpts bopts)]
-    , if (boptsLibProfile bopts || boptsExeProfile bopts)
+    , if boptsLibProfile bopts || boptsExeProfile bopts
          then ["-auto-all","-caf-all"]
          else []
+    , if not $ boptsLibStrip bopts || boptsExeStrip bopts
+         then ["-g"]
+         else []
     , if includeExtraOptions
          then boptsCLIGhcOptions boptsCli
          else []
     ]
   where
     bopts = configBuild config
-    config = bcConfig bconfig
+    config = view configL bconfig
     includeExtraOptions =
         case configApplyGhcOptions config of
             AGOTargets -> isTarget
@@ -188,26 +236,46 @@ getGhcOptions bconfig boptsCli name isTarget isLocal = concat
             AGOEverything -> True
 
 -- | Use the build options and environment to parse targets.
+--
+-- If the local packages views are already known, use 'parseTargetsFromBuildOptsWith'
+-- instead.
+--
+-- Along with the 'Map' of targets, this yields the loaded
+-- 'MiniBuildPlan' for the resolver, as well as a Map of extra-deps
+-- derived from the commandline. These extra-deps targets come from when
+-- the user specifies a particular package version on the commonadline,
+-- or when a flag is specified for a snapshot package.
 parseTargetsFromBuildOpts
-    :: (MonadIO m, MonadMask m, MonadReader env m, MonadLogger m, HasEnvConfig env)
+    :: (StackM env m, HasEnvConfig env)
     => NeedTargets
     -> BuildOptsCLI
     -> m (MiniBuildPlan, M.Map PackageName Version, M.Map PackageName SimpleTarget)
 parseTargetsFromBuildOpts needTargets boptscli = do
-    bconfig <- asks getBuildConfig
+    rawLocals <- getLocalPackageViews
+    parseTargetsFromBuildOptsWith rawLocals needTargets boptscli
+
+parseTargetsFromBuildOptsWith
+    :: (StackM env m, HasEnvConfig env)
+    => Map PackageName (LocalPackageView, GenericPackageDescription)
+       -- ^ Local package views
+    -> NeedTargets
+    -> BuildOptsCLI
+    -> m (MiniBuildPlan, M.Map PackageName Version, M.Map PackageName SimpleTarget)
+parseTargetsFromBuildOptsWith rawLocals needTargets boptscli = do
+    $logDebug "Parsing the targets"
+    bconfig <- view buildConfigL
     mbp0 <-
         case bcResolver bconfig of
             ResolverCompiler _ -> do
                 -- We ignore the resolver version, as it might be
                 -- GhcMajorVersion, and we want the exact version
                 -- we're using.
-                version <- asks (envConfigCompilerVersion . getEnvConfig)
+                version <- view actualCompilerVersionL
                 return MiniBuildPlan
                     { mbpCompilerVersion = version
                     , mbpPackages = Map.empty
                     }
             _ -> return (bcWantedMiniBuildPlan bconfig)
-    rawLocals <- getLocalPackageViews
     workingDir <- getCurrentDir
 
     let snapshot = mpiVersion <$> mbpPackages mbp0
@@ -254,11 +322,12 @@ convertSnapshotToExtra snapshot extra0 locals = go Map.empty
                 go (Map.insert flag version extra) flags
 
 -- | Parse out the local package views for the current project
-getLocalPackageViews :: (MonadThrow m, MonadIO m, MonadReader env m, HasEnvConfig env, MonadLogger m)
+getLocalPackageViews :: (StackM env m, HasEnvConfig env)
                      => m (Map PackageName (LocalPackageView, GenericPackageDescription))
 getLocalPackageViews = do
-    econfig <- asks getEnvConfig
-    locals <- forM (Map.toList $ envConfigPackages econfig) $ \(dir, treatLikeExtraDep) -> do
+    $logDebug "Parsing the cabal files of the local packages"
+    packages <- getLocalPackages
+    locals <- forM (Map.toList packages) $ \(dir, treatLikeExtraDep) -> do
         cabalfp <- findOrGenerateCabalFile dir
         (warnings,gpkg) <- readPackageUnresolved cabalfp
         mapM_ (printCabalFileWarning cabalfp) warnings
@@ -311,8 +380,7 @@ splitComponents =
 -- | Upgrade the initial local package info to a full-blown @LocalPackage@
 -- based on the selected components
 loadLocalPackage
-    :: forall m env.
-       (MonadReader env m, HasEnvConfig env, MonadMask m, MonadLogger m, MonadIO m)
+    :: forall m env. (StackM env m, HasEnvConfig env)
     => BuildOptsCLI
     -> Map PackageName SimpleTarget
     -> (PackageName, (LocalPackageView, GenericPackageDescription))
@@ -320,18 +388,16 @@ loadLocalPackage
 loadLocalPackage boptsCli targets (name, (lpv, gpkg)) = do
     let mtarget = Map.lookup name targets
     config  <- getPackageConfig boptsCli name (isJust mtarget) True
-    bopts <- asks (configBuild . getConfig)
-    let pkg = resolvePackage config gpkg
-
-        (exes, tests, benches) =
+    bopts <- view buildOptsL
+    let (exes, tests, benches) =
             case mtarget of
                 Just (STLocalComps comps) -> splitComponents $ Set.toList comps
                 Just STLocalAll ->
                     ( packageExes pkg
-                    , if boptsTests bopts
+                    , if boptsTests bopts && not (lpvExtraDep lpv)
                         then Map.keysSet (packageTests pkg)
                         else Set.empty
-                    , if boptsBenchmarks bopts
+                    , if boptsBenchmarks bopts && not (lpvExtraDep lpv)
                         then packageBenchmarks pkg
                         else Set.empty
                     )
@@ -358,11 +424,28 @@ loadLocalPackage boptsCli targets (name, (lpv, gpkg)) = do
             , packageConfigEnableBenchmarks = True
             }
 
+        -- We resolve the package in 4 different configurations:
+        --
+        -- - pkg doesn't have tests or benchmarks enabled.
+        --
+        -- - btpkg has them enabled if they are present.
+        --
+        -- - testpkg has tests enabled, but not benchmarks.
+        --
+        -- - benchpkg has benchmarks enablde, but not tests.
+        --
+        -- The latter two configurations are used to compute the deps
+        -- when --enable-benchmarks or --enable-tests are configured.
+        -- This allows us to do an optimization where these are passed
+        -- if the deps are present. This can avoid doing later
+        -- unnecessary reconfigures.
+        pkg = resolvePackage config gpkg
         btpkg
             | Set.null tests && Set.null benches = Nothing
             | otherwise = Just (resolvePackage btconfig gpkg)
         testpkg = resolvePackage testconfig gpkg
         benchpkg = resolvePackage benchconfig gpkg
+
     mbuildCache <- tryGetBuildCache $ lpvRoot lpv
     (files,_) <- getPackageFilesSimple pkg (lpvCabalFP lpv)
 
@@ -409,7 +492,7 @@ checkFlagsUsed :: (MonadThrow m, MonadReader env m, HasBuildConfig env)
                -> Map PackageName snapshot -- ^ snapshot, for error messages
                -> m ()
 checkFlagsUsed boptsCli lps extraDeps snapshot = do
-    bconfig <- asks getBuildConfig
+    bconfig <- view buildConfigL
 
         -- Check if flags specified in stack.yaml and the command line are
         -- used, see https://github.com/commercialhaskell/stack/issues/617
@@ -451,7 +534,7 @@ checkFlagsUsed boptsCli lps extraDeps snapshot = do
 -- this was then superseded by
 -- https://github.com/commercialhaskell/stack/issues/651
 extendExtraDeps
-    :: (HasBuildConfig env, MonadIO m, MonadLogger m, MonadReader env m, HasHttpManager env, MonadBaseControl IO m, MonadMask m)
+    :: (StackM env m, HasBuildConfig env)
     => Map PackageName Version -- ^ original extra deps
     -> Map PackageName Version -- ^ package identifiers from the command line
     -> Set PackageName -- ^ all packages added on the command line
@@ -461,7 +544,7 @@ extendExtraDeps extraDeps0 cliExtraDeps unknowns = do
     case errs of
         [] -> return $ Map.unions $ extraDeps1 : unknowns'
         _ -> do
-            bconfig <- asks getBuildConfig
+            bconfig <- view buildConfigL
             throwM $ UnknownTargets
                 (Set.fromList errs)
                 Map.empty -- TODO check the cliExtraDeps for presence in index
@@ -520,7 +603,7 @@ checkBuildCache oldCache files = do
 
 -- | Returns entries to add to the build cache for any newly found unlisted modules
 addUnlistedToBuildCache
-    :: (MonadIO m, MonadReader env m, MonadMask m, MonadLogger m, HasEnvConfig env)
+    :: (StackM env m, HasEnvConfig env)
     => ModTime
     -> Package
     -> Path Abs File
@@ -547,7 +630,7 @@ addUnlistedToBuildCache preBuildTime pkg cabalFP buildCache = do
 
 -- | Gets list of Paths for files in a package
 getPackageFilesSimple
-    :: (MonadIO m, MonadReader env m, MonadMask m, MonadLogger m, HasEnvConfig env)
+    :: (StackM env m, HasEnvConfig env)
     => Package -> Path Abs File -> m (Set (Path Abs File), [PackageWarning])
 getPackageFilesSimple pkg cabalFP = do
     (_,compFiles,cabalFiles,warnings) <-
@@ -582,7 +665,7 @@ calcFci modTime' fp = liftIO $
         return FileCacheInfo
             { fciModTime = modTime'
             , fciSize = size
-            , fciHash = toBytes (digest :: Digest SHA256)
+            , fciHash = Mem.convert (digest :: Digest SHA256)
             }
 
 checkComponentsBuildable :: MonadThrow m => [LocalPackage] -> m ()
@@ -598,15 +681,15 @@ checkComponentsBuildable lps =
 getDefaultPackageConfig :: (MonadIO m, MonadReader env m, HasEnvConfig env)
   => m PackageConfig
 getDefaultPackageConfig = do
-  econfig <- asks getEnvConfig
-  bconfig <- asks getBuildConfig
+  platform <- view platformL
+  compilerVersion <- view actualCompilerVersionL
   return PackageConfig
     { packageConfigEnableTests = False
     , packageConfigEnableBenchmarks = False
     , packageConfigFlags = M.empty
     , packageConfigGhcOptions = []
-    , packageConfigCompilerVersion = envConfigCompilerVersion econfig
-    , packageConfigPlatform = configPlatform $ getConfig bconfig
+    , packageConfigCompilerVersion = compilerVersion
+    , packageConfigPlatform = platform
     }
 
 -- | Get 'PackageConfig' for package given its name.
@@ -617,13 +700,14 @@ getPackageConfig :: (MonadIO m, MonadReader env m, HasEnvConfig env)
   -> Bool
   -> m PackageConfig
 getPackageConfig boptsCli name isTarget isLocal = do
-  econfig <- asks getEnvConfig
-  bconfig <- asks getBuildConfig
+  bconfig <- view buildConfigL
+  platform <- view platformL
+  compilerVersion <- view actualCompilerVersionL
   return PackageConfig
     { packageConfigEnableTests = False
     , packageConfigEnableBenchmarks = False
     , packageConfigFlags = getLocalFlags bconfig boptsCli name
     , packageConfigGhcOptions = getGhcOptions bconfig boptsCli name isTarget isLocal
-    , packageConfigCompilerVersion = envConfigCompilerVersion econfig
-    , packageConfigPlatform = configPlatform $ getConfig bconfig
+    , packageConfigCompilerVersion = compilerVersion
+    , packageConfigPlatform = platform
     }
