@@ -1,3 +1,4 @@
+{-# LANGUAGE ConstraintKinds    #-}
 {-# LANGUAGE DeriveDataTypeable #-}
 {-# LANGUAGE DeriveGeneric      #-}
 {-# LANGUAGE EmptyDataDecls     #-}
@@ -31,6 +32,7 @@ module Stack.BuildPlan
     , showItems
     , showPackageFlags
     , parseCustomMiniBuildPlan
+    , loadBuildPlan
     ) where
 
 import           Control.Applicative
@@ -39,13 +41,13 @@ import           Control.Monad (liftM, forM, unless)
 import           Control.Monad.Catch
 import           Control.Monad.IO.Class
 import           Control.Monad.Logger
-import           Control.Monad.Reader (asks)
+import           Control.Monad.Reader (MonadReader)
 import           Control.Monad.State.Strict      (State, execState, get, modify,
                                                   put)
-import           Control.Monad.Trans.Control (MonadBaseControl)
-import qualified Crypto.Hash.SHA256 as SHA256
+import           Crypto.Hash (hashWith, SHA256(..))
 import           Data.Aeson.Extended (WithJSONWarnings(..), logJSONWarnings)
-import           Data.Binary.VersionTagged (taggedDecodeOrLoad, decodeFileOrFailDeep, taggedEncodeFile)
+import           Data.Store.VersionTagged
+import qualified Data.ByteArray as Mem (convert)
 import qualified Data.ByteString as S
 import qualified Data.ByteString.Base64.URL as B64URL
 import qualified Data.ByteString.Char8 as S8
@@ -76,9 +78,7 @@ import qualified Distribution.PackageDescription as C
 import           Distribution.System (Platform)
 import           Distribution.Text (display)
 import qualified Distribution.Version as C
-import           Network.HTTP.Client (checkStatus)
 import           Network.HTTP.Download
-import           Network.HTTP.Types (Status(..))
 import           Path
 import           Path.IO
 import           Prelude -- Fix AMP warning
@@ -86,7 +86,16 @@ import           Stack.Constants
 import           Stack.Fetch
 import           Stack.Package
 import           Stack.PackageIndex
-import           Stack.Types
+import           Stack.Types.BuildPlan
+import           Stack.Types.FlagName
+import           Stack.Types.PackageIdentifier
+import           Stack.Types.PackageIndex
+import           Stack.Types.PackageName
+import           Stack.Types.Version
+import           Stack.Types.Config
+import           Stack.Types.Urls
+import           Stack.Types.Compiler
+import           Stack.Types.Resolver
 import           Stack.Types.StackT
 
 data BuildPlanException
@@ -159,10 +168,7 @@ instance Show BuildPlanException where
                 , ["Note: further dependencies may need to be added"]
                 ]
           where
-            go (dep, users) | Set.null users = concat
-                [ packageNameString dep
-                , " (internal stack error: this should never be null)"
-                ]
+            go (dep, users) | Set.null users = packageNameString dep ++ " (internal stack error: this should never be null)"
             go (dep, users) = concat
                 [ packageNameString dep
                 , " (used by "
@@ -192,18 +198,19 @@ instance Show BuildPlanException where
 -- This function will not provide test suite and benchmark dependencies.
 --
 -- This may fail if a target package is not present in the @BuildPlan@.
-resolveBuildPlan :: (MonadThrow m, MonadIO m, MonadReader env m, HasBuildConfig env, MonadLogger m, HasHttpManager env, MonadBaseControl IO m,MonadCatch m)
-                 => MiniBuildPlan
-                 -> (PackageName -> Bool) -- ^ is it shadowed by a local package?
-                 -> Map PackageName (Set PackageName) -- ^ required packages, and users of it
-                 -> m ( Map PackageName (Version, Map FlagName Bool)
-                      , Map PackageName (Set PackageName)
-                      )
+resolveBuildPlan
+    :: (StackMiniM env m, HasBuildConfig env)
+    => MiniBuildPlan
+    -> (PackageName -> Bool) -- ^ is it shadowed by a local package?
+    -> Map PackageName (Set PackageName) -- ^ required packages, and users of it
+    -> m ( Map PackageName (Version, Map FlagName Bool)
+         , Map PackageName (Set PackageName)
+         )
 resolveBuildPlan mbp isShadowed packages
     | Map.null (rsUnknown rs) && Map.null (rsShadowed rs) = return (rsToInstall rs, rsUsedBy rs)
     | otherwise = do
-        bconfig <- asks getBuildConfig
-        caches <- getPackageCaches
+        bconfig <- view buildConfigL
+        (caches, _gitShaCaches) <- getPackageCaches
         let maxVer =
                 Map.fromListWith max $
                 map toTuple $
@@ -225,11 +232,12 @@ data ResolveState = ResolveState
     , rsUsedBy    :: Map PackageName (Set PackageName)
     }
 
-toMiniBuildPlan :: (MonadIO m, MonadLogger m, MonadReader env m, HasHttpManager env, MonadMask m, HasConfig env, MonadBaseControl IO m)
-                => CompilerVersion -- ^ Compiler version
-                -> Map PackageName Version -- ^ cores
-                -> Map PackageName (Version, Map FlagName Bool, [Text], Maybe GitSHA1) -- ^ non-core packages
-                -> m MiniBuildPlan
+toMiniBuildPlan
+    :: (StackMiniM env m, HasConfig env)
+    => CompilerVersion -- ^ Compiler version
+    -> Map PackageName Version -- ^ cores
+    -> Map PackageName (Version, Map FlagName Bool, [Text], Maybe GitSHA1) -- ^ non-core packages
+    -> m MiniBuildPlan
 toMiniBuildPlan compilerVersion corePackages packages = do
     -- Determine the dependencies of all of the packages in the build plan. We
     -- handle core packages specially, because some of them will not be in the
@@ -266,39 +274,39 @@ toMiniBuildPlan compilerVersion corePackages packages = do
         }
 
 -- | Add in the resolved dependencies from the package index
-addDeps :: (MonadIO m, MonadLogger m, MonadReader env m, HasHttpManager env, MonadMask m, HasConfig env, MonadBaseControl IO m)
-        => Bool -- ^ allow missing
-        -> CompilerVersion -- ^ Compiler version
-        -> Map PackageName (Version, Map FlagName Bool, [Text], Maybe GitSHA1)
-        -> m (Map PackageName MiniPackageInfo, Set PackageIdentifier)
+addDeps
+    :: (StackMiniM env m, HasConfig env)
+    => Bool -- ^ allow missing
+    -> CompilerVersion -- ^ Compiler version
+    -> Map PackageName (Version, Map FlagName Bool, [Text], Maybe GitSHA1)
+    -> m (Map PackageName MiniPackageInfo, Set PackageIdentifier)
 addDeps allowMissing compilerVersion toCalc = do
-    menv <- getMinimalEnvOverride
-    platform <- asks $ configPlatform . getConfig
+    platform <- view platformL
     (resolvedMap, missingIdents) <-
         if allowMissing
             then do
                 (missingNames, missingIdents, m) <-
-                    resolvePackagesAllowMissing shaMap Set.empty
+                    resolvePackagesAllowMissing Nothing shaMap Set.empty
                 assert (Set.null missingNames)
                     $ return (m, missingIdents)
             else do
-                m <- resolvePackages menv shaMap Set.empty
+                m <- resolvePackages Nothing shaMap Set.empty
                 return (m, Set.empty)
-    let byIndex = Map.fromListWith (++) $ flip map (Map.toList resolvedMap)
-            $ \(ident, rp) ->
+    let byIndex = Map.fromListWith (++) $ flip map resolvedMap
+            $ \rp ->
                 let (cache, ghcOptions, sha) =
-                        case Map.lookup (packageIdentifierName ident) toCalc of
+                        case Map.lookup (packageIdentifierName (rpIdent rp)) toCalc of
                             Nothing -> (Map.empty, [], Nothing)
                             Just (_, x, y, z) -> (x, y, z)
-                 in (indexName $ rpIndex rp,
-                    [( ident
-                    , rpCache rp
-                    , sha
-                    , (cache, ghcOptions, sha)
-                    )])
-    res <- forM (Map.toList byIndex) $ \(indexName', pkgs) -> withCabalFiles indexName' pkgs
-        $ \ident (flags, ghcOptions, mgitSha) cabalBS -> do
-            (_warnings,gpd) <- readPackageUnresolvedBS Nothing cabalBS
+                 in (indexName $ rpIndex rp, [(rp, (cache, ghcOptions, sha))])
+    res <- forM (Map.toList byIndex) $ \(indexName', pkgs) ->
+      fmap Map.unions $ withCabalFiles indexName' pkgs
+        $ \ident (flags, ghcOptions, mgitSha) cabalBS ->
+        case readPackageUnresolvedBS (Right ident) cabalBS of
+          Left e
+            | allowedToSkip ident -> return Map.empty
+            | otherwise -> throwM e
+          Right (_warnings, gpd) -> do
             let packageConfig = PackageConfig
                     { packageConfigEnableTests = False
                     , packageConfigEnableBenchmarks = False
@@ -311,7 +319,7 @@ addDeps allowMissing compilerVersion toCalc = do
                 pd = resolvePackageDescription packageConfig gpd
                 exes = Set.fromList $ map (ExeName . T.pack . exeName) $ executables pd
                 notMe = Set.filter (/= name) . Map.keysSet
-            return (name, MiniPackageInfo
+            return $ Map.singleton name MiniPackageInfo
                 { mpiVersion = packageIdentifierVersion ident
                 , mpiFlags = flags
                 , mpiGhcOptions = ghcOptions
@@ -323,13 +331,31 @@ addDeps allowMissing compilerVersion toCalc = do
                     (buildable . libBuildInfo)
                     (library pd)
                 , mpiGitSHA1 = mgitSha
-                })
-    return (Map.fromList $ concat res, missingIdents)
+                }
+    return (Map.unions res, missingIdents)
   where
     shaMap = Map.fromList
         $ map (\(n, (v, _f, _ghcOptions, gitsha)) -> (PackageIdentifier n v, gitsha))
         $ Map.toList toCalc
 
+    -- Michael Snoyman, 2017-07-31:
+    --
+    -- This is a stop-gap measure to address a specific concern around
+    -- the GHC 8.2.1 release. The current Stack version (1.5.0) will
+    -- eagerly parse all cabal files mentioned in a snapshot,
+    -- including global packages. Additionally, for the first time
+    -- (AFAICT), GHC 8.2.1 is providing a package on Hackage with a
+    -- ghc.cabal file, which requires the (not yet supported) Cabal
+    -- 2.0 file format. To work around this, we're adding a special
+    -- dispensation to ignore parse failures for this package.
+    --
+    -- Master already does better by simply ignoring global
+    -- information and looking things up in the database. We may want
+    -- to consider going a step further and simply ignoring _all_
+    -- parse failures, or turning them into warnings, though I haven't
+    -- considered the repercussions of that.
+    allowedToSkip (PackageIdentifier name _) = name == $(mkPackageName "ghc")
+
 -- | Resolve all packages necessary to install for the needed packages.
 getDeps :: MiniBuildPlan
         -> (PackageName -> Bool) -- ^ is it shadowed by a local package?
@@ -417,7 +443,7 @@ getToolMap mbp =
       $ mpiExes mpi
 
 loadResolver
-    :: (MonadIO m, MonadLogger m, MonadReader env m, HasHttpManager env, HasConfig env, HasGHCVariant env, MonadBaseControl IO m, MonadMask m)
+    :: (StackMiniM env m, HasConfig env, HasGHCVariant env)
     => Maybe (Path Abs File)
     -> Resolver
     -> m (MiniBuildPlan, LoadedResolver)
@@ -441,17 +467,16 @@ loadResolver mconfigPath resolver =
 
 -- | Load up a 'MiniBuildPlan', preferably from cache
 loadMiniBuildPlan
-    :: (MonadIO m, MonadLogger m, MonadReader env m, HasHttpManager env, HasConfig env, HasGHCVariant env, MonadBaseControl IO m, MonadMask m)
-    => SnapName
-    -> m MiniBuildPlan
+    :: (StackMiniM env m, HasConfig env, HasGHCVariant env)
+    => SnapName -> m MiniBuildPlan
 loadMiniBuildPlan name = do
     path <- configMiniBuildPlanCache name
-    taggedDecodeOrLoad path $ liftM buildPlanFixes $ do
+    $(versionedDecodeOrLoad miniBuildPlanVC) path $ liftM buildPlanFixes $ do
         bp <- loadBuildPlan name
         toMiniBuildPlan
             (siCompilerVersion $ bpSystemInfo bp)
             (siCorePackages $ bpSystemInfo bp)
-            (fmap goPP $ bpPackages bp)
+            (goPP <$> bpPackages bp)
   where
     goPP pp =
         ( ppVersion pp
@@ -482,12 +507,9 @@ buildPlanFixes mbp = mbp
 
 -- | Load the 'BuildPlan' for the given snapshot. Will load from a local copy
 -- if available, otherwise downloading from Github.
-loadBuildPlan :: (MonadIO m, MonadThrow m, MonadLogger m, MonadReader env m, HasHttpManager env, HasConfig env)
-              => SnapName
-              -> m BuildPlan
+loadBuildPlan :: (StackMiniM env m, HasConfig env) => SnapName -> m BuildPlan
 loadBuildPlan name = do
-    env <- ask
-    let stackage = getStackRoot env
+    stackage <- view stackRootL
     file' <- parseRelFile $ T.unpack file
     let fp = buildPlanDir stackage </> file'
     $logDebug $ "Decoding build plan from: " <> T.pack (toFilePath fp)
@@ -498,21 +520,19 @@ loadBuildPlan name = do
             $logDebug $ "Decoding build plan from file failed: " <> T.pack (show e)
             ensureDir (parent fp)
             url <- buildBuildPlanUrl name file
-            req <- parseUrl $ T.unpack url
+            req <- parseRequest $ T.unpack url
             $logSticky $ "Downloading " <> renderSnapName name <> " build plan ..."
             $logDebug $ "Downloading build plan from: " <> url
-            _ <- redownload req { checkStatus = handle404 } fp
+            _ <- redownload req fp
             $logStickyDone $ "Downloaded " <> renderSnapName name <> " build plan."
             liftIO (decodeFileEither $ toFilePath fp) >>= either throwM return
 
   where
     file = renderSnapName name <> ".yaml"
-    handle404 (Status 404 _) _ _ = Just $ SomeException $ SnapshotNotFound name
-    handle404 _ _ _              = Nothing
 
 buildBuildPlanUrl :: (MonadReader env m, HasConfig env) => SnapName -> Text -> m Text
 buildBuildPlanUrl name file = do
-    urls <- asks (configUrls . getConfig)
+    urls <- view $ configL.to configUrls
     return $
         case name of
              LTS _ _ -> urlsLtsBuildPlans urls <> "/" <> file
@@ -634,10 +654,11 @@ checkPackageBuildPlan platform compiler pool flags gpd =
 -- | Checks if the given package dependencies can be satisfied by the given set
 -- of packages. Will fail if a package is either missing or has a version
 -- outside of the version range.
-checkPackageDeps :: PackageName -- ^ package using dependencies, for constructing DepErrors
-          -> Map PackageName VersionRange -- ^ dependency constraints
-          -> Map PackageName Version -- ^ Available package pool or index
-          -> DepErrors
+checkPackageDeps
+    :: PackageName -- ^ package using dependencies, for constructing DepErrors
+    -> Map PackageName VersionRange -- ^ dependency constraints
+    -> Map PackageName Version -- ^ Available package pool or index
+    -> DepErrors
 checkPackageDeps myName deps packages =
     Map.unionsWith combineDepError $ map go $ Map.toList deps
   where
@@ -687,7 +708,7 @@ checkBundleBuildPlan platform compiler pool flags gpds =
             selectPackageBuildPlan platform compiler pool' gpd
         pkgPlan (Just f) gpd =
             checkPackageBuildPlan platform compiler pool' (flags' f gpd) gpd
-        flags' f gpd = maybe Map.empty id (Map.lookup (gpdPackageName gpd) f)
+        flags' f gpd = fromMaybe Map.empty (Map.lookup (gpdPackageName gpd) f)
         pool' = Map.union (gpdPackages gpds) pool
 
         dupError _ _ = error "Bug: Duplicate packages are not expected here"
@@ -721,20 +742,18 @@ instance Show BuildPlanCheck where
 -- given snapshot. Returns how well the snapshot satisfies the dependencies of
 -- the packages.
 checkSnapBuildPlan
-    :: ( MonadIO m, MonadMask m, MonadLogger m, MonadReader env m
-       , HasHttpManager env, HasConfig env, HasGHCVariant env
-       , MonadBaseControl IO m)
+    :: (StackM env m, HasConfig env, HasGHCVariant env)
     => [GenericPackageDescription]
     -> Maybe (Map PackageName (Map FlagName Bool))
     -> SnapName
     -> m BuildPlanCheck
 checkSnapBuildPlan gpds flags snap = do
-    platform <- asks (configPlatform . getConfig)
+    platform <- view platformL
     mbp <- loadMiniBuildPlan snap
 
     let
         compiler = mbpCompilerVersion mbp
-        snapPkgs = fmap mpiVersion $ mbpPackages mbp
+        snapPkgs = mpiVersion <$> mbpPackages mbp
         (f, errs) = checkBundleBuildPlan platform compiler snapPkgs flags gpds
         cerrs = compilerErrors compiler errs
 
@@ -756,9 +775,7 @@ checkSnapBuildPlan gpds flags snap = do
 -- | Find a snapshot and set of flags that is compatible with and matches as
 -- best as possible with the given 'GenericPackageDescription's.
 selectBestSnapshot
-    :: ( MonadIO m, MonadMask m, MonadLogger m, MonadReader env m
-       , HasHttpManager env, HasConfig env, HasGHCVariant env
-       , MonadBaseControl IO m)
+    :: (StackM env m, HasConfig env, HasGHCVariant env)
     => [GenericPackageDescription]
     -> NonEmpty SnapName
     -> m (SnapName, BuildPlanCheck)
@@ -808,7 +825,7 @@ showItems items = T.concat (map formatItem items)
 
 showPackageFlags :: PackageName -> Map FlagName Bool -> Text
 showPackageFlags pkg fl =
-    if (not $ Map.null fl) then
+    if not $ Map.null fl then
         T.concat
             [ "    - "
             , T.pack $ packageNameString pkg
@@ -819,7 +836,7 @@ showPackageFlags pkg fl =
             ]
     else ""
     where
-        formatFlags (f, v) = (show f) ++ " = " ++ (show v)
+        formatFlags (f, v) = show f ++ " = " ++ show v
 
 showMapPackages :: Map PackageName a -> Text
 showMapPackages mp = showItems $ Map.keys mp
@@ -842,7 +859,7 @@ showDepErrors flags errs =
     T.concat
         [ T.concat $ map formatError (Map.toList errs)
         , if T.null flagVals then ""
-          else ("Using package flags:\n" <> flagVals)
+          else "Using package flags:\n" <> flagVals
         ]
     where
         formatError (depName, DepError mversion neededBy) = T.concat
@@ -874,13 +891,17 @@ showDepErrors flags errs =
         userPkgs = Map.keys $ Map.unions (Map.elems (fmap deNeededBy errs))
         showFlags pkg = maybe "" (showPackageFlags pkg) (Map.lookup pkg flags)
 
+-- | Given a set of packages to shadow, this removes them, and any
+-- packages that transitively depend on them, from the 'MiniBuildPlan'.
+-- The 'Map' result yields all of the packages that were downstream of
+-- the shadowed packages. It does not include the shadowed packages.
 shadowMiniBuildPlan :: MiniBuildPlan
                     -> Set PackageName
                     -> (MiniBuildPlan, Map PackageName MiniPackageInfo)
 shadowMiniBuildPlan (MiniBuildPlan cv pkgs0) shadowed =
     (MiniBuildPlan cv (Map.fromList met), Map.fromList unmet)
   where
-    pkgs1 = Map.difference pkgs0 $ Map.fromSet (\_ -> ()) shadowed
+    pkgs1 = Map.difference pkgs0 $ Map.fromSet (const ()) shadowed
 
     depsMet = flip execState Map.empty $ mapM_ (check Set.empty) (Map.keys pkgs1)
 
@@ -953,13 +974,13 @@ shadowMiniBuildPlan (MiniBuildPlan cv pkgs0) shadowed =
 -- TODO: Allow custom plan to specify a name.
 
 parseCustomMiniBuildPlan
-    :: (MonadIO m, MonadMask m, MonadLogger m, MonadReader env m, HasHttpManager env, HasConfig env, HasGHCVariant env, MonadBaseControl IO m)
+    :: (StackMiniM env m, HasConfig env, HasGHCVariant env)
     => Maybe (Path Abs File) -- ^ Root directory for when url is a filepath
     -> T.Text
     -> m (MiniBuildPlan, SnapshotHash)
 parseCustomMiniBuildPlan mconfigPath0 url0 = do
     $logDebug $ "Loading " <> url0 <> " build plan"
-    case parseUrl $ T.unpack url0 of
+    case parseUrlThrow $ T.unpack url0 of
         Just req -> downloadCustom url0 req
         Nothing ->
            case mconfigPath0 of
@@ -975,7 +996,7 @@ parseCustomMiniBuildPlan mconfigPath0 url0 = do
                    -- cases.
                    binaryPath <- getBinaryPath hash
                    alreadyCached <- doesFileExist binaryPath
-                   unless alreadyCached $ taggedEncodeFile binaryPath mbp
+                   unless alreadyCached $ $(versionedEncodeFile miniBuildPlanVC) binaryPath mbp
                    return (mbp, hash)
   where
     downloadCustom url req = do
@@ -987,7 +1008,7 @@ parseCustomMiniBuildPlan mconfigPath0 url0 = do
         yamlBS <- liftIO $ S.readFile $ toFilePath cacheFP
         let yamlHash = doHash yamlBS
         binaryPath <- getBinaryPath yamlHash
-        liftM (, yamlHash) $ taggedDecodeOrLoad binaryPath $ do
+        liftM (, yamlHash) $ $(versionedDecodeOrLoad miniBuildPlanVC) binaryPath $ do
             (cs, mresolver) <- decodeYaml yamlBS
             parentMbp <- case (csCompilerVersion cs, mresolver) of
                 (Nothing, Nothing) -> throwM (NeitherCompilerOrResolverSpecified url)
@@ -1004,7 +1025,7 @@ parseCustomMiniBuildPlan mconfigPath0 url0 = do
         (cs, mresolver) <- decodeYaml yamlBS
         (getMbp, hash) <- case mresolver of
             Just (ResolverCustom _ url ) ->
-                case parseUrl $ T.unpack url of
+                case parseUrlThrow $ T.unpack url of
                     Just req -> do
                         let getMbp = do
                                 -- Ignore custom hash, under the
@@ -1024,11 +1045,11 @@ parseCustomMiniBuildPlan mconfigPath0 url0 = do
                                 exists <- doesFileExist binaryPath
                                 if exists
                                     then do
-                                        eres <- decodeFileOrFailDeep binaryPath
+                                        eres <- $(versionedDecodeFile miniBuildPlanVC) binaryPath
                                         case eres of
-                                            Right (Just mbp) -> return mbp
+                                            Just mbp -> return mbp
                                             -- Invalid format cache file, remove.
-                                            _ -> do
+                                            Nothing -> do
                                                 removeFile binaryPath
                                                 getMbp0
                                     else getMbp0
@@ -1066,12 +1087,12 @@ parseCustomMiniBuildPlan mconfigPath0 url0 = do
          , mbpPackages = mempty
          }
     getCustomPlanDir = do
-        root <- asks $ configStackRoot . getConfig
+        root <- view stackRootL
         return $ root </> $(mkRelDir "custom-plan")
-    doHash = SnapshotHash . B64URL.encode . SHA256.hash
+    doHash = SnapshotHash . B64URL.encode . Mem.convert . hashWith SHA256
 
 applyCustomSnapshot
-    :: (MonadIO m, MonadLogger m, MonadReader env m, HasHttpManager env, HasConfig env, MonadBaseControl IO m, MonadMask m)
+    :: (StackMiniM env m, HasConfig env)
     => CustomSnapshot
     -> MiniBuildPlan
     -> m MiniBuildPlan
@@ -1096,9 +1117,9 @@ applyCustomSnapshot cs mbp0 = do
         packageMap = Map.fromList $ map addFlagsAndOpts $ Set.toList packages
         cv = fromMaybe (mbpCompilerVersion mbp0) mcompilerVersion
         packages0 =
-             mbpPackages mbp0 `Map.difference` (Map.fromSet (\_ -> ()) dropPackages)
+             mbpPackages mbp0 `Map.difference` Map.fromSet (const ()) dropPackages
     mbp1 <- toMiniBuildPlan cv mempty packageMap
-    return $ MiniBuildPlan
+    return MiniBuildPlan
         { mbpCompilerVersion = cv
         , mbpPackages = Map.union (mbpPackages mbp1) packages0
         }
