@@ -1,4 +1,6 @@
+{-# LANGUAGE ConstraintKinds #-}
 {-# LANGUAGE DeriveDataTypeable #-}
+{-# LANGUAGE FlexibleContexts #-}
 
 -- | Clean a project.
 module Stack.Clean
@@ -8,10 +10,7 @@ module Stack.Clean
     ) where
 
 import           Control.Exception (Exception)
-import           Control.Monad.Catch (MonadCatch, MonadThrow, throwM)
-import           Control.Monad.IO.Class (MonadIO)
-import           Control.Monad.Logger (MonadLogger)
-import           Control.Monad.Reader (MonadReader, asks)
+import           Control.Monad.Catch (throwM)
 import           Data.Foldable (forM_)
 import           Data.List ((\\),intercalate)
 import qualified Data.Map.Strict as Map
@@ -21,14 +20,17 @@ import           Path (Path, Abs, Dir)
 import           Path.IO (ignoringAbsence, removeDirRecur)
 import           Stack.Build.Source (getLocalPackageViews)
 import           Stack.Build.Target (LocalPackageView(..))
+import           Stack.Config (getLocalPackages)
 import           Stack.Constants (distDirFromDir, workDirFromDir)
-import           Stack.Types
+import           Stack.Types.PackageName
+import           Stack.Types.Config
+import           Stack.Types.StackT
 
 -- | Deletes build artifacts in the current project.
 --
 -- Throws 'StackCleanException'.
 clean
-    :: (MonadCatch m, MonadIO m, MonadReader env m, HasEnvConfig env, MonadLogger m)
+    :: (StackM env m, HasEnvConfig env)
     => CleanOpts
     -> m ()
 clean cleanOpts = do
@@ -36,14 +38,15 @@ clean cleanOpts = do
     forM_ dirs (ignoringAbsence . removeDirRecur)
 
 dirsToDelete
-    :: (MonadThrow m, MonadIO m, MonadReader env m, HasEnvConfig env, MonadLogger m)
+    :: (StackM env m, HasEnvConfig env)
     => CleanOpts
     -> m [Path Abs Dir]
 dirsToDelete cleanOpts = do
-    localPkgDirs <- asks (Map.keys . envConfigPackages . getEnvConfig)
+    packages <- getLocalPackages
     case cleanOpts of
-        CleanShallow [] -> do
-            mapM distDirFromDir localPkgDirs
+        CleanShallow [] ->
+            -- Filter out packages listed as extra-deps
+            mapM distDirFromDir . Map.keys . Map.filter (== False) $ packages
         CleanShallow targets -> do
             localPkgViews <- getLocalPackageViews
             let localPkgNames = Map.keys localPkgViews
@@ -52,7 +55,7 @@ dirsToDelete cleanOpts = do
                 [] -> mapM distDirFromDir (mapMaybe getPkgDir targets)
                 xs -> throwM (NonLocalPackages xs)
         CleanFull -> do
-            pkgWorkDirs <- mapM workDirFromDir localPkgDirs
+            pkgWorkDirs <- mapM workDirFromDir (Map.keys packages)
             projectWorkDir <- getProjectWorkDir
             return (projectWorkDir : pkgWorkDirs)
 
