@@ -3,17 +3,20 @@
 -- | Docker configuration
 module Stack.Config.Docker where
 
-import Control.Exception.Lifted
-import Control.Monad
-import Control.Monad.Catch (throwM, MonadThrow)
-import Data.List (find)
-import Data.Maybe
-import Data.Monoid.Extra
+import           Control.Exception.Lifted
+import           Control.Monad.Catch (MonadThrow)
+import           Data.List (find)
+import           Data.Maybe
+import           Data.Monoid.Extra
 import qualified Data.Text as T
-import Data.Typeable (Typeable)
-import Distribution.Version (simplifyVersionRange)
-import Path
-import Stack.Types
+import           Data.Typeable (Typeable)
+import           Distribution.Version (simplifyVersionRange)
+import           Path
+import           Stack.Types.BuildPlan
+import           Stack.Types.Version
+import           Stack.Types.Config
+import           Stack.Types.Docker
+import           Stack.Types.Resolver
 
 -- | Interprets DockerOptsMonoid options.
 dockerOptsFromMonoid
@@ -73,17 +76,8 @@ dockerOptsFromMonoid mproject stackRoot maresolver DockerOptsMonoid{..} = do
         dockerSetUser = getFirst dockerMonoidSetUser
         dockerRequireDockerVersion =
             simplifyVersionRange (getIntersectingVersionRange dockerMonoidRequireDockerVersion)
-    dockerDatabasePath <-
-        case getFirst dockerMonoidDatabasePath of
-            Nothing -> return $ stackRoot </> $(mkRelFile "docker.db")
-            Just fp ->
-                case parseAbsFile fp of
-                    Left e -> throwM (InvalidDatabasePathException e)
-                    Right p -> return p
-    dockerStackExe <-
-        case getFirst dockerMonoidStackExe of
-            Just e -> liftM Just (parseDockerStackExe e)
-            Nothing -> return Nothing
+        dockerDatabasePath = fromFirst (stackRoot </> $(mkRelFile "docker.db")) dockerMonoidDatabasePath
+        dockerStackExe = getFirst dockerMonoidStackExe
     return DockerOpts{..}
   where emptyToNothing Nothing = Nothing
         emptyToNothing (Just s) | null s = Nothing
@@ -109,5 +103,4 @@ instance Show StackDockerConfigException where
             , "\nUse an LTS resolver, or set the '"
             , T.unpack dockerImageArgName
             , "' explicitly, in your configuration file."]
-    show (InvalidDatabasePathException ex) =
-        concat ["Invalid database path: ", show ex]
+    show (InvalidDatabasePathException ex) = "Invalid database path: " ++ show ex
