@@ -2,7 +2,7 @@
 
 module Stack.Config.Urls (urlsFromMonoid) where
 
-import           Stack.Types
+import           Stack.Types.Urls
 import           Data.Monoid.Extra
 
 urlsFromMonoid :: UrlsMonoid -> Urls
@@ -13,7 +13,7 @@ urlsFromMonoid monoid =
         (fromFirst defaultNightlyBuildPlans $ urlsMonoidNightlyBuildPlans monoid)
     where
     defaultLatestSnapshot =
-        "https://www.stackage.org/download/snapshots.json"
+        "https://s3.amazonaws.com/haddock.stackage.org/snapshots.json"
     defaultLtsBuildPlans =
         "https://raw.githubusercontent.com/fpco/lts-haskell/master/"
     defaultNightlyBuildPlans =
