@@ -12,10 +12,12 @@ module Stack.Constants
     ,imageStagingDir
     ,projectDockerSandboxDir
     ,stackDotYaml
+    ,stackWorkEnvVar
     ,stackRootEnvVar
     ,stackRootOptionName
     ,deprecatedStackRootOptionName
     ,inContainerEnvVar
+    ,inNixShellEnvVar
     ,configCacheFile
     ,configCabalMod
     ,buildCacheFile
@@ -30,13 +32,14 @@ module Stack.Constants
     ,implicitGlobalProjectDir
     ,hpcRelativeDir
     ,hpcDirFromDir
-    ,objectInterfaceDir
+    ,objectInterfaceDirL
     ,templatesDir
     ,defaultUserConfigPathDeprecated
     ,defaultUserConfigPath
     ,defaultGlobalConfigPathDeprecated
     ,defaultGlobalConfigPath
     ,platformVariantEnvVar
+    ,compilerOptionsCabalFlag
     )
     where
 
@@ -46,6 +49,7 @@ import           Data.Char (toUpper)
 import           Data.HashSet (HashSet)
 import qualified Data.HashSet as HashSet
 import           Data.Text (Text)
+import           Lens.Micro (Getting)
 import           Path as FL
 import           Prelude
 import           Stack.Types.Compiler
@@ -66,11 +70,11 @@ haskellPreprocessorExts :: [Text]
 haskellPreprocessorExts = ["gc", "chs", "hsc", "x", "y", "ly", "cpphs"]
 
 -- | Output .o/.hi directory.
-objectInterfaceDir :: (MonadReader env m, HasConfig env)
-  => BuildConfig -> m (Path Abs Dir)
-objectInterfaceDir bconfig = do
-  bcwd <- bcWorkDir bconfig
-  return (bcwd </> $(mkRelDir "odir/"))
+objectInterfaceDirL :: HasBuildConfig env => Getting r env (Path Abs Dir)
+objectInterfaceDirL = to $ \env -> -- FIXME is this idomatic lens code?
+  let workDir = view workDirL env
+      root = view projectRootL env
+   in root </> workDir </> $(mkRelDir "odir/")
 
 -- | The filename used for dirtiness check of source files.
 buildCacheFile :: (MonadThrow m, MonadReader env m, HasEnvConfig env)
@@ -139,11 +143,10 @@ distDirFromDir fp =
     liftM (fp </>) distRelativeDir
 
 -- | Package's working directory.
-workDirFromDir :: (MonadThrow m, MonadReader env m, HasEnvConfig env)
+workDirFromDir :: (MonadReader env m, HasEnvConfig env)
                => Path Abs Dir
                -> m (Path Abs Dir)
-workDirFromDir fp =
-    liftM (fp </>) getWorkDir
+workDirFromDir fp = view $ workDirL.to (fp </>)
 
 -- | Directory for project templates.
 templatesDir :: Config -> Path Abs Dir
@@ -153,9 +156,9 @@ templatesDir config = configStackRoot config </> $(mkRelDir "templates")
 distRelativeDir :: (MonadThrow m, MonadReader env m, HasEnvConfig env)
                 => m (Path Rel Dir)
 distRelativeDir = do
-    cabalPkgVer <- asks (envConfigCabalVersion . getEnvConfig)
+    cabalPkgVer <- view cabalVersionL
     platform <- platformGhcRelDir
-    wc <- getWhichCompiler
+    wc <- view $ actualCompilerVersionL.to whichCompiler
     -- Cabal version, suffixed with "_ghcjs" if we're using GHCJS.
     envDir <-
         parseRelDir $
@@ -163,7 +166,7 @@ distRelativeDir = do
         packageIdentifierString $
         PackageIdentifier cabalPackageName cabalPkgVer
     platformAndCabal <- useShaPathOnWindows (platform </> envDir)
-    workDir <- getWorkDir
+    workDir <- view workDirL
     return $
         workDir </>
         $(mkRelDir "dist") </>
@@ -174,7 +177,7 @@ projectDockerSandboxDir :: (MonadReader env m, HasConfig env)
   => Path Abs Dir      -- ^ Project root
   -> m (Path Abs Dir)  -- ^ Docker sandbox
 projectDockerSandboxDir projectRoot = do
-  workDir <- getWorkDir
+  workDir <- view workDirL
   return $ projectRoot </> workDir </> $(mkRelDir "docker/")
 
 -- | Image staging dir from project root.
@@ -183,7 +186,7 @@ imageStagingDir :: (MonadReader env m, HasConfig env, MonadThrow m)
   -> Int               -- ^ Index of image
   -> m (Path Abs Dir)  -- ^ Docker sandbox
 imageStagingDir projectRoot imageIdx = do
-  workDir <- getWorkDir
+  workDir <- view workDirL
   idxRelDir <- parseRelDir (show imageIdx)
   return $ projectRoot </> workDir </> $(mkRelDir "image") </> idxRelDir
 
@@ -199,6 +202,10 @@ stackProgName = "stack"
 stackDotYaml :: Path Rel File
 stackDotYaml = $(mkRelFile "stack.yaml")
 
+-- | Environment variable used to override the '.stack-work' relative dir.
+stackWorkEnvVar :: String
+stackWorkEnvVar = "STACK_WORK"
+
 -- | Environment variable used to override the '~/.stack' location.
 stackRootEnvVar :: String
 stackRootEnvVar = "STACK_ROOT"
@@ -220,6 +227,12 @@ deprecatedStackRootOptionName = "global-stack-root"
 inContainerEnvVar :: String
 inContainerEnvVar = stackProgNameUpper ++ "_IN_CONTAINER"
 
+-- | Environment variable used to indicate stack is running in container.
+-- although we already have STACK_IN_NIX_EXTRA_ARGS that is set in the same conditions,
+-- it can happen that STACK_IN_NIX_EXTRA_ARGS is set to empty.
+inNixShellEnvVar :: String
+inNixShellEnvVar = map toUpper stackProgName ++ "_IN_NIXSHELL"
+
 -- See https://downloads.haskell.org/~ghc/7.10.1/docs/html/libraries/ghc/src/Module.html#integerPackageKey
 wiredInPackages :: HashSet PackageName
 wiredInPackages =
@@ -334,3 +347,9 @@ buildPlanDir = (</> $(mkRelDir "build-plan"))
 -- names.  Used to ensure incompatible binaries aren't shared between Docker builds and host
 platformVariantEnvVar :: String
 platformVariantEnvVar = stackProgNameUpper ++ "_PLATFORM_VARIANT"
+
+-- | Provides --ghc-options for 'Ghc', and similarly, --ghcjs-options
+-- for 'Ghcjs'.
+compilerOptionsCabalFlag :: WhichCompiler -> String
+compilerOptionsCabalFlag Ghc = "--ghc-options"
+compilerOptionsCabalFlag Ghcjs = "--ghcjs-options"
