@@ -16,13 +16,11 @@ module Stack.Coverage
     , generateHpcMarkupIndex
     ) where
 
-import           Control.Applicative
+import           Control.Exception.Safe (handleIO)
 import           Control.Exception.Lifted
-import           Control.Monad (liftM, when, unless, void)
-import           Control.Monad.Catch (MonadMask)
+import           Control.Monad (liftM, when, unless, void, (<=<))
 import           Control.Monad.IO.Class
 import           Control.Monad.Logger
-import           Control.Monad.Reader (MonadReader, asks)
 import           Control.Monad.Trans.Resource
 import qualified Data.ByteString.Char8 as S8
 import           Data.Foldable (forM_, asum, toList)
@@ -32,6 +30,7 @@ import qualified Data.Map.Strict as Map
 import           Data.Maybe
 import           Data.Maybe.Extra (mapMaybeM)
 import           Data.Monoid ((<>))
+import           Data.String
 import           Data.Text (Text)
 import qualified Data.Text as T
 import qualified Data.Text.Encoding as T
@@ -44,16 +43,26 @@ import           Path.IO
 import           Prelude hiding (FilePath, writeFile)
 import           Stack.Build.Source (parseTargetsFromBuildOpts)
 import           Stack.Build.Target
+import           Stack.Config (getLocalPackages)
 import           Stack.Constants
 import           Stack.Package
-import           Stack.Types
+import           Stack.PrettyPrint
+import           Stack.Types.Compiler
+import           Stack.Types.Config
+import           Stack.Types.Package
+import           Stack.Types.PackageIdentifier
+import           Stack.Types.PackageName
+import           Stack.Types.StackT (StackM)
+import           Stack.Types.StringError
+import           Stack.Types.Version
 import           System.FilePath (isPathSeparator)
 import           System.Process.Read
 import           Text.Hastache (htmlEscape)
 import           Trace.Hpc.Tix
+import           Web.Browser (openBrowser)
 
 -- | Invoked at the beginning of running with "--coverage"
-deleteHpcReports :: (MonadIO m, MonadMask m, MonadReader env m, HasEnvConfig env)
+deleteHpcReports :: (StackM env m, HasEnvConfig env)
                  => m ()
 deleteHpcReports = do
     hpcDir <- hpcReportDir
@@ -61,8 +70,8 @@ deleteHpcReports = do
 
 -- | Move a tix file into a sub-directory of the hpc report directory. Deletes the old one if one is
 -- present.
-updateTixFile :: (MonadIO m,MonadReader env m,MonadLogger m,MonadBaseControl IO m,MonadMask m,HasEnvConfig env)
-            => PackageName -> Path Abs File -> String -> m ()
+updateTixFile :: (StackM env m, HasEnvConfig env)
+              => PackageName -> Path Abs File -> String -> m ()
 updateTixFile pkgName tixSrc testName = do
     exists <- doesFileExist tixSrc
     when exists $ do
@@ -76,10 +85,14 @@ updateTixFile pkgName tixSrc testName = do
             Nothing -> $logError $ "Failed to read " <> T.pack (toFilePath tixSrc)
             Just tix -> do
                 liftIO $ writeTix (toFilePath tixDest) (removeExeModules tix)
+                -- TODO: ideally we'd do a file move, but IIRC this can
+                -- have problems. Something about moving between drives
+                -- on windows?
+                copyFile tixSrc =<< parseAbsFile (toFilePath tixDest ++ ".premunging")
                 ignoringAbsence (removeFile tixSrc)
 
 -- | Get the directory used for hpc reports for the given pkgId.
-hpcPkgPath :: (MonadIO m,MonadReader env m,MonadMask m,HasEnvConfig env)
+hpcPkgPath :: (StackM env m, HasEnvConfig env)
             => PackageName -> m (Path Abs Dir)
 hpcPkgPath pkgName = do
     outputDir <- hpcReportDir
@@ -88,7 +101,7 @@ hpcPkgPath pkgName = do
 
 -- | Get the tix file location, given the name of the file (without extension), and the package
 -- identifier string.
-tixFilePath :: (MonadIO m,MonadReader env m,MonadMask m,HasEnvConfig env)
+tixFilePath :: (StackM env m, HasEnvConfig env)
             => PackageName -> String ->  m (Path Abs File)
 tixFilePath pkgName testName = do
     pkgPath <- hpcPkgPath pkgName
@@ -96,29 +109,32 @@ tixFilePath pkgName testName = do
     return (pkgPath </> tixRel)
 
 -- | Generates the HTML coverage report and shows a textual coverage summary for a package.
-generateHpcReport :: (MonadIO m,MonadReader env m,MonadLogger m,MonadBaseControl IO m,MonadMask m,HasEnvConfig env)
+generateHpcReport :: (StackM env m, HasEnvConfig env)
                   => Path Abs Dir -> Package -> [Text] -> m ()
 generateHpcReport pkgDir package tests = do
+    compilerVersion <- view actualCompilerVersionL
     -- If we're using > GHC 7.10, the hpc 'include' parameter must specify a ghc package key. See
     -- https://github.com/commercialhaskell/stack/issues/785
     let pkgName = packageNameText (packageName package)
         pkgId = packageIdentifierString (packageIdentifier package)
-    compilerVersion <- asks (envConfigCompilerVersion . getEnvConfig)
+        ghcVersion = getGhcVersion compilerVersion
     eincludeName <-
         -- Pre-7.8 uses plain PKG-version in tix files.
-        if getGhcVersion compilerVersion < $(mkVersion "7.10") then return $ Right $ Just pkgId
+        if ghcVersion < $(mkVersion "7.10") then return $ Right $ Just pkgId
         -- We don't expect to find a package key if there is no library.
         else if not (packageHasLibrary package) then return $ Right Nothing
         -- Look in the inplace DB for the package key.
         -- See https://github.com/commercialhaskell/stack/issues/1181#issuecomment-148968986
         else do
-            mghcPkgKey <- findPackageKeyForBuiltPackage pkgDir (packageIdentifier package)
-            case mghcPkgKey of
-                Nothing -> do
-                    let msg = "Failed to find GHC package key for " <> pkgName
-                    $logError msg
-                    return $ Left msg
-                Just ghcPkgKey -> return $ Right $ Just $ T.unpack ghcPkgKey
+            -- GHC 8.0 uses package id instead of package key.
+            -- See https://github.com/commercialhaskell/stack/issues/2424
+            let hpcNameField = if ghcVersion >= $(mkVersion "8.0") then "id" else "key"
+            eincludeName <- findPackageFieldForBuiltPackage pkgDir (packageIdentifier package) hpcNameField
+            case eincludeName of
+                Left err -> do
+                    $logError err
+                    return $ Left err
+                Right includeName -> return $ Right $ Just $ T.unpack includeName
     forM_ tests $ \testName -> do
         tixSrc <- tixFilePath (packageName package) (T.unpack testName)
         let report = "coverage report for " <> pkgName <> "'s test-suite \"" <> testName <> "\""
@@ -131,30 +147,34 @@ generateHpcReport pkgDir package tests = do
                 let extraArgs = case mincludeName of
                         Just includeName -> ["--include", includeName ++ ":"]
                         Nothing -> []
-                generateHpcReportInternal tixSrc reportDir report extraArgs extraArgs
+                mreportPath <- generateHpcReportInternal tixSrc reportDir report extraArgs extraArgs
+                forM_ mreportPath (displayReportPath report)
 
-generateHpcReportInternal :: (MonadIO m,MonadReader env m,MonadLogger m,MonadBaseControl IO m,MonadMask m,HasEnvConfig env)
-                          => Path Abs File -> Path Abs Dir -> Text -> [String] -> [String] -> m ()
+generateHpcReportInternal :: (StackM env m, HasEnvConfig env)
+                          => Path Abs File -> Path Abs Dir -> Text -> [String] -> [String] -> m (Maybe (Path Abs File))
 generateHpcReportInternal tixSrc reportDir report extraMarkupArgs extraReportArgs = do
     -- If a .tix file exists, move it to the HPC output directory and generate a report for it.
     tixFileExists <- doesFileExist tixSrc
     if not tixFileExists
-        then $logError $ T.concat
-            [ "Didn't find .tix for "
-            , report
-            , " - expected to find it at "
-            , T.pack (toFilePath tixSrc)
-            , "."
-            ]
+        then do
+            $logError $ T.concat
+                 [ "Didn't find .tix for "
+                 , report
+                 , " - expected to find it at "
+                 , T.pack (toFilePath tixSrc)
+                 , "."
+                 ]
+            return Nothing
         else (`catch` \err -> do
                  let msg = show (err :: ReadProcessException)
                  $logError (T.pack msg)
-                 generateHpcErrorReport reportDir $ sanitize msg) $
+                 generateHpcErrorReport reportDir $ sanitize msg
+                 return Nothing) $
              (`onException` $logError ("Error occurred while producing " <> report)) $ do
             -- Directories for .mix files.
             hpcRelDir <- hpcRelativeDir
             -- Compute arguments used for both "hpc markup" and "hpc report".
-            pkgDirs <- Map.keys . envConfigPackages <$> asks getEnvConfig
+            pkgDirs <- liftM Map.keys getLocalPackages
             let args =
                     -- Use index files from all packages (allows cross-package coverage results).
                     concatMap (\x -> ["--srcdir", toFilePathNoTrailingSep x]) pkgDirs ++
@@ -184,12 +204,11 @@ generateHpcReportInternal tixSrc reportDir report extraMarkupArgs extraReportArg
                             ]
                     $logError (msg False)
                     generateHpcErrorReport reportDir (msg True)
+                    return Nothing
                 else do
+                    let reportPath = reportDir </> $(mkRelFile "hpc_index.html")
                     -- Print output, stripping @\r@ characters because Windows.
                     forM_ outputLines ($logInfo . T.decodeUtf8)
-                    $logInfo
-                        ("The " <> report <> " is available at " <>
-                         T.pack (toFilePath (reportDir </> $(mkRelFile "hpc_index.html"))))
                     -- Generate the markup.
                     void $ readProcessStdout Nothing menv "hpc"
                         ( "markup"
@@ -197,20 +216,22 @@ generateHpcReportInternal tixSrc reportDir report extraMarkupArgs extraReportArg
                         : ("--destdir=" ++ toFilePathNoTrailingSep reportDir)
                         : (args ++ extraMarkupArgs)
                         )
+                    return (Just reportPath)
 
 data HpcReportOpts = HpcReportOpts
     { hroptsInputs :: [Text]
     , hroptsAll :: Bool
     , hroptsDestDir :: Maybe String
+    , hroptsOpenBrowser :: Bool
     } deriving (Show)
 
-generateHpcReportForTargets :: (MonadIO m, MonadReader env m, MonadBaseControl IO m, MonadMask m, MonadLogger m, HasEnvConfig env)
+generateHpcReportForTargets :: (StackM env m, HasEnvConfig env)
                             => HpcReportOpts -> m ()
 generateHpcReportForTargets opts = do
     let (tixFiles, targetNames) = partition (".tix" `T.isSuffixOf`) (hroptsInputs opts)
     targetTixFiles <-
-         -- When there aren't any package component arguments, then
-         -- don't default to all package components.
+         -- When there aren't any package component arguments, and --all
+         -- isn't passed, default to not considering any targets.
          if not (hroptsAll opts) && null targetNames
          then return []
          else do
@@ -222,10 +243,10 @@ generateHpcReportForTargets opts = do
                     { boptsCLITargets = if hroptsAll opts then [] else targetNames }
              liftM concat $ forM (Map.toList targets) $ \(name, target) ->
                  case target of
-                     STUnknown -> fail $
-                         packageNameString name ++ " isn't a known local page"
-                     STNonLocal -> fail $
-                         "Expected a local package, but " ++
+                     STUnknown -> throwString $
+                         "Error: " ++ packageNameString name ++ " isn't a known local page"
+                     STNonLocal -> throwString $
+                         "Error: Expected a local package, but " ++
                          packageNameString name ++
                          " is either an extra-dep or in the snapshot."
                      STLocalComps comps -> do
@@ -233,7 +254,7 @@ generateHpcReportForTargets opts = do
                          forM (toList comps) $ \nc ->
                              case nc of
                                  CTest testName ->
-                                     liftM (pkgPath </>) $ parseRelFile (T.unpack testName ++ ".tix")
+                                     liftM (pkgPath </>) $ parseRelFile (T.unpack testName ++ "/" ++ T.unpack testName ++ ".tix")
                                  _ -> fail $
                                      "Can't specify anything except test-suites as hpc report targets (" ++
                                      packageNameString name ++
@@ -243,32 +264,44 @@ generateHpcReportForTargets opts = do
                          exists <- doesDirExist pkgPath
                          if exists
                              then do
-                                 (_, files) <- listDir pkgPath
-                                 return (filter ((".tix" `isSuffixOf`) . toFilePath) files)
+                                 (dirs, _) <- listDir pkgPath
+                                 liftM concat $ forM dirs $ \dir -> do
+                                     (_, files) <- listDir dir
+                                     return (filter ((".tix" `isSuffixOf`) . toFilePath) files)
                              else return []
-    tixPaths <- liftM (++ targetTixFiles) $ mapM (resolveFile' . T.unpack) tixFiles
+    tixPaths <- liftM (\xs -> xs ++ targetTixFiles) $ mapM (resolveFile' . T.unpack) tixFiles
     when (null tixPaths) $
-        fail "Not generating combined report, because no targets or tix files are specified."
+        throwString "Not generating combined report, because no targets or tix files are specified."
+    outputDir <- hpcReportDir
     reportDir <- case hroptsDestDir opts of
-        Nothing -> liftM (</> $(mkRelDir "combined/custom")) hpcReportDir
+        Nothing -> return (outputDir </> $(mkRelDir "combined/custom"))
         Just destDir -> do
             dest <- resolveDir' destDir
             ensureDir dest
             return dest
-    generateUnionReport "combined report" reportDir tixPaths
+    let report = "combined report"
+    mreportPath <- generateUnionReport report reportDir tixPaths
+    forM_ mreportPath $ \reportPath ->
+        if hroptsOpenBrowser opts
+            then do
+                $prettyInfo $ "Opening" <+> display reportPath <+> "in the browser."
+                void $ liftIO $ openBrowser (toFilePath reportPath)
+            else displayReportPath report reportPath
 
-generateHpcUnifiedReport :: (MonadIO m,MonadReader env m,MonadLogger m,MonadBaseControl IO m,MonadMask m,HasEnvConfig env)
+generateHpcUnifiedReport :: (StackM env m, HasEnvConfig env)
                          => m ()
 generateHpcUnifiedReport = do
     outputDir <- hpcReportDir
     ensureDir outputDir
     (dirs, _) <- listDir outputDir
-    tixFiles <- liftM (concat . concat) $ forM (filter (("combined" /=) . dirnameString) dirs) $ \dir -> do
+    tixFiles0 <- liftM (concat . concat) $ forM (filter (("combined" /=) . dirnameString) dirs) $ \dir -> do
         (dirs', _) <- listDir dir
         forM dirs' $ \dir' -> do
             (_, files) <- listDir dir'
             return (filter ((".tix" `isSuffixOf`) . toFilePath) files)
-    let reportDir = outputDir </> $(mkRelDir "combined/all")
+    extraTixFiles <- findExtraTixFiles
+    let tixFiles = tixFiles0  ++ extraTixFiles
+        reportDir = outputDir </> $(mkRelDir "combined/all")
     if length tixFiles < 2
         then $logInfo $ T.concat
             [ if null tixFiles then "No tix files" else "Only one tix file"
@@ -276,10 +309,13 @@ generateHpcUnifiedReport = do
             , T.pack (toFilePath outputDir)
             , ", so not generating a unified coverage report."
             ]
-        else generateUnionReport "unified report" reportDir tixFiles
+        else do
+            let report = "unified report"
+            mreportPath <- generateUnionReport report reportDir tixFiles
+            forM_ mreportPath (displayReportPath report)
 
-generateUnionReport :: (MonadIO m,MonadReader env m,MonadLogger m,MonadBaseControl IO m,MonadMask m,HasEnvConfig env)
-                    => Text -> Path Abs Dir -> [Path Abs File] -> m ()
+generateUnionReport :: (StackM env m, HasEnvConfig env)
+                    => Text -> Path Abs Dir -> [Path Abs File] -> m (Maybe (Path Abs File))
 generateUnionReport report reportDir tixFiles = do
     (errs, tix) <- fmap (unionTixes . map removeExeModules) (mapMaybeM readTixOrLog tixFiles)
     $logDebug $ "Using the following tix files: " <> T.pack (show tixFiles)
@@ -315,7 +351,7 @@ unionTixes tixes = (Map.keys errs, Tix (Map.elems outputs))
         | hash1 == hash2 && len1 == len2 = Right (TixModule k hash1 len1 (zipWith (+) tix1 tix2))
     merge _ _ = Left ()
 
-generateHpcMarkupIndex :: (MonadIO m,MonadReader env m,MonadLogger m,MonadMask m,HasEnvConfig env)
+generateHpcMarkupIndex :: (StackM env m, HasEnvConfig env)
                        => m ()
 generateHpcMarkupIndex = do
     outputDir <- hpcReportDir
@@ -390,15 +426,50 @@ sanitize = LT.toStrict . htmlEscape . LT.pack
 dirnameString :: Path r Dir -> String
 dirnameString = dropWhileEnd isPathSeparator . toFilePath . dirname
 
-findPackageKeyForBuiltPackage :: (MonadIO m, MonadReader env m, MonadThrow m, HasEnvConfig env)
-                              => Path Abs Dir -> PackageIdentifier -> m (Maybe Text)
-findPackageKeyForBuiltPackage pkgDir pkgId = do
+findPackageFieldForBuiltPackage
+    :: (StackM env m, HasEnvConfig env)
+    => Path Abs Dir -> PackageIdentifier -> Text -> m (Either Text Text)
+findPackageFieldForBuiltPackage pkgDir pkgId field = do
     distDir <- distDirFromDir pkgDir
-    path <- liftM (distDir </>) $
-        parseRelFile ("package.conf.inplace/" ++ packageIdentifierString pkgId ++ "-inplace.conf")
-    exists <- doesFileExist path
-    if exists
-        then do
+    let inplaceDir = distDir </> $(mkRelDir "package.conf.inplace")
+        pkgIdStr = packageIdentifierString pkgId
+        notFoundErr = return $ Left $ "Failed to find package key for " <> T.pack pkgIdStr
+        extractField path = do
             contents <- liftIO $ T.readFile (toFilePath path)
-            return $ asum (map (T.stripPrefix "key: ") (T.lines contents))
-        else return Nothing
+            case asum (map (T.stripPrefix (field <> ": ")) (T.lines contents)) of
+                Just result -> return $ Right result
+                Nothing -> notFoundErr
+    cabalVer <- view cabalVersionL
+    if cabalVer < $(mkVersion "1.24")
+        then do
+            path <- liftM (inplaceDir </>) $ parseRelFile (pkgIdStr ++ "-inplace.conf")
+            $logDebug $ "Parsing config in Cabal < 1.24 location: " <> T.pack (toFilePath path)
+            exists <- doesFileExist path
+            if exists then extractField path else notFoundErr
+        else do
+            -- With Cabal-1.24, it's in a different location.
+            $logDebug $ "Scanning " <> T.pack (toFilePath inplaceDir) <> " for files matching " <> T.pack pkgIdStr
+            (_, files) <- handleIO (const $ return ([], [])) $ listDir inplaceDir
+            $logDebug $ T.pack (show files)
+            case mapMaybe (\file -> fmap (const file) . (T.stripSuffix ".conf" <=< T.stripPrefix (T.pack (pkgIdStr ++ "-")))
+                          . T.pack . toFilePath . filename $ file) files of
+                [] -> notFoundErr
+                [path] -> extractField path
+                _ -> return $ Left $ "Multiple files matching " <> T.pack (pkgIdStr ++ "-*.conf") <> " found in " <>
+                    T.pack (toFilePath inplaceDir) <> ". Maybe try 'stack clean' on this package?"
+
+displayReportPath :: (StackM env m, HasAnsiAnn (Ann a), Display a)
+                  => Text -> a -> m ()
+displayReportPath report reportPath =
+     $prettyInfo $ "The" <+> fromString (T.unpack report) <+> "is available at" <+> display reportPath
+
+findExtraTixFiles :: (StackM env m , HasEnvConfig env) => m [Path Abs File]
+findExtraTixFiles = do
+    outputDir <- hpcReportDir
+    let dir = outputDir </> $(mkRelDir "extra-tix-files")
+    dirExists <- doesDirExist dir
+    if dirExists
+        then do
+            (_, files) <- listDir dir
+            return $ filter ((".tix" `isSuffixOf`) . toFilePath) files
+        else return []
