@@ -24,13 +24,13 @@ import           Control.Applicative
 import           Control.Concurrent.MVar.Lifted (MVar,modifyMVar_,newMVar)
 import           Control.Exception.Lifted
 import           Control.Monad
-import           Control.Monad.Catch (MonadThrow,throwM,MonadCatch,MonadMask)
+import           Control.Monad.Catch (MonadThrow,throwM,MonadCatch)
 import           Control.Monad.IO.Class (MonadIO,liftIO)
 import           Control.Monad.Logger (MonadLogger,logError,logInfo,logWarn)
-import           Control.Monad.Reader (MonadReader,asks,runReaderT)
-import           Control.Monad.Writer (execWriter,runWriter,tell)
+import           Control.Monad.Reader (MonadReader,runReaderT)
 import           Control.Monad.Trans.Control (MonadBaseControl)
-import qualified "cryptohash" Crypto.Hash as Hash
+import           Control.Monad.Writer (execWriter,runWriter,tell)
+import qualified Crypto.Hash as Hash (Digest, MD5, hash)
 import           Data.Aeson.Extended (FromJSON(..),(.:),(.:?),(.!=),eitherDecode)
 import           Data.ByteString.Builder (stringUtf8,charUtf8,toLazyByteString)
 import qualified Data.ByteString.Char8 as BS
@@ -38,7 +38,7 @@ import qualified Data.ByteString.Lazy.Char8 as LBS
 import           Data.Char (isSpace,toUpper,isAscii,isDigit)
 import           Data.Conduit.List (sinkNull)
 import           Data.List (dropWhileEnd,intercalate,isPrefixOf,isInfixOf,foldl')
-import           Data.List.Extra (trim)
+import           Data.List.Extra (trim, nubOrd)
 import           Data.Map.Strict (Map)
 import qualified Data.Map.Strict as Map
 import           Data.Maybe
@@ -48,12 +48,8 @@ import           Data.Text (Text)
 import qualified Data.Text as T
 import qualified Data.Text.Encoding as T
 import           Data.Time (UTCTime,LocalTime(..),diffDays,utcToLocalTime,getZonedTime,ZonedTime(..))
-import           Data.Typeable (Typeable)
 import           Data.Version (showVersion)
-import           Distribution.System (Platform (Platform),Arch (X86_64),OS (Linux))
-import           Distribution.Text (display)
 import           GHC.Exts (sortWith)
-import           Network.HTTP.Client.Conduit (HasHttpManager)
 import           Path
 import           Path.Extra (toFilePathNoTrailingSep)
 import           Path.IO hiding (canonicalizePath)
@@ -62,27 +58,31 @@ import           Prelude -- Fix redundant import warnings
 import           Stack.Config (getInContainer)
 import           Stack.Constants
 import           Stack.Docker.GlobalDB
-import           Stack.Types
+import           Stack.Types.PackageIndex
+import           Stack.Types.Version
+import           Stack.Types.Config
+import           Stack.Types.Docker
 import           Stack.Types.Internal
+import           Stack.Types.StackT
 import           Stack.Setup (ensureDockerStackExe)
 import           System.Directory (canonicalizePath,getHomeDirectory)
 import           System.Environment (getEnv,getEnvironment,getProgName,getArgs,getExecutablePath)
-import           System.Exit (exitSuccess, exitWith)
+import           System.Exit (exitSuccess, exitWith, ExitCode(..))
 import qualified System.FilePath as FP
-import           System.IO (stderr,stdin,stdout,hIsTerminalDevice)
+import           System.IO (stderr,stdin,stdout,hIsTerminalDevice, hClose)
 import           System.IO.Error (isDoesNotExistError)
 import           System.IO.Unsafe (unsafePerformIO)
 import qualified System.PosixCompat.User as User
 import qualified System.PosixCompat.Files as Files
+import           System.Process (CreateProcess(..), StdStream(..), waitForProcess)
 import           System.Process.PagerEditor (editByteString)
 import           System.Process.Read
 import           System.Process.Run
-import           System.Process (CreateProcess(delegate_ctlc))
 import           Text.Printf (printf)
 
 #ifndef WINDOWS
 import           Control.Concurrent (threadDelay)
-import           Control.Monad.Trans.Control (liftBaseWith)
+import qualified Control.Monad.Trans.Control as Control
 import           System.Posix.Signals
 import qualified System.Posix.User as PosixUser
 #endif
@@ -95,7 +95,7 @@ import qualified System.Posix.User as PosixUser
 -- for this is releasing a lock.  After launching reexecution, the host process becomes
 -- nothing but an manager for the call into docker and thus may not hold the lock.
 reexecWithOptionalContainer
-    :: M env m
+    :: (StackM env m, HasConfig env)
     => Maybe (Path Abs Dir)
     -> Maybe (m ())
     -> IO ()
@@ -106,13 +106,13 @@ reexecWithOptionalContainer mprojectRoot =
     execWithOptionalContainer mprojectRoot getCmdArgs
   where
     getCmdArgs docker envOverride imageInfo isRemoteDocker = do
-        config <- asks getConfig
+        config <- view configL
         deUser <-
             if fromMaybe (not isRemoteDocker) (dockerSetUser docker)
                 then liftIO $ do
                   duUid <- User.getEffectiveUserID
                   duGid <- User.getEffectiveGroupID
-                  duGroups <- User.getGroups
+                  duGroups <- nubOrd <$> User.getGroups
                   duUmask <- Files.setFileCreationMask 0o022
                   -- Only way to get old umask seems to be to change it, so set it back afterward
                   _ <- Files.setFileCreationMask duUmask
@@ -197,7 +197,7 @@ reexecWithOptionalContainer mprojectRoot =
 --
 -- This takes an optional release action just like `reexecWithOptionalContainer`.
 execWithOptionalContainer
-    :: M env m
+    :: (StackM env m, HasConfig env)
     => Maybe (Path Abs Dir)
     -> GetCmdArgs env m
     -> Maybe (m ())
@@ -206,9 +206,9 @@ execWithOptionalContainer
     -> Maybe (m ())
     -> m ()
 execWithOptionalContainer mprojectRoot getCmdArgs mbefore inner mafter mrelease =
-  do config <- asks getConfig
+  do config <- view configL
      inContainer <- getInContainer
-     isReExec <- asks getReExec
+     isReExec <- view reExecL
      if | inContainer && not isReExec && (isJust mbefore || isJust mafter) ->
             throwM OnlyOnHostException
         | inContainer ->
@@ -239,7 +239,7 @@ preventInContainer inner =
         else inner
 
 -- | Run a command in a new Docker container, then exit the process.
-runContainerAndExit :: M env m
+runContainerAndExit :: (StackM env m, HasConfig env)
   => GetCmdArgs env m
   -> Maybe (Path Abs Dir) -- ^ Project root (maybe)
   -> m ()              -- ^ Action to run before
@@ -249,7 +249,7 @@ runContainerAndExit getCmdArgs
                     mprojectRoot
                     before
                     after =
-  do config <- asks getConfig
+  do config <- view configL
      let docker = configDocker config
      envOverride <- getEnvOverride (configPlatform config)
      checkDockerVersion envOverride docker
@@ -259,7 +259,7 @@ runContainerAndExit getCmdArgs
        <*> hIsTerminalDevice stdin
        <*> hIsTerminalDevice stderr
        <*> (parseAbsDir =<< getHomeDirectory)
-     isStdoutTerminal <- asks getTerminal
+     isStdoutTerminal <- view terminalL
      let dockerHost = lookup "DOCKER_HOST" env
          dockerCertPath = lookup "DOCKER_CERT_PATH" env
          bamboo = lookup "bamboo_buildKey" env
@@ -285,7 +285,7 @@ runContainerAndExit getCmdArgs
      sandboxDir <- projectDockerSandboxDir projectRoot
      let ImageConfig {..} = iiConfig
          imageEnvVars = map (break (== '=')) icEnv
-         platformVariant = BS.unpack $ Hash.digestToHexByteString $ hashRepoName image
+         platformVariant = show $ hashRepoName image
          stackRoot = configStackRoot config
          sandboxHomeDir = sandboxDir </> homeDirName
          isTerm = not (dockerDetach docker) &&
@@ -297,16 +297,16 @@ runContainerAndExit getCmdArgs
                          -- This is fixed in Docker 1.9.1, but will leave the workaround
                          -- in place for now, for users who haven't upgraded yet.
                          (isTerm || (isNothing bamboo && isNothing jenkins))
+     hostBinDirPath <- parseAbsDir hostBinDir
      newPathEnv <- augmentPath
-                      [ hostBinDir
-                      , toFilePathNoTrailingSep $ sandboxHomeDir
-                                            </> $(mkRelDir ".local/bin")]
+                      [ hostBinDirPath
+                      , sandboxHomeDir </> $(mkRelDir ".local/bin")]
                       (T.pack <$> lookupImageEnv "PATH" imageEnvVars)
      (cmnd,args,envVars,extraMount) <- getCmdArgs docker envOverride imageInfo isRemoteDocker
      pwd <- getCurrentDir
      liftIO
        (do updateDockerImageLastUsed config iiId (toFilePath projectRoot)
-           mapM_ (ensureDir) [sandboxHomeDir, stackRoot])
+           mapM_ ensureDir [sandboxHomeDir, stackRoot])
      -- Since $HOME is now mounted in the same place in the container we can
      -- just symlink $HOME/.ssh to the right place for the stack docker user
      let sshDir = homeDir </> sshRelDir
@@ -322,6 +322,7 @@ runContainerAndExit getCmdArgs
                  (toFilePathNoTrailingSep (sandboxHomeDir </> sshRelDir))))
      containerID <- (trim . decodeUtf8) <$> readDockerProcess
        envOverride
+       (Just projectRoot)
        (concat
          [["create"
           ,"--net=host"
@@ -363,7 +364,7 @@ runContainerAndExit getCmdArgs
          ,args])
      before
 #ifndef WINDOWS
-     runInBase <- liftBaseWith $ \run -> return (void . run)
+     runInBase <- Control.liftBaseWith $ \run -> return (void . run)
      oldHandlers <- forM [sigINT,sigABRT,sigHUP,sigPIPE,sigTERM,sigUSR1,sigUSR2] $ \sig -> do
        let sigHandler = runInBase $ do
              readProcessNull Nothing envOverride "docker"
@@ -413,14 +414,14 @@ runContainerAndExit getCmdArgs
     sshRelDir = $(mkRelDir ".ssh/")
 
 -- | Clean-up old docker images and containers.
-cleanup :: M env m
+cleanup :: (StackM env m, HasConfig env)
         => CleanupOpts -> m ()
 cleanup opts =
-  do config <- asks getConfig
+  do config <- view configL
      let docker = configDocker config
      envOverride <- getEnvOverride (configPlatform config)
      checkDockerVersion envOverride docker
-     let runDocker = readDockerProcess envOverride
+     let runDocker = readDockerProcess envOverride Nothing
      imagesOut <- runDocker ["images","--no-trunc","-f","dangling=false"]
      danglingImagesOut <- runDocker ["images","--no-trunc","-f","dangling=true"]
      runningContainersOut <- runDocker ["ps","-a","--no-trunc","-f","status=running"]
@@ -479,9 +480,9 @@ cleanup opts =
                             do $logInfo (concatT ["Removing container: '",v,"'"])
                                return ["rm","-f",v]
                         | otherwise -> throwM (InvalidCleanupCommandException line)
-             e <- try (readDockerProcess envOverride args)
+             e <- try (readDockerProcess envOverride Nothing args)
              case e of
-               Left ex@ReadProcessException{} ->
+               Left ex@ProcessFailed{} ->
                  $logError (concatT ["Could not remove: '",v,"': ", show ex])
                Left e' -> throwM e'
                Right _ -> return ()
@@ -655,28 +656,28 @@ inspects :: (MonadIO m, MonadLogger m, MonadBaseControl IO m, MonadCatch m)
 inspects _ [] = return Map.empty
 inspects envOverride images =
   do maybeInspectOut <-
-       try (readDockerProcess envOverride ("inspect" : images))
+       try (readDockerProcess envOverride Nothing ("inspect" : images))
      case maybeInspectOut of
        Right inspectOut ->
          -- filtering with 'isAscii' to workaround @docker inspect@ output containing invalid UTF-8
          case eitherDecode (LBS.pack (filter isAscii (decodeUtf8 inspectOut))) of
            Left msg -> throwM (InvalidInspectOutputException msg)
            Right results -> return (Map.fromList (map (\r -> (iiId r,r)) results))
-       Left (ReadProcessException _ _ _ err)
+       Left (ProcessFailed _ _ _ err)
          | "Error: No such image" `LBS.isPrefixOf` err -> return Map.empty
        Left e -> throwM e
 
 -- | Pull latest version of configured Docker image from registry.
-pull :: M env m => m ()
+pull :: (StackM env m, HasConfig env) => m ()
 pull =
-  do config <- asks getConfig
+  do config <- view configL
      let docker = configDocker config
      envOverride <- getEnvOverride (configPlatform config)
      checkDockerVersion envOverride docker
      pullImage envOverride docker (dockerImage docker)
 
 -- | Pull Docker image from registry.
-pullImage :: (MonadLogger m,MonadIO m,MonadThrow m,MonadBaseControl IO m)
+pullImage :: (MonadLogger m,MonadIO m,MonadThrow m)
           => EnvOverride -> DockerOpts -> String -> m ()
 pullImage envOverride docker image =
   do $logInfo (concatT ["Pulling image from registry: '",image,"'"])
@@ -691,10 +692,21 @@ pullImage envOverride docker image =
                    ,maybe [] (\n -> ["--username=" ++ n]) (dockerRegistryUsername docker)
                    ,maybe [] (\p -> ["--password=" ++ p]) (dockerRegistryPassword docker)
                    ,[takeWhile (/= '/') image]]))
-     e <- try (callProcess (Cmd Nothing "docker" envOverride ["pull",image]))
-     case e of
-       Left (ProcessExitedUnsuccessfully _ _) -> throwM (PullFailedException image)
-       Right () -> return ()
+     -- We redirect the stdout of the process to stderr so that the output
+     -- of @docker pull@ will not interfere with the output of other
+     -- commands when using --auto-docker-pull. See issue #2733.
+     let stdoutToStderr cp = cp
+           { std_out = UseHandle stderr
+           , std_err = UseHandle stderr
+           , std_in = CreatePipe
+           }
+     (Just hin, _, _, ph) <- createProcess' "pullImage" stdoutToStderr $
+       Cmd Nothing "docker" envOverride ["pull",image]
+     liftIO (hClose hin)
+     ec <- liftIO (waitForProcess ph)
+     case ec of
+       ExitSuccess -> return ()
+       ExitFailure _ -> throwM (PullFailedException image)
 
 -- | Check docker version (throws exception if incorrect)
 checkDockerVersion
@@ -703,7 +715,7 @@ checkDockerVersion
 checkDockerVersion envOverride docker =
   do dockerExists <- doesExecutableExist envOverride "docker"
      unless dockerExists (throwM DockerNotInstalledException)
-     dockerVersionOut <- readDockerProcess envOverride ["--version"]
+     dockerVersionOut <- readDockerProcess envOverride Nothing ["--version"]
      case words (decodeUtf8 dockerVersionOut) of
        (_:_:v:_) ->
          case parseVersionFromString (stripVersion v) of
@@ -720,7 +732,7 @@ checkDockerVersion envOverride docker =
        _ -> throwM InvalidVersionOutputException
   where minimumDockerVersion = $(mkVersion "1.6.0")
         prohibitedDockerVersions = []
-        stripVersion v = fst $ break (== '-') $ dropWhileEnd (not . isDigit) v
+        stripVersion v = takeWhile (/= '-') (dropWhileEnd (not . isDigit) v)
 
 -- | Remove the project's Docker sandbox.
 reset :: (MonadIO m, MonadReader env m, HasConfig env)
@@ -854,8 +866,8 @@ removeDirectoryContents path excludeDirs excludeFiles =
 -- process fails.  Logs process's stderr using @$logError@.
 readDockerProcess
     :: (MonadIO m, MonadLogger m, MonadBaseControl IO m, MonadCatch m)
-    => EnvOverride -> [String] -> m BS.ByteString
-readDockerProcess envOverride = readProcessStdout Nothing envOverride "docker"
+    => EnvOverride -> Maybe (Path Abs Dir) -> [String] -> m BS.ByteString
+readDockerProcess envOverride mpwd = readProcessStdout mpwd envOverride "docker"
 
 -- | Name of home directory within docker sandbox.
 homeDirName :: Path Rel Dir
@@ -882,29 +894,6 @@ fromMaybeProjectRoot = fromMaybe (throw CannotDetermineProjectRootException)
 oldSandboxIdEnvVar :: String
 oldSandboxIdEnvVar = "DOCKER_SANDBOX_ID"
 
--- | Command-line argument for "docker"
-dockerCmdName :: String
-dockerCmdName = "docker"
-
-dockerHelpOptName :: String
-dockerHelpOptName = dockerCmdName ++ "-help"
-
--- | Command-line argument for @docker pull@.
-dockerPullCmdName :: String
-dockerPullCmdName = "pull"
-
--- | Command-line argument for @docker cleanup@.
-dockerCleanupCmdName :: String
-dockerCleanupCmdName = "cleanup"
-
--- | Command-line option for @--internal-re-exec-version@.
-reExecArgName :: String
-reExecArgName = "internal-re-exec-version"
-
--- | Platform that Docker containers run
-dockerContainerPlatform :: Platform
-dockerContainerPlatform = Platform X86_64 Linux
-
 -- | Options for 'cleanup'.
 data CleanupOpts = CleanupOpts
   { dcAction                                :: !CleanupAction
@@ -952,143 +941,11 @@ instance FromJSON ImageConfig where
          <$> fmap join (o .:? "Env") .!= []
          <*> fmap join (o .:? "Entrypoint") .!= []
 
--- | Exceptions thrown by Stack.Docker.
-data StackDockerException
-  = DockerMustBeEnabledException
-    -- ^ Docker must be enabled to use the command.
-  | OnlyOnHostException
-    -- ^ Command must be run on host OS (not in a container).
-  | InspectFailedException String
-    -- ^ @docker inspect@ failed.
-  | NotPulledException String
-    -- ^ Image does not exist.
-  | InvalidCleanupCommandException String
-    -- ^ Input to @docker cleanup@ has invalid command.
-  | InvalidImagesOutputException String
-    -- ^ Invalid output from @docker images@.
-  | InvalidPSOutputException String
-    -- ^ Invalid output from @docker ps@.
-  | InvalidInspectOutputException String
-    -- ^ Invalid output from @docker inspect@.
-  | PullFailedException String
-    -- ^ Could not pull a Docker image.
-  | DockerTooOldException Version Version
-    -- ^ Installed version of @docker@ below minimum version.
-  | DockerVersionProhibitedException [Version] Version
-    -- ^ Installed version of @docker@ is prohibited.
-  | BadDockerVersionException VersionRange Version
-    -- ^ Installed version of @docker@ is out of range specified in config file.
-  | InvalidVersionOutputException
-    -- ^ Invalid output from @docker --version@.
-  | HostStackTooOldException Version (Maybe Version)
-    -- ^ Version of @stack@ on host is too old for version in image.
-  | ContainerStackTooOldException Version Version
-    -- ^ Version of @stack@ in container/image is too old for version on host.
-  | CannotDetermineProjectRootException
-    -- ^ Can't determine the project root (where to put docker sandbox).
-  | DockerNotInstalledException
-    -- ^ @docker --version@ failed.
-  | UnsupportedStackExeHostPlatformException
-    -- ^ Using host stack-exe on unsupported platform.
-  deriving (Typeable)
-
--- | Exception instance for StackDockerException.
-instance Exception StackDockerException
-
--- | Show instance for StackDockerException.
-instance Show StackDockerException where
-  show DockerMustBeEnabledException =
-    "Docker must be enabled in your configuration file to use this command."
-  show OnlyOnHostException =
-    "This command must be run on host OS (not in a Docker container)."
-  show (InspectFailedException image) =
-    concat ["'docker inspect' failed for image after pull: ",image,"."]
-  show (NotPulledException image) =
-    concat ["The Docker image referenced by your configuration file"
-           ," has not\nbeen downloaded:\n    "
-           ,image
-           ,"\n\nRun '"
-           ,unwords [stackProgName, dockerCmdName, dockerPullCmdName]
-           ,"' to download it, then try again."]
-  show (InvalidCleanupCommandException line) =
-    concat ["Invalid line in cleanup commands: '",line,"'."]
-  show (InvalidImagesOutputException line) =
-    concat ["Invalid 'docker images' output line: '",line,"'."]
-  show (InvalidPSOutputException line) =
-    concat ["Invalid 'docker ps' output line: '",line,"'."]
-  show (InvalidInspectOutputException msg) =
-    concat ["Invalid 'docker inspect' output: ",msg,"."]
-  show (PullFailedException image) =
-    concat ["Could not pull Docker image:\n    "
-           ,image
-           ,"\nThere may not be an image on the registry for your resolver's LTS version in\n"
-           ,"your configuration file."]
-  show (DockerTooOldException minVersion haveVersion) =
-    concat ["Minimum docker version '"
-           ,versionString minVersion
-           ,"' is required by "
-           ,stackProgName
-           ," (you have '"
-           ,versionString haveVersion
-           ,"')."]
-  show (DockerVersionProhibitedException prohibitedVersions haveVersion) =
-    concat ["These Docker versions are incompatible with "
-           ,stackProgName
-           ," (you have '"
-           ,versionString haveVersion
-           ,"'): "
-           ,intercalate ", " (map versionString prohibitedVersions)
-           ,"."]
-  show (BadDockerVersionException requiredRange haveVersion) =
-    concat ["The version of 'docker' you are using ("
-           ,show haveVersion
-           ,") is outside the required\n"
-           ,"version range specified in stack.yaml ("
-           ,T.unpack (versionRangeText requiredRange)
-           ,")."]
-  show InvalidVersionOutputException =
-    "Cannot get Docker version (invalid 'docker --version' output)."
-  show (HostStackTooOldException minVersion (Just hostVersion)) =
-    concat ["The host's version of '"
-           ,stackProgName
-           ,"' is too old for this Docker image.\nVersion "
-           ,versionString minVersion
-           ," is required; you have "
-           ,versionString hostVersion
-           ,"."]
-  show (HostStackTooOldException minVersion Nothing) =
-    concat ["The host's version of '"
-           ,stackProgName
-           ,"' is too old.\nVersion "
-           ,versionString minVersion
-           ," is required."]
-  show (ContainerStackTooOldException requiredVersion containerVersion) =
-    concat ["The Docker container's version of '"
-           ,stackProgName
-           ,"' is too old.\nVersion "
-           ,versionString requiredVersion
-           ," is required; the container has "
-           ,versionString containerVersion
-           ,"."]
-  show CannotDetermineProjectRootException =
-    "Cannot determine project root directory for Docker sandbox."
-  show DockerNotInstalledException =
-    "Cannot find 'docker' in PATH.  Is Docker installed?"
-  show UnsupportedStackExeHostPlatformException = concat
-    [ "Using host's "
-    , stackProgName
-    , " executable in Docker container is only supported on "
-    , display dockerContainerPlatform
-    , " platform" ]
-
 -- | Function to get command and arguments to run in Docker container
 type GetCmdArgs env m
-   = M env m
+   = (StackM env m, HasConfig env)
   => DockerOpts
   -> EnvOverride
   -> Inspect
   -> Bool
   -> m (FilePath,[String],[(String,String)],[Mount])
-
-type M env m = (MonadIO m,MonadReader env m,MonadLogger m,MonadBaseControl IO m,MonadCatch m
-               ,HasConfig env,HasTerminal env,HasReExec env,HasHttpManager env,MonadMask m)
