@@ -1,3 +1,4 @@
+{-# LANGUAGE ConstraintKinds       #-}
 {-# LANGUAGE DeriveDataTypeable    #-}
 {-# LANGUAGE FlexibleContexts      #-}
 {-# LANGUAGE MultiParamTypeClasses #-}
@@ -16,6 +17,7 @@ module Stack.Fetch
     ( unpackPackages
     , unpackPackageIdents
     , fetchPackages
+    , untar
     , resolvePackages
     , resolvePackagesAllowMissing
     , ResolvedPackage (..)
@@ -23,67 +25,60 @@ module Stack.Fetch
     , withCabalLoader
     ) where
 
-import qualified Codec.Archive.Tar              as Tar
-import qualified Codec.Archive.Tar.Check        as Tar
-import qualified Codec.Archive.Tar.Entry        as Tar
-import           Codec.Compression.GZip         (decompress)
-import           Control.Applicative
-import           Control.Concurrent.Async       (Concurrently (..))
-import           Control.Concurrent.MVar.Lifted (modifyMVar, newMVar)
-import           Control.Concurrent.STM         (TVar, atomically, modifyTVar,
-                                                 newTVarIO, readTVar,
-                                                 readTVarIO, writeTVar)
-import           Control.Exception              (assert)
-import           Control.Exception.Enclosed     (tryIO)
-import           Control.Monad                  (join, liftM, unless, void,
-                                                 when)
-import           Control.Monad.Catch
-import           Control.Monad.IO.Class
-import           Control.Monad.Logger
-import           Control.Monad.Reader           (asks, runReaderT)
-import           Control.Monad.Trans.Control
-import           Control.Monad.Trans.Unlift     (MonadBaseUnlift, askRunBase)
-import "cryptohash" Crypto.Hash                 (SHA512 (..))
-import           Data.ByteString                (ByteString)
-import qualified Data.ByteString                as S
-import qualified Data.ByteString.Lazy           as L
-import           Data.Either                    (partitionEithers)
-import qualified Data.Foldable                  as F
-import           Data.Function                  (fix)
-import qualified Data.Git                       as Git
-import qualified Data.Git.Ref                   as Git
-import qualified Data.Git.Storage               as Git
-import qualified Data.Git.Storage.Object        as Git
-import           Data.List                      (intercalate)
-import           Data.List.NonEmpty             (NonEmpty)
-import qualified Data.List.NonEmpty             as NE
-import           Data.Map                       (Map)
-import qualified Data.Map                       as Map
-import           Data.Maybe                     (maybeToList, catMaybes)
-import           Data.Monoid
-import           Data.Set                       (Set)
-import qualified Data.Set                       as Set
-import           Data.String                    (fromString)
-import qualified Data.Text                      as T
-import           Data.Text.Encoding             (decodeUtf8)
-import           Data.Typeable                  (Typeable)
-import           Data.Word                      (Word64)
-import           Network.HTTP.Download
-import           Path
-import           Path.IO
-import           Prelude -- Fix AMP warning
-import           Stack.GhcPkg
-import           Stack.PackageIndex
-import           Stack.Types
-import qualified System.Directory               as D
-import           System.FilePath                ((<.>))
-import qualified System.FilePath                as FP
-import           System.IO                      (IOMode (ReadMode),
-                                                 SeekMode (AbsoluteSeek), hSeek,
-                                                 withBinaryFile, openBinaryFile,
-                                                 hClose)
-import           System.PosixCompat             (setFileMode)
-import           Text.EditDistance              as ED
+import qualified    Codec.Archive.Tar as Tar
+import qualified    Codec.Archive.Tar.Check as Tar
+import qualified    Codec.Archive.Tar.Entry as Tar
+import              Codec.Compression.GZip (decompress)
+import              Control.Applicative
+import              Control.Concurrent.Async (Concurrently (..))
+import              Control.Concurrent.MVar.Lifted (modifyMVar, newMVar)
+import              Control.Concurrent.STM
+import              Control.Exception (assert)
+import              Control.Monad (join, liftM, unless, void, when)
+import              Control.Monad.Catch
+import              Control.Monad.IO.Class
+import              Control.Monad.Logger
+import              Control.Monad.Reader (ask, runReaderT)
+import              Control.Monad.Trans.Control
+import              Control.Monad.Trans.Unlift (MonadBaseUnlift, askRunBase)
+import              Crypto.Hash (SHA256 (..))
+import              Data.ByteString (ByteString)
+import qualified    Data.ByteString as S
+import qualified    Data.ByteString.Lazy as L
+import              Data.Either (partitionEithers)
+import qualified    Data.Foldable as F
+import              Data.Function (fix)
+import qualified    Data.HashMap.Strict as HashMap
+import              Data.List (intercalate)
+import              Data.List.NonEmpty (NonEmpty)
+import qualified    Data.List.NonEmpty as NE
+import              Data.Map (Map)
+import qualified    Data.Map as Map
+import              Data.Maybe (maybeToList, catMaybes, isJust)
+import              Data.Monoid
+import              Data.Set (Set)
+import qualified    Data.Set as Set
+import qualified    Data.Text as T
+import              Data.Text.Encoding (encodeUtf8, decodeUtf8)
+import              Data.Text.Metrics
+import              Data.Typeable (Typeable)
+import              Data.Word (Word64)
+import              Network.HTTP.Download
+import              Path
+import              Path.Extra (toFilePathNoTrailingSep)
+import              Path.IO
+import              Prelude -- Fix AMP warning
+import              Stack.PackageIndex
+import              Stack.Types.BuildPlan
+import              Stack.Types.Config
+import              Stack.Types.PackageIdentifier
+import              Stack.Types.PackageIndex
+import              Stack.Types.PackageName
+import              Stack.Types.Version
+import              System.FilePath ((<.>))
+import qualified    System.FilePath as FP
+import              System.IO
+import              System.PosixCompat (setFileMode)
 
 type PackageCaches = Map PackageIdentifier (PackageIndex, PackageCache)
 
@@ -125,12 +120,11 @@ instance Show FetchException where
         (if null suggestions then "" else "\n" ++ suggestions)
 
 -- | Fetch packages into the cache without unpacking
-fetchPackages :: (MonadIO m, MonadBaseControl IO m, MonadReader env m, HasHttpManager env, HasConfig env, MonadMask m, MonadLogger m)
-              => EnvOverride
-              -> Set PackageIdentifier
+fetchPackages :: (StackMiniM env m, HasConfig env)
+              => Set PackageIdentifier
               -> m ()
-fetchPackages menv idents' = do
-    resolved <- resolvePackages menv idents Set.empty
+fetchPackages idents' = do
+    resolved <- resolvePackages Nothing idents Set.empty
     ToFetchResult toFetch alreadyUnpacked <- getToFetch Nothing resolved
     assert (Map.null alreadyUnpacked) (return ())
     nowUnpacked <- fetchPackages' Nothing toFetch
@@ -141,18 +135,18 @@ fetchPackages menv idents' = do
     idents = Map.fromList $ map (, Nothing) $ Set.toList idents'
 
 -- | Intended to work for the command line command.
-unpackPackages :: (MonadIO m, MonadBaseControl IO m, MonadReader env m, HasHttpManager env, HasConfig env, MonadMask m, MonadLogger m)
-               => EnvOverride
+unpackPackages :: (StackMiniM env m, HasConfig env)
+               => Maybe MiniBuildPlan -- ^ when looking up by name, take from this build plan
                -> FilePath -- ^ destination
                -> [String] -- ^ names or identifiers
                -> m ()
-unpackPackages menv dest input = do
+unpackPackages mMiniBuildPlan dest input = do
     dest' <- resolveDir' dest
     (names, idents) <- case partitionEithers $ map parse input of
         ([], x) -> return $ partitionEithers x
         (errs, _) -> throwM $ CouldNotParsePackageSelectors errs
-    resolved <- resolvePackages menv
-        (Map.fromList $ map (, Nothing) idents)
+    resolved <- resolvePackages mMiniBuildPlan
+        (Map.fromList idents)
         (Set.fromList names)
     ToFetchResult toFetch alreadyUnpacked <- getToFetch (Just dest') resolved
     unless (Map.null alreadyUnpacked) $
@@ -171,82 +165,138 @@ unpackPackages menv dest input = do
             Right x -> Right $ Left x
             Left _ ->
                 case parsePackageIdentifierFromString s of
-                    Left _ -> Left s
-                    Right x -> Right $ Right x
+                    Right x -> Right $ Right (x, Nothing)
+                    Left _ -> maybe (Left s) (Right . Right) $ do
+                      (identS, '@':revisionS) <- return $ break (== '@') s
+                      Right ident <- return $ parsePackageIdentifierFromString identS
+                      hash <- T.stripPrefix "gitsha1:" $ T.pack revisionS
+                      Just (ident, Just $ GitSHA1 $ encodeUtf8 hash)
 
 -- | Ensure that all of the given package idents are unpacked into the build
 -- unpack directory, and return the paths to all of the subdirectories.
 unpackPackageIdents
-    :: (MonadBaseControl IO m, MonadIO m, MonadReader env m, HasHttpManager env, HasConfig env, MonadMask m, MonadLogger m)
-    => EnvOverride
-    -> Path Abs Dir -- ^ unpack directory
+    :: (StackMiniM env m, HasConfig env)
+    => Path Abs Dir -- ^ unpack directory
     -> Maybe (Path Rel Dir) -- ^ the dist rename directory, see: https://github.com/fpco/stack/issues/157
     -> Map PackageIdentifier (Maybe GitSHA1)
     -> m (Map PackageIdentifier (Path Abs Dir))
-unpackPackageIdents menv unpackDir mdistDir idents = do
-    resolved <- resolvePackages menv idents Set.empty
+unpackPackageIdents unpackDir mdistDir idents = do
+    resolved <- resolvePackages Nothing idents Set.empty
     ToFetchResult toFetch alreadyUnpacked <- getToFetch (Just unpackDir) resolved
     nowUnpacked <- fetchPackages' mdistDir toFetch
     return $ alreadyUnpacked <> nowUnpacked
 
 data ResolvedPackage = ResolvedPackage
-    { rpCache :: !PackageCache
+    { rpIdent :: !PackageIdentifier
+    , rpCache :: !PackageCache
     , rpIndex :: !PackageIndex
-    , rpGitSHA1 :: !(Maybe GitSHA1)
     }
+    deriving Show
 
 -- | Resolve a set of package names and identifiers into @FetchPackage@ values.
-resolvePackages :: (MonadIO m, MonadReader env m, HasHttpManager env, HasConfig env, MonadLogger m, MonadBaseControl IO m, MonadCatch m)
-                => EnvOverride
+resolvePackages :: (StackMiniM env m, HasConfig env)
+                => Maybe MiniBuildPlan -- ^ when looking up by name, take from this build plan
                 -> Map PackageIdentifier (Maybe GitSHA1)
                 -> Set PackageName
-                -> m (Map PackageIdentifier ResolvedPackage)
-resolvePackages menv idents0 names0 = do
+                -> m [ResolvedPackage]
+resolvePackages mMiniBuildPlan idents0 names0 = do
     eres <- go
     case eres of
         Left _ -> do
-            updateAllIndices menv
+            updateAllIndices
             go >>= either throwM return
         Right x -> return x
   where
-    go = r <$> resolvePackagesAllowMissing idents0 names0
+    go = r <$> resolvePackagesAllowMissing mMiniBuildPlan idents0 names0
     r (missingNames, missingIdents, idents)
       | not $ Set.null missingNames  = Left $ UnknownPackageNames       missingNames
       | not $ Set.null missingIdents = Left $ UnknownPackageIdentifiers missingIdents ""
       | otherwise                    = Right idents
 
 resolvePackagesAllowMissing
-    :: (MonadIO m, MonadReader env m, HasHttpManager env, HasConfig env, MonadLogger m, MonadBaseControl IO m, MonadCatch m)
-    => Map PackageIdentifier (Maybe GitSHA1)
+    :: (StackMiniM env m, HasConfig env)
+    => Maybe MiniBuildPlan -- ^ when looking up by name, take from this build plan
+    -> Map PackageIdentifier (Maybe GitSHA1)
     -> Set PackageName
-    -> m (Set PackageName, Set PackageIdentifier, Map PackageIdentifier ResolvedPackage)
-resolvePackagesAllowMissing idents0 names0 = do
-    caches <- getPackageCaches
-    let versions = Map.fromListWith max $ map toTuple $ Map.keys caches
-        (missingNames, idents1) = partitionEithers $ map
-            (\name -> maybe (Left name ) (Right . PackageIdentifier name)
-                (Map.lookup name versions))
-            (Set.toList names0)
-        (missingIdents, resolved) = partitionEithers $ map (goIdent caches)
-                                  $ Map.toList
-                                  $ idents0 <> Map.fromList (map (, Nothing) idents1)
-    return (Set.fromList missingNames, Set.fromList missingIdents, Map.fromList resolved)
+    -> m (Set PackageName, Set PackageIdentifier, [ResolvedPackage])
+resolvePackagesAllowMissing mMiniBuildPlan idents0 names0 = do
+    (res1, res2, resolved) <- inner
+    if any (isJust . snd) resolved
+        then do
+            $logInfo "Missing some cabal revision files, updating indices"
+            updateAllIndices
+            (res1', res2', resolved') <- inner
+
+            -- Print an error message if any SHAs are still missing.
+            F.forM_ resolved' $ \(rp, missing) -> F.forM_ missing $ \(GitSHA1 sha) ->
+                $logWarn $ mconcat
+                    [ "Did not find .cabal file for "
+                    , T.pack $ packageIdentifierString $ rpIdent rp
+                    , " with SHA of "
+                    , decodeUtf8 sha
+                    , " in tarball-based cache"
+                    ]
+
+            return (res1', res2', map fst resolved')
+        else return (res1, res2, map fst resolved)
   where
-    goIdent caches (ident, mgitsha) =
+    inner = do
+        (caches, shaCaches) <- getPackageCaches
+
+        let versions = Map.fromListWith max $ map toTuple $ Map.keys caches
+
+            getNamed :: PackageName -> Maybe (PackageIdentifier, Maybe GitSHA1)
+            getNamed =
+                case mMiniBuildPlan of
+                    Nothing -> getNamedFromIndex
+                    Just mbp -> getNamedFromBuildPlan mbp
+
+            getNamedFromBuildPlan mbp name = do
+                mpi <- Map.lookup name $ mbpPackages mbp
+                Just (PackageIdentifier name (mpiVersion mpi), mpiGitSHA1 mpi)
+            getNamedFromIndex name = fmap
+                (\ver -> (PackageIdentifier name ver, Nothing))
+                (Map.lookup name versions)
+
+            (missingNames, idents1) = partitionEithers $ map
+                (\name -> maybe (Left name) Right (getNamed name))
+                (Set.toList names0)
+        let (missingIdents, resolved) = partitionEithers $ map (goIdent caches shaCaches)
+                                    $ Map.toList
+                                    $ idents0 <> Map.fromList idents1
+        return (Set.fromList missingNames, Set.fromList missingIdents, resolved)
+
+    goIdent caches shaCaches (ident, mgitsha) =
         case Map.lookup ident caches of
             Nothing -> Left ident
-            Just (index, cache) -> Right (ident, ResolvedPackage
-                { rpCache = cache
-                , rpIndex = index
-                , rpGitSHA1 = mgitsha
-                })
+            Just (index, cache) ->
+                let (index', cache', missingGitSHA) =
+                      case mgitsha of
+                        Nothing -> (index, cache, mgitsha)
+                        Just gitsha ->
+                            case HashMap.lookup gitsha shaCaches of
+                                Just (index'', offsetSize) ->
+                                        ( index''
+                                        , cache { pcOffsetSize = offsetSize }
+                                        -- we already got the info
+                                        -- about this SHA, don't do
+                                        -- any lookups later
+                                        , Nothing
+                                        )
+                                -- Index using HTTP, so we're missing the Git SHA
+                                Nothing -> (index, cache, mgitsha)
+                 in Right (ResolvedPackage
+                        { rpIdent = ident
+                        , rpCache = cache'
+                        , rpIndex = index'
+                        }, missingGitSHA)
 
 data ToFetch = ToFetch
     { tfTarball :: !(Path Abs File)
     , tfDestDir :: !(Maybe (Path Abs Dir))
     , tfUrl     :: !T.Text
     , tfSize    :: !(Maybe Word64)
-    , tfSHA512  :: !(Maybe ByteString)
+    , tfSHA256  :: !(Maybe ByteString)
     , tfCabal   :: !ByteString
     -- ^ Contents of the .cabal file
     }
@@ -258,56 +308,32 @@ data ToFetchResult = ToFetchResult
 
 -- | Add the cabal files to a list of idents with their caches.
 withCabalFiles
-    :: (MonadMask m, MonadIO m, MonadLogger m, MonadReader env m, HasConfig env)
+    :: (StackMiniM env m, HasConfig env)
     => IndexName
-    -> [(PackageIdentifier, PackageCache, Maybe GitSHA1, a)]
+    -> [(ResolvedPackage, a)]
     -> (PackageIdentifier -> a -> ByteString -> IO b)
     -> m [b]
 withCabalFiles name pkgs f = do
     indexPath <- configPackageIndex name
-    mgitRepo <- configPackageIndexRepo name
     bracket
         (liftIO $ openBinaryFile (toFilePath indexPath) ReadMode)
-        (liftIO . hClose) $ \h ->
-            let inner mgit = mapM (goPkg h mgit) pkgs
-             in case mgitRepo of
-                    Nothing -> inner Nothing
-                    Just repo -> bracket
-                        (liftIO $ Git.openRepo
-                                $ fromString
-                                $ toFilePath repo FP.</> ".git")
-                        (liftIO . Git.closeRepo)
-                        (inner . Just)
+        (liftIO . hClose) $ \h -> mapM (goPkg h) pkgs
   where
-    goPkg h (Just git) (ident, pc, Just (GitSHA1 sha), tf) = do
-        let ref = Git.fromHex sha
-        mobj <- liftIO $ tryIO $ Git.getObject git ref True
-        case mobj of
-            Right (Just (Git.ObjBlob (Git.Blob bs))) -> liftIO $ f ident tf (L.toStrict bs)
-            -- fallback when the appropriate SHA isn't found
-            e -> do
-                $logWarn $ mconcat
-                    [ "Did not find .cabal file for "
-                    , T.pack $ packageIdentifierString ident
-                    , " with Git SHA of "
-                    , decodeUtf8 sha
-                    , "\n"
-                    , T.pack $ show e
-                    ]
-                goPkg h Nothing (ident, pc, Nothing, tf)
-    goPkg h _mgit (ident, pc, _mgitsha, tf) = liftIO $ do
-        hSeek h AbsoluteSeek $ fromIntegral $ pcOffset pc
-        cabalBS <- S.hGet h $ fromIntegral $ pcSize pc
-        f ident tf cabalBS
+    goPkg h (ResolvedPackage ident pc _index, tf) = do
+        -- Did not find warning for tarballs is handled above
+        let OffsetSize offset size = pcOffsetSize pc
+        liftIO $ do
+            hSeek h AbsoluteSeek $ fromIntegral offset
+            cabalBS <- S.hGet h $ fromIntegral size
+            f ident tf cabalBS
 
 -- | Provide a function which will load up a cabal @ByteString@ from the
 -- package indices.
 withCabalLoader
-    :: (MonadIO m, MonadReader env m, HasConfig env, MonadLogger m, HasHttpManager env, MonadBaseUnlift IO m, MonadMask m)
-    => EnvOverride
-    -> ((PackageIdentifier -> IO ByteString) -> m a)
+    :: (StackMiniM env m, HasConfig env, MonadBaseUnlift IO m)
+    => ((PackageIdentifier -> IO ByteString) -> m a)
     -> m a
-withCabalLoader menv inner = do
+withCabalLoader inner = do
     env <- ask
 
     -- Want to try updating the index once during a single run for missing
@@ -325,7 +351,7 @@ withCabalLoader menv inner = do
     let doLookup :: PackageIdentifier
                  -> IO ByteString
         doLookup ident = do
-            caches <- loadCaches
+            (caches, _gitSHACaches) <- loadCaches
             eres <- unlift $ lookupPackageIdentifierExact ident env caches
             case eres of
                 Just bs -> return bs
@@ -339,7 +365,7 @@ withCabalLoader menv inner = do
                                   Just cs -> "Perhaps you meant " <>
                                     orSeparated cs <> "?"
                             Just cs -> "Possible candidates: " <>
-                              commaSeparated (NE.map packageIdentifierString cs)
+                              commaSeparated (NE.map packageIdentifierText cs)
                               <> "."
                     join $ modifyMVar updateRef $ \toUpdate ->
                         if toUpdate then do
@@ -350,17 +376,17 @@ withCabalLoader menv inner = do
                                     , " in your package indices.\n"
                                     , "Updating and trying again."
                                     ]
-                                updateAllIndices menv
+                                updateAllIndices
                                 _ <- getPackageCaches
                                 return ()
                             return (False, doLookup ident)
                         else return (toUpdate,
                                      throwM $ UnknownPackageIdentifiers
-                                       (Set.singleton ident) suggestions)
+                                       (Set.singleton ident) (T.unpack suggestions))
     inner doLookup
 
 lookupPackageIdentifierExact
-  :: (MonadMask m, MonadIO m, MonadLogger m, HasConfig env)
+  :: (StackMiniM env m, HasConfig env)
   => PackageIdentifier
   -> env
   -> PackageCaches
@@ -370,7 +396,12 @@ lookupPackageIdentifierExact ident env caches =
         Nothing -> return Nothing
         Just (index, cache) -> do
             [bs] <- flip runReaderT env
-                  $ withCabalFiles (indexName index) [(ident, cache, Nothing, ())]
+                  $ withCabalFiles (indexName index)
+                        [(ResolvedPackage
+                            { rpIdent = ident
+                            , rpCache = cache
+                            , rpIndex = index
+                            }, ())]
                   $ \_ _ bs -> return bs
             return $ Just bs
 
@@ -395,29 +426,32 @@ fuzzyLookupCandidates (PackageIdentifier name ver) caches =
 typoCorrectionCandidates
   :: PackageIdentifier
   -> PackageCaches
-  -> Maybe (NonEmpty String)
+  -> Maybe (NonEmpty T.Text)
 typoCorrectionCandidates ident =
-  let getName = packageNameString . packageIdentifierName
+  let getName = packageNameText . packageIdentifierName
       name    = getName ident
   in  NE.nonEmpty
-    . Map.keys
-    . Map.filterWithKey (const . (== 1) . damerauLevenshtein name)
-    . Map.mapKeys getName
+    . take 10
+    . map snd
+    . filter (\(distance, _) -> distance < 4)
+    . map (\(k, _) -> (damerauLevenshtein name (getName k), getName k))
+    . Map.toList
 
 -- | Figure out where to fetch from.
-getToFetch :: (MonadMask m, MonadLogger m, MonadIO m, MonadReader env m, HasConfig env)
+getToFetch :: (StackMiniM env m, HasConfig env)
            => Maybe (Path Abs Dir) -- ^ directory to unpack into, @Nothing@ means no unpack
-           -> Map PackageIdentifier ResolvedPackage
+           -> [ResolvedPackage]
            -> m ToFetchResult
 getToFetch mdest resolvedAll = do
-    (toFetch0, unpacked) <- liftM partitionEithers $ mapM checkUnpacked $ Map.toList resolvedAll
+    (toFetch0, unpacked) <- liftM partitionEithers $ mapM checkUnpacked resolvedAll
     toFetch1 <- mapM goIndex $ Map.toList $ Map.fromListWith (++) toFetch0
     return ToFetchResult
         { tfrToFetch = Map.unions toFetch1
         , tfrAlreadyUnpacked = Map.fromList unpacked
         }
   where
-    checkUnpacked (ident, resolved) = do
+    checkUnpacked resolved = do
+        let ident = rpIdent resolved
         dirRel <- parseRelDir $ packageIdentifierString ident
         let mdestDir = (</> dirRel) <$> mdest
         mexists <-
@@ -433,14 +467,14 @@ getToFetch mdest resolvedAll = do
                     d = pcDownload $ rpCache resolved
                     targz = T.pack $ packageIdentifierString ident ++ ".tar.gz"
                 tarball <- configPackageTarball (indexName index) ident
-                return $ Left (indexName index, [(ident, rpCache resolved, rpGitSHA1 resolved, ToFetch
+                return $ Left (indexName index, [(resolved, ToFetch
                     { tfTarball = tarball
                     , tfDestDir = mdestDir
-                    , tfUrl = case d of
-                        Just d' -> decodeUtf8 $ pdUrl d'
-                        Nothing -> indexDownloadPrefix index <> targz
+                    , tfUrl = case fmap pdUrl d of
+                        Just url | not (S.null url) -> decodeUtf8 url
+                        _ -> indexDownloadPrefix index <> targz
                     , tfSize = fmap pdSize d
-                    , tfSHA512 = fmap pdSHA512 d
+                    , tfSHA256 = fmap pdSHA256 d
                     , tfCabal = S.empty -- filled in by goIndex
                     })])
 
@@ -463,12 +497,12 @@ getToFetch mdest resolvedAll = do
 -- @
 --
 -- Since 0.1.0.0
-fetchPackages' :: (MonadIO m,MonadReader env m,HasHttpManager env,HasConfig env,MonadLogger m,MonadThrow m,MonadBaseControl IO m)
+fetchPackages' :: (StackMiniM env m, HasConfig env)
                => Maybe (Path Rel Dir) -- ^ the dist rename directory, see: https://github.com/fpco/stack/issues/157
                -> Map PackageIdentifier ToFetch
                -> m (Map PackageIdentifier (Path Abs Dir))
 fetchPackages' mdistDir toFetchAll = do
-    connCount <- asks $ configConnectionCount . getConfig
+    connCount <- view $ configL.to configConnectionCount
     outputVar <- liftIO $ newTVarIO Map.empty
 
     runInBase <- liftBaseWith $ \run -> return (void . run)
@@ -479,19 +513,19 @@ fetchPackages' mdistDir toFetchAll = do
 
     liftIO $ readTVarIO outputVar
   where
-    go :: (MonadIO m,MonadThrow m,MonadLogger m,MonadReader env m,HasHttpManager env)
+    go :: (MonadIO m,MonadThrow m,MonadLogger m)
        => TVar (Map PackageIdentifier (Path Abs Dir))
        -> (m () -> IO ())
        -> (PackageIdentifier, ToFetch)
        -> m ()
     go outputVar runInBase (ident, toFetch) = do
-        req <- parseUrl $ T.unpack $ tfUrl toFetch
+        req <- parseUrlThrow $ T.unpack $ tfUrl toFetch
         let destpath = tfTarball toFetch
 
-        let toHashCheck bs = HashCheck SHA512 (CheckHexDigestByteString bs)
+        let toHashCheck bs = HashCheck SHA256 (CheckHexDigestByteString bs)
         let downloadReq = DownloadRequest
                 { drRequest = req
-                , drHashChecks = map toHashCheck $ maybeToList (tfSHA512 toFetch)
+                , drHashChecks = map toHashCheck $ maybeToList (tfSHA256 toFetch)
                 , drLengthCheck = fromIntegral <$> tfSize toFetch
                 , drRetryPolicy = drRetryPolicyDefault
                 }
@@ -499,54 +533,30 @@ fetchPackages' mdistDir toFetchAll = do
                 liftIO $ runInBase $ $logInfo $ packageIdentifierText ident <> ": download"
         _ <- verifiedDownload downloadReq destpath progressSink
 
-        let fp = toFilePath destpath
+        identStrP <- parseRelDir $ packageIdentifierString ident
 
         F.forM_ (tfDestDir toFetch) $ \destDir -> do
-            let dest = toFilePath $ parent destDir
-                innerDest = toFilePath destDir
+            let innerDest = toFilePath destDir
 
-            liftIO $ ensureDir (parent destDir)
-
-            liftIO $ withBinaryFile fp ReadMode $ \h -> do
-                -- Avoid using L.readFile, which is more likely to leak
-                -- resources
-                lbs <- L.hGetContents h
-                let entries = fmap (either wrap wrap)
-                            $ Tar.checkTarbomb identStr
-                            $ Tar.read $ decompress lbs
-                    wrap :: Exception e => e -> FetchException
-                    wrap = Couldn'tReadPackageTarball fp . toException
-                    identStr = packageIdentifierString ident
-
-                    getPerms :: Tar.Entry -> (FilePath, Tar.Permissions)
-                    getPerms e = (dest FP.</> Tar.fromTarPath (Tar.entryTarPath e),
-                                  Tar.entryPermissions e)
-
-                    filePerms :: [(FilePath, Tar.Permissions)]
-                    filePerms = catMaybes $ Tar.foldEntries (\e -> (:) (Just $ getPerms e))
-                                                            [] (const []) entries
-                Tar.unpack dest entries
-                -- Reset file permissions as they were in the tarball
-                mapM_ (\(fp', perm) -> setFileMode
-                    (FP.dropTrailingPathSeparator fp')
-                    perm) filePerms
+            unexpectedEntries <- liftIO $ untar destpath identStrP (parent destDir)
 
+            liftIO $ do
                 case mdistDir of
                     Nothing -> return ()
                     -- See: https://github.com/fpco/stack/issues/157
                     Just distDir -> do
-                        let inner = dest FP.</> identStr
-                            oldDist = inner FP.</> "dist"
-                            newDist = inner FP.</> toFilePath distDir
-                        exists <- D.doesDirectoryExist oldDist
+                        let inner = parent destDir </> identStrP
+                            oldDist = inner </> $(mkRelDir "dist")
+                            newDist = inner </> distDir
+                        exists <- doesDirExist oldDist
                         when exists $ do
                             -- Previously used takeDirectory, but that got confused
                             -- by trailing slashes, see:
                             -- https://github.com/commercialhaskell/stack/issues/216
                             --
                             -- Instead, use Path which is a bit more resilient
-                            ensureDir . parent =<< parseAbsDir newDist
-                            D.renameDirectory oldDist newDist
+                            ensureDir $ parent newDist
+                            renameDir oldDist newDist
 
                 let cabalFP =
                         innerDest FP.</>
@@ -556,6 +566,71 @@ fetchPackages' mdistDir toFetchAll = do
 
                 atomically $ modifyTVar outputVar $ Map.insert ident destDir
 
+            F.forM_ unexpectedEntries $ \(path, entryType) ->
+                $logWarn $ "Unexpected entry type " <> entryType <> " for entry " <> T.pack path
+
+-- | Internal function used to unpack tarball.
+--
+-- Takes a path to a .tar.gz file, the name of the directory it should contain,
+-- and a destination folder to extract the tarball into. Returns unexpected
+-- entries, as pairs of paths and descriptions.
+untar :: forall b1 b2. Path b1 File -> Path Rel Dir -> Path b2 Dir -> IO [(FilePath, T.Text)]
+untar tarPath expectedTarFolder destDirParent = do
+  ensureDir destDirParent
+  withBinaryFile (toFilePath tarPath) ReadMode $ \h -> do
+                -- Avoid using L.readFile, which is more likely to leak
+                -- resources
+                lbs <- L.hGetContents h
+                let rawEntries = fmap (either wrap wrap)
+                            $ Tar.checkTarbomb (toFilePathNoTrailingSep expectedTarFolder)
+                            $ Tar.read $ decompress lbs
+
+                    filterEntries
+                      :: Monoid w => (Tar.Entry -> (Bool, w))
+                         -> Tar.Entries b -> (Tar.Entries b, w)
+                    -- Allow collecting warnings, Writer-monad style.
+                    filterEntries f =
+                      Tar.foldEntries
+                        (\e -> let (res, w) = f e in
+                            \(rest, wOld) -> ((if res then Tar.Next e else id) rest, wOld <> w))
+                        (Tar.Done, mempty)
+                        (\err -> (Tar.Fail err, mempty))
+
+                    extractableEntry e =
+                      case Tar.entryContent e of
+                        Tar.NormalFile _ _ -> (True, [])
+                        Tar.Directory -> (True, [])
+                        Tar.SymbolicLink _ -> (True, [])
+                        Tar.HardLink _ -> (True, [])
+                        Tar.OtherEntryType 'g' _ _ -> (False, [])
+                        Tar.OtherEntryType 'x' _ _ -> (False, [])
+                        Tar.CharacterDevice _ _ -> (False, [(path, "character device")])
+                        Tar.BlockDevice _ _ -> (False, [(path, "block device")])
+                        Tar.NamedPipe -> (False, [(path, "named pipe")])
+                        Tar.OtherEntryType code _ _ -> (False, [(path, "other entry type with code " <> T.pack (show code))])
+                        where
+                          path = Tar.fromTarPath $ Tar.entryTarPath e
+                    (entries, unexpectedEntries) = filterEntries extractableEntry rawEntries
+
+                    wrap :: Exception e => e -> FetchException
+                    wrap = Couldn'tReadPackageTarball (toFilePath tarPath) . toException
+
+                    getPerms :: Tar.Entry -> (FilePath, Tar.Permissions)
+                    getPerms e = (toFilePath destDirParent FP.</> Tar.fromTarPath (Tar.entryTarPath e),
+                                  Tar.entryPermissions e)
+
+                    filePerms :: [(FilePath, Tar.Permissions)]
+                    filePerms = catMaybes $ Tar.foldEntries (\e -> (:) (Just $ getPerms e))
+                                                            [] (const []) entries
+                Tar.unpack (toFilePath destDirParent) entries
+                -- Reset file permissions as they were in the tarball, but only
+                -- for extracted entries (whence filterEntries extractableEntry above).
+                -- See https://github.com/commercialhaskell/stack/issues/2361
+                mapM_ (\(fp, perm) -> setFileMode
+                    (FP.dropTrailingPathSeparator fp)
+                    perm) filePerms
+                return unexpectedEntries
+
 parMapM_ :: (F.Foldable f,MonadIO m,MonadBaseControl IO m)
          => Int
          -> (a -> m ())
@@ -581,14 +656,11 @@ parMapM_ cnt f xs0 = do
         workers i = Concurrently worker *> workers (i - 1)
     liftIO $ runConcurrently $ workers cnt
 
-damerauLevenshtein :: String -> String -> Int
-damerauLevenshtein = ED.restrictedDamerauLevenshteinDistance ED.defaultEditCosts
-
-orSeparated :: NonEmpty String -> String
+orSeparated :: NonEmpty T.Text -> T.Text
 orSeparated xs
   | NE.length xs == 1 = NE.head xs
   | NE.length xs == 2 = NE.head xs <> " or " <> NE.last xs
-  | otherwise = intercalate ", " (NE.init xs) <> ", or " <> NE.last xs
+  | otherwise = T.intercalate ", " (NE.init xs) <> ", or " <> NE.last xs
 
-commaSeparated :: NonEmpty String -> String
+commaSeparated :: NonEmpty T.Text -> T.Text
 commaSeparated = F.fold . NE.intersperse ", "
