@@ -1,3 +1,4 @@
+{-# LANGUAGE ConstraintKinds       #-}
 {-# LANGUAGE FlexibleContexts      #-}
 {-# LANGUAGE MultiParamTypeClasses #-}
 {-# LANGUAGE OverloadedStrings     #-}
@@ -8,13 +9,11 @@ module Stack.Init
     ) where
 
 import           Control.Exception               (assert)
-import           Control.Exception.Enclosed      (catchAny)
+import           Control.Exception.Safe          (catchAny)
 import           Control.Monad
-import           Control.Monad.Catch             (MonadMask, throwM)
+import           Control.Monad.Catch             (throwM)
 import           Control.Monad.IO.Class
 import           Control.Monad.Logger
-import           Control.Monad.Reader            (MonadReader, asks)
-import           Control.Monad.Trans.Control     (MonadBaseControl)
 import qualified Data.ByteString.Builder         as B
 import qualified Data.ByteString.Char8           as BC
 import qualified Data.ByteString.Lazy            as L
@@ -22,9 +21,9 @@ import qualified Data.Foldable                   as F
 import           Data.Function                   (on)
 import qualified Data.HashMap.Strict             as HM
 import qualified Data.IntMap                     as IntMap
-import           Data.List                       ( intercalate, intersect
-                                                 , maximumBy)
-import           Data.List.NonEmpty              (NonEmpty(..))
+import           Data.List                       (intercalate, intersect,
+                                                  maximumBy)
+import           Data.List.NonEmpty              (NonEmpty (..))
 import qualified Data.List.NonEmpty              as NonEmpty
 import           Data.Map                        (Map)
 import qualified Data.Map                        as Map
@@ -35,8 +34,8 @@ import qualified Data.Yaml                       as Yaml
 import qualified Distribution.PackageDescription as C
 import qualified Distribution.Text               as C
 import qualified Distribution.Version            as C
-import           Network.HTTP.Client.Conduit     (HasHttpManager)
 import           Path
+import           Path.Extra                      (toFilePathNoTrailingSep)
 import           Path.IO
 import qualified Paths_stack                     as Meta
 import           Stack.BuildPlan
@@ -44,17 +43,20 @@ import           Stack.Config                    (getSnapshots,
                                                   makeConcreteResolver)
 import           Stack.Constants
 import           Stack.Solver
-import           Stack.Types
-import           Stack.Types.Internal            (HasLogLevel, HasReExec,
-                                                  HasTerminal)
+import           Stack.Types.Build
+import           Stack.Types.BuildPlan
+import           Stack.Types.Config
+import           Stack.Types.FlagName
+import           Stack.Types.PackageName
+import           Stack.Types.Resolver
+import           Stack.Types.StackT              (StackM)
+import           Stack.Types.StringError
+import           Stack.Types.Version
 import qualified System.FilePath                 as FP
 
 -- | Generate stack.yaml
 initProject
-    :: ( MonadBaseControl IO m, MonadIO m, MonadLogger m, MonadMask m
-       , MonadReader env m, HasConfig env , HasGHCVariant env
-       , HasHttpManager env , HasLogLevel env , HasReExec env
-       , HasTerminal env)
+    :: (StackM env m, HasConfig env, HasGHCVariant env)
     => WhichSolverCmd
     -> Path Abs Dir
     -> InitOpts
@@ -66,10 +68,11 @@ initProject whichCmd currDir initOpts mresolver = do
     reldest <- toFilePath `liftM` makeRelativeToCurrentDir dest
 
     exists <- doesFileExist dest
-    when (not (forceOverwrite initOpts) && exists) $ do
-        error ("Stack configuration file " <> reldest <>
-               " exists, use 'stack solver' to fix the existing config file or \
-               \'--force' to overwrite it.")
+    when (not (forceOverwrite initOpts) && exists) $
+        throwString
+            ("Error: Stack configuration file " <> reldest <>
+             " exists, use 'stack solver' to fix the existing config file or \
+             \'--force' to overwrite it.")
 
     dirs <- mapM (resolveDir' . T.unpack) (searchDirs initOpts)
     let noPkgMsg =  "In order to init, you should have an existing .cabal \
@@ -85,26 +88,25 @@ initProject whichCmd currDir initOpts mresolver = do
 
     let ignored = Map.difference bundle rbundle
         dupPkgMsg
-            | (dupPkgs /= []) =
+            | dupPkgs /= [] =
                 "Warning (added by new or init): Some packages were found to \
                 \have names conflicting with others and have been commented \
                 \out in the packages section.\n"
             | otherwise = ""
 
         missingPkgMsg
-            | (Map.size ignored > 0) =
+            | Map.size ignored > 0 =
                 "Warning (added by new or init): Some packages were found to \
                 \be incompatible with the resolver and have been left commented \
                 \out in the packages section.\n"
             | otherwise = ""
 
         extraDepMsg
-            | (Map.size extraDeps > 0) =
+            | Map.size extraDeps > 0 =
                 "Warning (added by new or init): Specified resolver could not \
                 \satisfy all dependencies. Some external packages have been \
                 \added as dependencies.\n"
             | otherwise = ""
-
         makeUserMsg msgs =
             let msg = concat msgs
             in if msg /= "" then
@@ -129,14 +131,14 @@ initProject whichCmd currDir initOpts mresolver = do
             case stripDir currDir dir of
                 Nothing
                     | currDir == dir -> "."
-                    | otherwise -> assert False $ toFilePath dir
-                Just rel -> toFilePath rel
+                    | otherwise -> assert False $ toFilePathNoTrailingSep dir
+                Just rel -> toFilePathNoTrailingSep rel
 
         makeRel = fmap toFilePath . makeRelativeToCurrentDir
 
         pkgs = map toPkg $ Map.elems (fmap (parent . fst) rbundle)
         toPkg dir = PackageEntry
-            { peExtraDep = False
+            { peExtraDepMaybe = Nothing
             , peLocation = PLFilePath $ makeRelDir dir
             , peSubdirs = []
             }
@@ -144,24 +146,24 @@ initProject whichCmd currDir initOpts mresolver = do
 
     $logInfo $ "Initialising configuration using resolver: " <> resolverName r
     $logInfo $ "Total number of user packages considered: "
-               <> (T.pack $ show $ (Map.size bundle + length dupPkgs))
+               <> T.pack (show (Map.size bundle + length dupPkgs))
 
     when (dupPkgs /= []) $ do
         $logWarn $ "Warning! Ignoring "
-                   <> (T.pack $ show $ length dupPkgs)
+                   <> T.pack (show $ length dupPkgs)
                    <> " duplicate packages:"
         rels <- mapM makeRel dupPkgs
         $logWarn $ indent $ showItems rels
 
     when (Map.size ignored > 0) $ do
         $logWarn $ "Warning! Ignoring "
-                   <> (T.pack $ show $ Map.size ignored)
+                   <> T.pack (show $ Map.size ignored)
                    <> " packages due to dependency conflicts:"
         rels <- mapM makeRel (Map.elems (fmap fst ignored))
-        $logWarn $ indent $ showItems $ rels
+        $logWarn $ indent $ showItems rels
 
     when (Map.size extraDeps > 0) $ do
-        $logWarn $ "Warning! " <> (T.pack $ show $ Map.size extraDeps)
+        $logWarn $ "Warning! " <> T.pack (show $ Map.size extraDeps)
                    <> " external dependencies were added."
     $logInfo $
         (if exists then "Overwriting existing configuration file: "
@@ -196,10 +198,12 @@ renderStackYaml p ignoredPackages dupPackages =
                 B.byteString comment <>
                 B.byteString "\n" <>
                 B.byteString (Yaml.encode $ Yaml.object [(name, v)]) <>
-                if (name == "packages") then commentedPackages else "" <>
+                if name == "packages" then commentedPackages else "" <>
                 B.byteString "\n"
 
-    commentHelp = BC.pack .  intercalate "\n" . map ("# " ++)
+    commentLine l | null l = "#"
+                  | otherwise = "# " ++ l
+    commentHelp = BC.pack .  intercalate "\n" . map commentLine
     commentedPackages =
         let ignoredComment = commentHelp
                 [ "The following packages have been ignored due to incompatibility with the"
@@ -217,8 +221,8 @@ renderStackYaml p ignoredPackages dupPackages =
         | pkgs /= [] =
                B.byteString comment
             <> B.byteString "\n"
-            <> (B.byteString $ BC.pack $ concat
-                 $ (map (\x -> "#- " ++ x ++ "\n") pkgs) ++ ["\n"])
+            <> B.byteString (BC.pack $ concat
+                 $ map (\x -> "#- " ++ x ++ "\n") pkgs ++ ["\n"])
         | otherwise = ""
 
     goOthers o
@@ -241,7 +245,7 @@ renderStackYaml p ignoredPackages dupPackages =
         , ""
         , "Some commonly used options have been documented as comments in this file."
         , "For advanced use and comprehensive documentation of the format, please see:"
-        , "http://docs.haskellstack.org/en/stable/yaml_configuration/"
+        , "https://docs.haskellstack.org/en/stable/yaml_configuration/"
         ]
 
     resolverHelp = commentHelp
@@ -306,9 +310,9 @@ renderStackYaml p ignoredPackages dupPackages =
         , "compiler-check: newer-minor"
         ]
 
-getSnapshots' :: (MonadIO m, MonadMask m, MonadReader env m, HasConfig env, HasHttpManager env, MonadLogger m, MonadBaseControl IO m)
+getSnapshots' :: (StackM env m, HasConfig env)
               => m Snapshots
-getSnapshots' =
+getSnapshots' = do
     getSnapshots `catchAny` \e -> do
         $logError $
             "Unable to download snapshot list, and therefore could " <>
@@ -324,14 +328,11 @@ getSnapshots' =
         $logError "    http://docs.haskellstack.org/en/stable/yaml_configuration/"
         $logError ""
         $logError $ "Exception was: " <> T.pack (show e)
-        error ""
+        errorString ""
 
 -- | Get the default resolver value
 getDefaultResolver
-    :: ( MonadBaseControl IO m, MonadIO m, MonadLogger m, MonadMask m
-       , MonadReader env m, HasConfig env , HasGHCVariant env
-       , HasHttpManager env , HasLogLevel env , HasReExec env
-       , HasTerminal env)
+    :: (StackM env m, HasConfig env, HasGHCVariant env)
     => WhichSolverCmd
     -> Path Abs File   -- ^ stack.yaml
     -> InitOpts
@@ -361,10 +362,7 @@ getDefaultResolver whichCmd stackYaml initOpts mresolver bundle =
                 _ -> return $ ResolverSnapshot s
 
 getWorkingResolverPlan
-    :: ( MonadBaseControl IO m, MonadIO m, MonadLogger m, MonadMask m
-       , MonadReader env m, HasConfig env , HasGHCVariant env
-       , HasHttpManager env , HasLogLevel env , HasReExec env
-       , HasTerminal env)
+    :: (StackM env m, HasConfig env, HasGHCVariant env)
     => WhichSolverCmd
     -> Path Abs File   -- ^ stack.yaml
     -> InitOpts
@@ -395,7 +393,7 @@ getWorkingResolverPlan whichCmd stackYaml initOpts bundle resolver = do
                                  \config anyway."
                         return (resolver, Map.empty, Map.empty, Map.empty)
                     | otherwise -> do
-                        when ((Map.size available) == (Map.size info)) $
+                        when (Map.size available == Map.size info) $
                             error "Bug: No packages to ignore"
 
                         if length ignored > 1 then do
@@ -403,19 +401,16 @@ getWorkingResolverPlan whichCmd stackYaml initOpts bundle resolver = do
                           $logWarn $ indent $ showItems ignored
                         else
                           $logWarn $ "*** Ignoring package: "
-                                 <> (T.pack $ packageNameString (head ignored))
+                                 <> T.pack (packageNameString (head ignored))
 
                         go available
                     where
                       indent t   = T.unlines $ fmap ("    " <>) (T.lines t)
-                      isAvailable k _ = not (k `elem` ignored)
+                      isAvailable k _ = k `notElem` ignored
                       available       = Map.filterWithKey isAvailable info
 
 checkBundleResolver
-    :: ( MonadBaseControl IO m, MonadIO m, MonadLogger m, MonadMask m
-       , MonadReader env m, HasConfig env , HasGHCVariant env
-       , HasHttpManager env , HasLogLevel env , HasReExec env
-       , HasTerminal env)
+    :: (StackM env m, HasConfig env, HasGHCVariant env)
     => WhichSolverCmd
     -> Path Abs File   -- ^ stack.yaml
     -> InitOpts
@@ -468,21 +463,21 @@ checkBundleResolver whichCmd stackYaml initOpts bundle resolver = do
                       pkg <- findOneIndependent srcpkgs flags
                       return $ Left [pkg]
                   | otherwise -> throwM (SolverGiveUp giveUpMsg)
-                  where srcpkgs = intersect (Map.keys bundle) packages
+                  where srcpkgs = Map.keys bundle `intersect` packages
 
       -- among a list of packages find one on which none among the rest of the
       -- packages depend. This package is a good candidate to be removed from
       -- the list of packages when there is conflict in dependencies among this
       -- set of packages.
       findOneIndependent packages flags = do
-          platform <- asks (configPlatform . getConfig)
+          platform <- view platformL
           (compiler, _) <- getResolverConstraints stackYaml resolver
           let getGpd pkg = snd (fromJust (Map.lookup pkg bundle))
               getFlags pkg = fromJust (Map.lookup pkg flags)
               deps pkg = gpdPackageDeps (getGpd pkg) compiler platform
                                         (getFlags pkg)
-              allDeps = concat $ map (Map.keys . deps) packages
-              isIndependent pkg = not $ pkg `elem` allDeps
+              allDeps = concatMap (Map.keys . deps) packages
+              isIndependent pkg = pkg `notElem` allDeps
 
               -- prefer to reject packages in deeper directories
               path pkg = fst (fromJust (Map.lookup pkg bundle))
@@ -500,11 +495,11 @@ checkBundleResolver whichCmd stackYaml initOpts bundle resolver = do
           , "    - Update external packages with 'stack update' and try again.\n"
           ]
 
-      needSolver _ (InitOpts {useSolver = True}) = True
+      needSolver _ InitOpts {useSolver = True} = True
       needSolver (ResolverCompiler _)  _ = True
       needSolver _ _ = False
 
-getRecommendedSnapshots :: Snapshots -> (NonEmpty SnapName)
+getRecommendedSnapshots :: Snapshots -> NonEmpty SnapName
 getRecommendedSnapshots snapshots =
     -- in order - Latest LTS, Latest Nightly, all LTS most recent first
     case NonEmpty.nonEmpty ltss of
