@@ -1,3 +1,4 @@
+{-# LANGUAGE ConstraintKinds #-}
 {-# LANGUAGE ScopedTypeVariables #-}
 {-# LANGUAGE DeriveDataTypeable #-}
 {-# LANGUAGE FlexibleContexts #-}
@@ -21,12 +22,11 @@ import           Control.Monad
 import           Control.Monad.Catch
 import           Control.Monad.IO.Class
 import           Control.Monad.Logger
-import           Control.Monad.Reader
 import           Control.Monad.Trans.Writer.Strict
 import           Data.Aeson
 import           Data.Aeson.Types
+import qualified Data.ByteString as SB
 import qualified Data.ByteString.Lazy as LB
-import qualified Data.ByteString.Lazy.Char8 as L8
 import           Data.Conduit
 import           Data.Foldable (asum)
 import qualified Data.HashMap.Strict as HM
@@ -41,20 +41,22 @@ import qualified Data.Set as S
 import           Data.Text (Text)
 import qualified Data.Text as T
 import qualified Data.Text.Encoding as T
+import qualified Data.Text.Encoding.Error as T (lenientDecode)
 import qualified Data.Text.IO as T
 import qualified Data.Text.Lazy as LT
 import           Data.Time.Calendar
 import           Data.Time.Clock
 import           Data.Typeable
 import qualified Data.Yaml as Yaml
-import           Network.HTTP.Client.Conduit hiding (path)
 import           Network.HTTP.Download
-import           Network.HTTP.Types.Status
+import           Network.HTTP.Simple
 import           Path
 import           Path.IO
 import           Prelude
 import           Stack.Constants
-import           Stack.Types
+import           Stack.Types.Config
+import           Stack.Types.PackageName
+import           Stack.Types.StackT
 import           Stack.Types.TemplateName
 import           System.Process.Run
 import           Text.Hastache
@@ -79,15 +81,17 @@ data NewOpts = NewOpts
 
 -- | Create a new project with the given options.
 new
-    :: (HasConfig r, MonadReader r m, MonadLogger m, MonadCatch m, MonadIO m, HasHttpManager r)
+    :: (StackM env m, HasConfig env)
     => NewOpts -> Bool -> m (Path Abs Dir)
 new opts forceOverwrite = do
+    when (newOptsProjectName opts `elem` wiredInPackages) $
+      throwM $ Can'tUseWiredInName (newOptsProjectName opts)
     pwd <- getCurrentDir
     absDir <- if bare then return pwd
                       else do relDir <- parseRelDir (packageNameString project)
                               liftM (pwd </>) (return relDir)
     exists <- doesDirExist absDir
-    configTemplate <- asks (configDefaultTemplate . getConfig)
+    configTemplate <- view $ configL.to configDefaultTemplate
     let template = fromMaybe defaultTemplateName $ asum [ cliOptionTemplate
                                                         , configTemplate
                                                         ]
@@ -128,18 +132,15 @@ data TemplateFrom = LocalTemp | RemoteTemp
 
 -- | Download and read in a template's text content.
 loadTemplate
-    :: forall m r.
-       (HasConfig r, HasHttpManager r, MonadReader r m, MonadIO m, MonadThrow m, MonadCatch m, MonadLogger m)
+    :: forall env m. (StackM env m, HasConfig env)
     => TemplateName -> (TemplateFrom -> m ()) -> m Text
 loadTemplate name logIt = do
-    templateDir <- asks (templatesDir . getConfig)
+    templateDir <- view $ configL.to templatesDir
     case templatePath name of
         AbsPath absFile -> logIt LocalTemp >> loadLocalFile absFile
         UrlPath s -> do
-            let req = fromMaybe (error "impossible happened: already valid \
-                                       \URL couldn't be parsed")
-                                (parseUrl s)
-                rel = fromMaybe backupUrlRelPath (parseRelFile s)
+            req <- parseRequest s
+            let rel = fromMaybe backupUrlRelPath (parseRelFile s)
             downloadTemplate req (templateDir </> rel)
         RelPath relFile ->
             catch
@@ -159,10 +160,10 @@ loadTemplate name logIt = do
                                                 <> "\"")
         exists <- doesFileExist path
         if exists
-            then liftIO (T.readFile (toFilePath path))
+            then liftIO (fmap (T.decodeUtf8With T.lenientDecode) (SB.readFile (toFilePath path)))
             else throwM (FailedToLoadTemplate name (toFilePath path))
     relRequest :: MonadThrow n => Path Rel File -> n Request
-    relRequest rel = parseUrl (defaultTemplateUrl <> "/" <> toFilePath rel)
+    relRequest rel = parseRequest (defaultTemplateUrl <> "/" <> toFilePath rel)
     downloadTemplate :: Request -> Path Abs File -> m Text
     downloadTemplate req path = do
         logIt RemoteTemp
@@ -175,7 +176,7 @@ loadTemplate name logIt = do
 
 -- | Apply and unpack a template into a directory.
 applyTemplate
-    :: (MonadIO m, MonadCatch m, MonadReader r m, HasConfig r, MonadLogger m)
+    :: (StackM env m, HasConfig env)
     => PackageName
     -> TemplateName
     -> Map Text Text
@@ -183,14 +184,18 @@ applyTemplate
     -> Text
     -> m (Map (Path Abs File) LB.ByteString)
 applyTemplate project template nonceParams dir templateText = do
-    config <- asks getConfig
+    config <- view configL
     currentYear <- do
       now <- liftIO getCurrentTime
       (year, _, _) <- return $ toGregorian . utctDay $ now
       return $ T.pack . show $ year
     let context = M.union (M.union nonceParams extraParams) configParams
           where
+            nameAsVarId = T.replace "-" "_" $ packageNameText project
+            nameAsModule = T.filter (/= '-') $ T.toTitle $ packageNameText project
             extraParams = M.fromList [ ("name", packageNameText project)
+                                     , ("name-as-varid", nameAsVarId)
+                                     , ("name-as-module", nameAsModule)
                                      , ("year", currentYear) ]
             configParams = configTemplateParams config
     (applied,missingKeys) <-
@@ -257,11 +262,11 @@ writeTemplateFiles files =
 
 -- | Run any initialization functions, such as Git.
 runTemplateInits
-    :: (MonadIO m, MonadReader r m, HasConfig r, MonadLogger m, MonadCatch m)
+    :: (StackM env m, HasConfig env)
     => Path Abs Dir -> m ()
 runTemplateInits dir = do
     menv <- getMinimalEnvOverride
-    config <- asks getConfig
+    config <- view configL
     case configScmInit config of
         Nothing -> return ()
         Just Git ->
@@ -270,9 +275,7 @@ runTemplateInits dir = do
                          $logInfo "git init failed to run, ignoring ...")
 
 -- | Display the set of templates accompanied with description if available.
-listTemplates
-    :: (MonadIO m, MonadReader r m, HasHttpManager r, MonadCatch m)
-    => m ()
+listTemplates :: StackM env m => m ()
 listTemplates = do
     templates <- getTemplates
     templateInfo <- getTemplateInfo
@@ -289,49 +292,37 @@ listTemplates = do
       else mapM_ (liftIO . T.putStrLn . templateName) (S.toList templates)
 
 -- | Get the set of templates.
-getTemplates
-    :: (MonadIO m, MonadReader r m, HasHttpManager r, MonadCatch m)
-    => m (Set TemplateName)
+getTemplates :: StackM env m => m (Set TemplateName)
 getTemplates = do
-    req <- liftM addHeaders (parseUrl defaultTemplatesList)
-    resp <- catch (httpLbs req) (throwM . FailedToDownloadTemplates)
-    case statusCode (responseStatus resp) of
-        200 ->
-            case eitherDecode (responseBody resp) >>=
-                 parseEither parseTemplateSet of
-                Left err -> throwM (BadTemplatesJSON err (responseBody resp))
-                Right value -> return value
+    req <- liftM setGithubHeaders (parseUrlThrow defaultTemplatesList)
+    resp <- catch (httpJSON req) (throwM . FailedToDownloadTemplates)
+    case getResponseStatusCode resp of
+        200 -> return $ unTemplateSet $ getResponseBody resp
         code -> throwM (BadTemplatesResponse code)
 
-getTemplateInfo
-    :: (MonadIO m, MonadReader r m, HasHttpManager r, MonadCatch m)
-    => m (Map Text TemplateInfo)
+getTemplateInfo :: StackM env m => m (Map Text TemplateInfo)
 getTemplateInfo = do
-  req <- liftM addHeaders (parseUrl defaultTemplateInfoUrl)
+  req <- liftM setGithubHeaders (parseUrlThrow defaultTemplateInfoUrl)
   resp <- catch (liftM Right $ httpLbs req) (\(ex :: HttpException) -> return . Left $ "Failed to download template info. The HTTP error was: " <> show ex)
   case resp >>= is200 of
     Left err -> do
       liftIO . putStrLn $ err
       return M.empty
     Right resp' ->
-      case Yaml.decodeEither (LB.toStrict $ responseBody resp') :: Either String Object of
+      case Yaml.decodeEither (LB.toStrict $ getResponseBody resp') :: Either String Object of
         Left err ->
           throwM $ BadTemplateInfo err
         Right o ->
           return (M.mapMaybe (Yaml.parseMaybe Yaml.parseJSON) (M.fromList . HM.toList $ o) :: Map Text TemplateInfo)
   where
     is200 resp =
-      if statusCode (responseStatus resp) == 200
-        then return resp
-        else Left $ "Unexpected status code while retrieving templates info: " <> show (statusCode $ responseStatus resp)
+      case getResponseStatusCode resp of
+        200 -> return resp
+        code -> Left $ "Unexpected status code while retrieving templates info: " <> show code
 
-addHeaders :: Request -> Request
-addHeaders req =
-  req
-    { requestHeaders = [ ("User-Agent", "The Haskell Stack")
-                       , ("Accept", "application/vnd.github.v3+json")] <>
-      requestHeaders req
-    }
+newtype TemplateSet = TemplateSet { unTemplateSet :: Set TemplateName }
+instance FromJSON TemplateSet where
+  parseJSON = fmap TemplateSet . parseTemplateSet
 
 -- | Parser the set of templates from the JSON.
 parseTemplateSet :: Value -> Parser (Set TemplateName)
@@ -382,7 +373,6 @@ data NewException
                                !DownloadException
     | FailedToDownloadTemplates !HttpException
     | BadTemplatesResponse !Int
-    | BadTemplatesJSON !String !LB.ByteString
     | AlreadyExists !(Path Abs Dir)
     | MissingParameters !PackageName !TemplateName !(Set String) !(Path Abs File)
     | InvalidTemplate !TemplateName !String
@@ -390,6 +380,7 @@ data NewException
     | FailedToDownloadTemplateInfo !HttpException
     | BadTemplateInfo !String
     | BadTemplateInfoResponse !Int
+    | Can'tUseWiredInName !PackageName
     deriving (Typeable)
 
 instance Exception NewException
@@ -400,25 +391,19 @@ instance Show NewException where
         " from " <>
         path
     show (FailedToDownloadTemplate name (RedownloadFailed _ _ resp)) =
-        case statusCode (responseStatus resp) of
+        case getResponseStatusCode resp of
             404 ->
                 "That template doesn't exist. Run `stack templates' to see a list of available templates."
             code ->
                 "Failed to download template " <> T.unpack (templateName name) <>
                 ": unknown reason, status code was: " <>
                 show code
-    show (FailedToDownloadTemplate name _) =
-        "Failed to download template " <> T.unpack (templateName name) <>
-        ", reason unknown."
     show (AlreadyExists path) =
         "Directory " <> toFilePath path <> " already exists. Aborting."
     show (FailedToDownloadTemplates ex) =
         "Failed to download templates. The HTTP error was: " <> show ex
     show (BadTemplatesResponse code) =
         "Unexpected status code while retrieving templates list: " <> show code
-    show (BadTemplatesJSON err bytes) =
-        "Github returned some JSON that couldn't be parsed: " <> err <> "\n\n" <>
-        L8.unpack bytes
     show (MissingParameters name template missingKeys userConfigPath) =
         intercalate
             "\n"
@@ -458,3 +443,5 @@ instance Show NewException where
         "Template info couldn't be parsed: " <> err
     show (BadTemplateInfoResponse code) =
         "Unexpected status code while retrieving templates info: " <> show code
+    show (Can'tUseWiredInName name) =
+        "The name \"" <> packageNameString name <> "\" is used by GHC wired-in packages, and so shouldn't be used as a package name"
