@@ -18,9 +18,11 @@ module Stack.Package
   (readPackage
   ,readPackageBS
   ,readPackageDescriptionDir
+  ,readDotBuildinfo
   ,readPackageUnresolved
   ,readPackageUnresolvedBS
   ,resolvePackage
+  ,packageFromPackageDescription
   ,findOrGenerateCabalFile
   ,hpack
   ,Package(..)
@@ -38,8 +40,8 @@ module Stack.Package
   ,cabalFilePackageId)
   where
 
-import Prelude ()
-import Prelude.Compat
+import           Prelude ()
+import           Prelude.Compat
 
 import           Control.Arrow ((&&&))
 import           Control.Exception hiding (try,catch)
@@ -76,7 +78,7 @@ import qualified Distribution.PackageDescription.Parse as D
 import           Distribution.ParseUtils
 import           Distribution.Simple.Utils
 import           Distribution.System (OS (..), Arch, Platform (..))
-import           Distribution.Text (display, simpleParse)
+import qualified Distribution.Text as D
 import qualified Distribution.Verbosity as D
 import qualified Hpack
 import qualified Hpack.Config as Hpack
@@ -87,7 +89,16 @@ import           Path.IO hiding (findFiles)
 import           Safe (headDef, tailSafe)
 import           Stack.Build.Installed
 import           Stack.Constants
-import           Stack.Types
+import           Stack.PrettyPrint
+import           Stack.Types.Build
+import           Stack.Types.Compiler
+import           Stack.Types.Config
+import           Stack.Types.FlagName
+import           Stack.Types.GhcPkgId
+import           Stack.Types.Package
+import           Stack.Types.PackageIdentifier
+import           Stack.Types.PackageName
+import           Stack.Types.Version
 import qualified System.Directory as D
 import           System.FilePath (splitExtensions, replaceExtension)
 import qualified System.FilePath as FilePath
@@ -99,17 +110,17 @@ readPackageUnresolved :: (MonadIO m, MonadThrow m)
                       -> m ([PWarning],GenericPackageDescription)
 readPackageUnresolved cabalfp =
   liftIO (BS.readFile (FL.toFilePath cabalfp))
-  >>= readPackageUnresolvedBS (Just cabalfp)
+  >>= readPackageUnresolvedBS (Left cabalfp)
 
 -- | Read the raw, unresolved package information from a ByteString.
 readPackageUnresolvedBS :: (MonadThrow m)
-                        => Maybe (Path Abs File)
+                        => Either (Path Abs File) PackageIdentifier
                         -> BS.ByteString
                         -> m ([PWarning],GenericPackageDescription)
-readPackageUnresolvedBS mcabalfp bs =
+readPackageUnresolvedBS source bs =
     case parsePackageDescription chars of
        ParseFailed per ->
-         throwM (PackageInvalidCabalFile mcabalfp per)
+         throwM (PackageInvalidCabalFile source per)
        ParseOk warnings gpkg -> return (warnings,gpkg)
   where
     chars = T.unpack (dropBOM (decodeUtf8With lenientDecode bs))
@@ -129,10 +140,11 @@ readPackage packageConfig cabalfp =
 -- | Reads and exposes the package information, from a ByteString
 readPackageBS :: (MonadThrow m)
               => PackageConfig
+              -> PackageIdentifier
               -> BS.ByteString
               -> m ([PWarning],Package)
-readPackageBS packageConfig bs =
-  do (warnings,gpkg) <- readPackageUnresolvedBS Nothing bs
+readPackageBS packageConfig ident bs =
+  do (warnings,gpkg) <- readPackageUnresolvedBS (Right ident) bs
      return (warnings,resolvePackage packageConfig gpkg)
 
 -- | Get 'GenericPackageDescription' and 'PackageDescription' reading info
@@ -146,6 +158,17 @@ readPackageDescriptionDir config pkgDir = do
     gdesc   <- liftM snd (readPackageUnresolved cabalfp)
     return (gdesc, resolvePackageDescription config gdesc)
 
+-- | Read @<package>.buildinfo@ ancillary files produced by some Setup.hs hooks.
+-- The file includes Cabal file syntax to be merged into the package description
+-- derived from the package's .cabal file.
+--
+-- NOTE: not to be confused with BuildInfo, an Stack-internal datatype.
+readDotBuildinfo :: MonadIO m
+                 => Path Abs File
+                 -> m HookedBuildInfo
+readDotBuildinfo buildinfofp =
+    liftIO $ readHookedBuildInfo D.silent (toFilePath buildinfofp)
+
 -- | Print cabal file warnings.
 printCabalFileWarning
     :: (MonadLogger m)
@@ -157,10 +180,10 @@ printCabalFileWarning cabalfp =
                 ("Cabal file warning in " <> T.pack (toFilePath cabalfp) <>
                  ": " <>
                  T.pack x)
-        (UTFWarning line msg) ->
+        (UTFWarning ln msg) ->
             $logWarn
                 ("Cabal file warning in " <> T.pack (toFilePath cabalfp) <> ":" <>
-                 T.pack (show line) <>
+                 T.pack (show ln) <>
                  ": " <>
                  T.pack msg)
 
@@ -174,21 +197,34 @@ checkCabalFileName name cabalfp = do
     when (expected /= toFilePath (filename cabalfp))
         $ throwM $ MismatchedCabalName cabalfp name
 
--- | Resolve a parsed cabal file into a 'Package'.
+-- | Resolve a parsed cabal file into a 'Package', which contains all of
+-- the info needed for stack to build the 'Package' given the current
+-- configuration.
 resolvePackage :: PackageConfig
                -> GenericPackageDescription
                -> Package
 resolvePackage packageConfig gpkg =
+    packageFromPackageDescription
+        packageConfig
+        (genPackageFlags gpkg)
+        (resolvePackageDescription packageConfig gpkg)
+
+packageFromPackageDescription :: PackageConfig
+                              -> [D.Flag]
+                              -> PackageDescription
+                              -> Package
+packageFromPackageDescription packageConfig pkgFlags pkg =
     Package
     { packageName = name
     , packageVersion = fromCabalVersion (pkgVersion pkgId)
+    , packageLicense = license pkg
     , packageDeps = deps
     , packageFiles = pkgFiles
     , packageTools = packageDescTools pkg
     , packageGhcOptions = packageConfigGhcOptions packageConfig
     , packageFlags = packageConfigFlags packageConfig
     , packageDefaultFlags = M.fromList
-      [(fromCabalFlagName (flagName flag), flagDefault flag) | flag <- genPackageFlags gpkg]
+      [(fromCabalFlagName (flagName flag), flagDefault flag) | flag <- pkgFlags]
     , packageAllDeps = S.fromList (M.keys deps)
     , packageHasLibrary = maybe False (buildable . libBuildInfo) (library pkg)
     , packageTests = M.fromList
@@ -200,6 +236,8 @@ resolvePackage packageConfig gpkg =
     , packageExes = S.fromList
       [T.pack (exeName biBuildInfo) | biBuildInfo <- executables pkg
                                     , buildable (buildInfo biBuildInfo)]
+    -- This is an action used to collect info needed for "stack ghci".
+    -- This info isn't usually needed, so computation of it is deferred.
     , packageOpts = GetPackageOpts $
       \sourceMap installedMap omitPkgs addPkgs cabalfp ->
            do (componentsModules,componentFiles,_,_) <- getPackageFiles pkgFiles cabalfp
@@ -210,36 +248,49 @@ resolvePackage packageConfig gpkg =
           False
           (not . null . exposedModules)
           (library pkg)
-    , packageSimpleType = buildType (packageDescription gpkg) == Just Simple
+    , packageBuildType = buildType pkg
+    , packageSetupDeps = msetupDeps
     }
   where
+    -- Gets all of the modules, files, build files, and data files that
+    -- constitute the package. This is primarily used for dirtiness
+    -- checking during build, as well as use by "stack ghci"
     pkgFiles = GetPackageFiles $
-        \cabalfp ->
-             do let pkgDir = parent cabalfp
-                distDir <- distDirFromDir pkgDir
-                (componentModules,componentFiles,dataFiles',warnings) <-
-                    runReaderT
-                        (packageDescModulesAndFiles pkg)
-                        (cabalfp, buildDir distDir)
-                setupFiles <-
-                    if buildType pkg `elem` [Nothing, Just Custom]
-                    then do
-                        let setupHsPath = pkgDir </> $(mkRelFile "Setup.hs")
-                            setupLhsPath = pkgDir </> $(mkRelFile "Setup.lhs")
-                        setupHsExists <- doesFileExist setupHsPath
-                        if setupHsExists then return (S.singleton setupHsPath) else do
-                            setupLhsExists <- doesFileExist setupLhsPath
-                            if setupLhsExists then return (S.singleton setupLhsPath) else return S.empty
-                    else return S.empty
-                buildFiles <- liftM (S.insert cabalfp . S.union setupFiles) $ do
-                    let hpackPath = pkgDir </> $(mkRelFile Hpack.packageConfig)
-                    hpackExists <- doesFileExist hpackPath
-                    return $ if hpackExists then S.singleton hpackPath else S.empty
-                return (componentModules, componentFiles, buildFiles <> dataFiles', warnings)
-    pkgId = package (packageDescription gpkg)
+        \cabalfp -> $debugBracket ("getPackageFiles" <+> display cabalfp) $ do
+             let pkgDir = parent cabalfp
+             distDir <- distDirFromDir pkgDir
+             (componentModules,componentFiles,dataFiles',warnings) <-
+                 runReaderT
+                     (packageDescModulesAndFiles pkg)
+                     (cabalfp, buildDir distDir)
+             setupFiles <-
+                 if buildType pkg `elem` [Nothing, Just Custom]
+                 then do
+                     let setupHsPath = pkgDir </> $(mkRelFile "Setup.hs")
+                         setupLhsPath = pkgDir </> $(mkRelFile "Setup.lhs")
+                     setupHsExists <- doesFileExist setupHsPath
+                     if setupHsExists then return (S.singleton setupHsPath) else do
+                         setupLhsExists <- doesFileExist setupLhsPath
+                         if setupLhsExists then return (S.singleton setupLhsPath) else return S.empty
+                 else return S.empty
+             buildFiles <- liftM (S.insert cabalfp . S.union setupFiles) $ do
+                 let hpackPath = pkgDir </> $(mkRelFile Hpack.packageConfig)
+                 hpackExists <- doesFileExist hpackPath
+                 return $ if hpackExists then S.singleton hpackPath else S.empty
+             return (componentModules, componentFiles, buildFiles <> dataFiles', warnings)
+    pkgId = package pkg
     name = fromCabalPackageName (pkgName pkgId)
-    pkg = resolvePackageDescription packageConfig gpkg
-    deps = M.filterWithKey (const . (/= name)) (packageDependencies pkg)
+    deps = M.filterWithKey (const . (/= name)) (M.union
+        (packageDependencies pkg)
+        -- We include all custom-setup deps - if present - in the
+        -- package deps themselves. Stack always works with the
+        -- invariant that there will be a single installed package
+        -- relating to a package name, and this applies at the setup
+        -- dependency level as well.
+        (fromMaybe M.empty msetupDeps))
+    msetupDeps = fmap
+        (M.fromList . map (depName &&& depRange) . setupDepends)
+        (setupBuildInfo pkg)
 
 -- | Generate GHC options for the package's components, and a list of
 -- options which apply generally to the package, not one specific
@@ -255,7 +306,7 @@ generatePkgDescOpts
     -> Map NamedComponent (Set DotCabalPath)
     -> m (Map NamedComponent BuildInfoOpts)
 generatePkgDescOpts sourceMap installedMap omitPkgs addPkgs cabalfp pkg componentPaths = do
-    config <- asks getConfig
+    config <- view configL
     distDir <- distDirFromDir cabalDir
     let cabalMacros = autogenDir distDir </> $(mkRelFile "cabal_macros.h")
     exists <- doesFileExist cabalMacros
@@ -308,6 +359,7 @@ generatePkgDescOpts sourceMap installedMap omitPkgs addPkgs cabalfp pkg componen
   where
     cabalDir = parent cabalfp
 
+-- | Input to 'generateBuildInfoOpts'
 data BioInput = BioInput
     { biSourceMap :: !SourceMap
     , biInstalledMap :: !InstalledMap
@@ -318,12 +370,14 @@ data BioInput = BioInput
     , biAddPackages :: ![PackageName]
     , biBuildInfo :: !BuildInfo
     , biDotCabalPaths :: !(Set DotCabalPath)
-    , biConfigLibDirs :: !(Set Text)
-    , biConfigIncludeDirs :: !(Set Text)
+    , biConfigLibDirs :: !(Set FilePath)
+    , biConfigIncludeDirs :: !(Set FilePath)
     , biComponentName :: !NamedComponent
     }
 
--- | Generate GHC options for the target.
+-- | Generate GHC options for the target. Since Cabal also figures out
+-- these options, currently this is only used for invoking GHCI (via
+-- stack ghci).
 generateBuildInfoOpts :: BioInput -> BuildInfoOpts
 generateBuildInfoOpts BioInput {..} =
     BuildInfoOpts
@@ -350,7 +404,7 @@ generateBuildInfoOpts BioInput {..} =
     deps =
         concat
             [ case M.lookup name biInstalledMap of
-                Just (_, Stack.Types.Library _ident ipid) -> ["-package-id=" <> ghcPkgIdString ipid]
+                Just (_, Stack.Types.Package.Library _ident ipid) -> ["-package-id=" <> ghcPkgIdString ipid]
                 _ -> ["-package=" <> packageNameString name <>
                  maybe "" -- This empty case applies to e.g. base.
                      ((("-" <>) . versionString) . piiVersion)
@@ -366,7 +420,7 @@ generateBuildInfoOpts BioInput {..} =
       where
         isGhc GHC = True
         isGhc _ = False
-    extOpts = map (("-X" ++) . display) (usedExtensions biBuildInfo)
+    extOpts = map (("-X" ++) . D.display) (usedExtensions biBuildInfo)
     srcOpts =
         map
             (("-i" <>) . toFilePathNoTrailingSep)
@@ -377,11 +431,10 @@ generateBuildInfoOpts BioInput {..} =
              | Just makeGenDir <- [fileGenDirFromComponentName biComponentName]]) ++
         ["-stubdir=" ++ toFilePathNoTrailingSep (buildDir biDistDir)]
     toIncludeDir "." = Just biCabalDir
-    toIncludeDir x = fmap (biCabalDir </>) (parseRelDir x)
+    toIncludeDir relDir = concatAndColapseAbsDir biCabalDir relDir
     includeOpts =
         map ("-I" <>) (configExtraIncludeDirs <> pkgIncludeOpts)
-    configExtraIncludeDirs =
-        map T.unpack (S.toList biConfigIncludeDirs)
+    configExtraIncludeDirs = S.toList biConfigIncludeDirs
     pkgIncludeOpts =
         [ toFilePathNoTrailingSep absDir
         | dir <- includeDirs biBuildInfo
@@ -390,8 +443,7 @@ generateBuildInfoOpts BioInput {..} =
     libOpts =
         map ("-l" <>) (extraLibs biBuildInfo) <>
         map ("-L" <>) (configExtraLibDirs <> pkgLibDirs)
-    configExtraLibDirs =
-        map T.unpack (S.toList biConfigLibDirs)
+    configExtraLibDirs = S.toList biConfigLibDirs
     pkgLibDirs =
         [ toFilePathNoTrailingSep absDir
         | dir <- extraLibDirs biBuildInfo
@@ -472,11 +524,11 @@ getBuildComponentDir (Just name) = parseRelDir (name FilePath.</> (name ++ "-tmp
 
 -- | Get all dependencies of the package (buildable targets only).
 packageDependencies :: PackageDescription -> Map PackageName VersionRange
-packageDependencies =
-  M.fromListWith intersectVersionRanges .
-  concatMap (fmap (depName &&& depRange) .
-             targetBuildDepends) .
-  allBuildInfo'
+packageDependencies pkg =
+  M.fromListWith intersectVersionRanges $
+  map (depName &&& depRange) $
+  concatMap targetBuildDepends (allBuildInfo' pkg) ++
+  maybe [] setupDepends (setupBuildInfo pkg)
 
 -- | Get all build tool dependencies of the package (buildable targets only).
 packageToolDependencies :: PackageDescription -> Map Text VersionRange
@@ -491,8 +543,8 @@ packageDescTools :: PackageDescription -> [Dependency]
 packageDescTools = concatMap buildTools . allBuildInfo'
 
 -- | This is a copy-paste from Cabal's @allBuildInfo@ function, but with the
--- @buildable@ test removed. The reason is that (surprise) Cabal is broken,
--- see: https://github.com/haskell/cabal/issues/1725
+-- @buildable@ test removed. The implementation is broken.
+-- See: https://github.com/haskell/cabal/issues/1725
 allBuildInfo' :: PackageDescription -> [BuildInfo]
 allBuildInfo' pkg_descr = [ bi | Just lib <- [library pkg_descr]
                               , let bi = libBuildInfo lib
@@ -536,7 +588,9 @@ packageDescModulesAndFiles pkg = do
             (mapM
                  (asModuleAndFileMap benchComponent benchmarkFiles)
                  (benchmarks pkg))
-    (dfiles) <- resolveGlobFiles (map (dataDir pkg FilePath.</>) (dataFiles pkg))
+    dfiles <- resolveGlobFiles
+                    (extraSrcFiles pkg
+                        ++ map (dataDir pkg FilePath.</>) (dataFiles pkg))
     let modules = libraryMods <> executableMods <> testMods <> benchModules
         files =
             libDotCabalFiles <> exeDotCabalFiles <> testDotCabalFiles <>
@@ -700,7 +754,7 @@ libraryFiles lib = do
         resolveFilesAndDeps
             Nothing
             (dirs ++ [dir])
-            (names <> exposed)
+            names
             haskellModuleExts
     cfiles <- buildOtherSources build
     return (modules, files <> cfiles, warnings)
@@ -724,14 +778,10 @@ buildOtherSources build =
 
 -- | Get the target's JS sources.
 targetJsSources :: BuildInfo -> [FilePath]
-#if MIN_VERSION_Cabal(1, 22, 0)
 targetJsSources = jsSources
-#else
-targetJsSources = const []
-#endif
 
--- | Get all dependencies of a package, including library,
--- executables, tests, benchmarks.
+-- | Evaluates the conditions of a 'GenericPackageDescription', yielding
+-- a resolved 'PackageDescription'.
 resolvePackageDescription :: PackageConfig
                           -> GenericPackageDescription
                           -> PackageDescription
@@ -834,11 +884,7 @@ resolveConditions rc addDeps (CondNode lib deps cs) = basic <> children
                       case (flavor, rcCompilerVersion rc) of
                         (GHC, GhcVersion vghc) -> vghc `withinRange` range
                         (GHC, GhcjsVersion _ vghc) -> vghc `withinRange` range
-#if MIN_VERSION_Cabal(1, 22, 0)
                         (GHCJS, GhcjsVersion vghcjs _) ->
-#else
-                        (OtherCompiler "ghcjs", GhcjsVersion vghcjs _) ->
-#endif
                           vghcjs `withinRange` range
                         _ -> False
 
@@ -898,17 +944,17 @@ resolveFilesAndDeps component dirs names0 exts = do
         let unlistedModules =
                 foundModules `S.difference`
                 S.fromList (mapMaybe dotCabalModule names0)
-        cabalfp <- asks fst
         return $
             if S.null unlistedModules
                 then []
                 else [ UnlistedModulesWarning
-                           cabalfp
                            component
                            (S.toList unlistedModules)]
     warnMissing _missingModules = do
         return []
-        {- FIXME: the issue with this is it's noisy for modules like Paths_*
+        -- TODO: bring this back - see
+        -- https://github.com/commercialhaskell/stack/issues/2649
+        {-
         cabalfp <- asks fst
         return $
             if null missingModules
@@ -960,7 +1006,7 @@ parseDumpHI dumpHIPath = do
             dropWhile (not . ("module dependencies:" `C8.isPrefixOf`)) dumpHI
         moduleDeps =
             S.fromList $
-            mapMaybe (simpleParse . T.unpack . decodeUtf8) $
+            mapMaybe (D.simpleParse . T.unpack . decodeUtf8) $
             C8.words $
             C8.concat $
             C8.dropWhile (/= ' ') (headDef "" startModuleDeps) :
@@ -969,14 +1015,14 @@ parseDumpHI dumpHIPath = do
             -- The dependent file path is surrounded by quotes but is not escaped.
             -- It can be an absolute or relative path.
             mapMaybe
-                ((fmap T.unpack .
+                (fmap T.unpack .
                   (T.stripSuffix "\"" <=< T.stripPrefix "\"") .
-                  T.dropWhileEnd (== '\r') . decodeUtf8 . C8.dropWhile (/= '"'))) $
+                  T.dropWhileEnd (== '\r') . decodeUtf8 . C8.dropWhile (/= '"')) $
             filter ("addDependentFile \"" `C8.isPrefixOf`) dumpHI
     thDepsResolved <- liftM catMaybes $ forM thDeps $ \x -> do
         mresolved <- forgivingAbsence (resolveFile dir x) >>= rejectMissingFile
         when (isNothing mresolved) $
-            $logWarn $ "Warning: qAddDepedency path listed in " <> T.pack dumpHIPath <>
+            $logWarn $ "Warning: addDependentFile path (Template Haskell) listed in " <> T.pack dumpHIPath <>
                 " does not exist: " <> T.pack x
         return mresolved
     return (moduleDeps, thDepsResolved)
@@ -1009,7 +1055,7 @@ findCandidate dirs exts name = do
         [] -> do
             case name of
                 DotCabalModule mn
-                  | display mn /= paths_pkg pkg -> logPossibilities dirs mn
+                  | D.display mn /= paths_pkg pkg -> logPossibilities dirs mn
                 _ -> return ()
             return Nothing
         (candidate:rest) -> do
@@ -1061,7 +1107,7 @@ warnMultiple name candidate rest =
          T.intercalate "," (map (T.pack . toFilePath) rest) <>
          "), picking " <>
          T.pack (toFilePath candidate))
-  where showName (DotCabalModule name') = T.pack (display name')
+  where showName (DotCabalModule name') = T.pack (D.display name')
         showName (DotCabalMain fp) = T.pack fp
         showName (DotCabalFile fp) = T.pack fp
         showName (DotCabalCFile fp) = T.pack fp
@@ -1081,7 +1127,7 @@ logPossibilities dirs mn = do
         _ ->
             $logWarn
                 ("Unable to find a known candidate for the Cabal entry \"" <>
-                 T.pack (display mn) <>
+                 T.pack (D.display mn) <>
                  "\", but did find: " <>
                  T.intercalate ", " (map (T.pack . toFilePath) possibilities) <>
                  ". If you are using a custom preprocessor for this module " <>
@@ -1096,7 +1142,7 @@ logPossibilities dirs mn = do
                          (map
                               filename
                               (filter
-                                   (isPrefixOf (display name) .
+                                   (isPrefixOf (D.display name) .
                                     toFilePath . filename)
                                    files)))
             dirs
@@ -1145,7 +1191,11 @@ hpack pkgDir = do
     when exists $ do
         let fpt = T.pack (toFilePath hpackFile)
         $logDebug $ "Running hpack on " <> fpt
+#if MIN_VERSION_hpack(0,18,0)
+        r <- liftIO $ Hpack.hpackResult (Just $ toFilePath pkgDir)
+#else
         r <- liftIO $ Hpack.hpackResult (toFilePath pkgDir)
+#endif
         forM_ (Hpack.resultWarnings r) $ \w -> $logWarn ("WARNING: " <> T.pack w)
         let cabalFile = T.pack (Hpack.resultCabalFile r)
         case Hpack.resultStatus r of
