@@ -5,6 +5,7 @@
 {-# LANGUAGE TupleSections #-}
 {-# LANGUAGE DeriveDataTypeable #-}
 {-# LANGUAGE DeriveGeneric #-}
+{-# LANGUAGE TemplateHaskell #-}
 module Stack.PackageDump
     ( Line
     , eachSection
@@ -13,20 +14,19 @@ module Stack.PackageDump
     , conduitDumpPackage
     , ghcPkgDump
     , ghcPkgDescribe
-    , InstalledCache
-    , InstalledCacheEntry (..)
     , newInstalledCache
     , loadInstalledCache
     , saveInstalledCache
     , addProfiling
     , addHaddock
+    , addSymbols
     , sinkMatching
     , pruneDeps
     ) where
 
 import           Control.Applicative
 import           Control.Arrow ((&&&))
-import           Control.Exception.Enclosed (tryIO)
+import           Control.Exception.Safe (tryIO)
 import           Control.Monad (liftM)
 import           Control.Monad.Catch
 import           Control.Monad.IO.Class
@@ -34,47 +34,37 @@ import           Control.Monad.Logger (MonadLogger)
 import           Control.Monad.Trans.Control
 import           Data.Attoparsec.Args
 import           Data.Attoparsec.Text as P
-import           Data.Binary.VersionTagged
 import           Data.Conduit
 import qualified Data.Conduit.List as CL
 import qualified Data.Conduit.Text as CT
 import           Data.Either (partitionEithers)
 import           Data.IORef
+import           Data.List (isPrefixOf)
 import           Data.Map (Map)
 import qualified Data.Map as Map
 import           Data.Maybe (catMaybes, listToMaybe)
 import           Data.Maybe.Extra (mapMaybeM)
 import qualified Data.Set as Set
-import qualified Data.Text as T
+import           Data.Store.VersionTagged
 import           Data.Text (Text)
+import qualified Data.Text as T
 import           Data.Typeable (Typeable)
-import           GHC.Generics (Generic)
+import qualified Distribution.License as C
+import qualified Distribution.System as OS
+import qualified Distribution.Text as C
 import           Path
-import           Path.IO (ensureDir)
 import           Path.Extra (toFilePathNoTrailingSep)
 import           Prelude -- Fix AMP warning
 import           Stack.GhcPkg
-import           Stack.Types
+import           Stack.Types.Compiler
+import           Stack.Types.GhcPkgId
+import           Stack.Types.PackageDump
+import           Stack.Types.PackageIdentifier
+import           Stack.Types.PackageName
+import           Stack.Types.Version
 import           System.Directory (getDirectoryContents, doesFileExist)
 import           System.Process.Read
 
--- | Cached information on whether package have profiling libraries and haddocks.
-newtype InstalledCache = InstalledCache (IORef InstalledCacheInner)
-newtype InstalledCacheInner = InstalledCacheInner (Map GhcPkgId InstalledCacheEntry)
-    deriving (Binary, NFData, Generic)
-instance HasStructuralInfo InstalledCacheInner
-instance HasSemanticVersion InstalledCacheInner
-
--- | Cached information on whether a package has profiling libraries and haddocks.
-data InstalledCacheEntry = InstalledCacheEntry
-    { installedCacheProfiling :: !Bool
-    , installedCacheHaddock :: !Bool
-    , installedCacheIdent :: !PackageIdentifier }
-    deriving (Eq, Generic)
-instance Binary InstalledCacheEntry
-instance HasStructuralInfo InstalledCacheEntry
-instance NFData InstalledCacheEntry
-
 -- | Call ghc-pkg dump with appropriate flags and stream to the given @Sink@, for a single database
 ghcPkgDump
     :: (MonadIO m, MonadLogger m, MonadBaseControl IO m, MonadCatch m)
@@ -127,16 +117,16 @@ newInstalledCache = liftIO $ InstalledCache <$> newIORef (InstalledCacheInner Ma
 
 -- | Load a @InstalledCache@ from disk, swallowing any errors and returning an
 -- empty cache.
-loadInstalledCache :: (MonadLogger m, MonadIO m) => Path Abs File -> m InstalledCache
+loadInstalledCache :: (MonadLogger m, MonadIO m, MonadBaseControl IO m)
+                   => Path Abs File -> m InstalledCache
 loadInstalledCache path = do
-    m <- taggedDecodeOrLoad path (return $ InstalledCacheInner Map.empty)
+    m <- $(versionedDecodeOrLoad installedCacheVC) path (return $ InstalledCacheInner Map.empty)
     liftIO $ InstalledCache <$> newIORef m
 
 -- | Save a @InstalledCache@ to disk
-saveInstalledCache :: MonadIO m => Path Abs File -> InstalledCache -> m ()
-saveInstalledCache path (InstalledCache ref) = liftIO $ do
-    ensureDir (parent path)
-    readIORef ref >>= taggedEncodeFile path
+saveInstalledCache :: (MonadLogger m, MonadIO m) => Path Abs File -> InstalledCache -> m ()
+saveInstalledCache path (InstalledCache ref) =
+    liftIO (readIORef ref) >>= $(versionedEncodeFile installedCacheVC) path
 
 -- | Prune a list of possible packages down to those whose dependencies are met.
 --
@@ -183,14 +173,16 @@ pruneDeps getName getId getDepends chooseBest =
 sinkMatching :: Monad m
              => Bool -- ^ require profiling?
              -> Bool -- ^ require haddock?
+             -> Bool -- ^ require debugging symbols?
              -> Map PackageName Version -- ^ allowed versions
-             -> Consumer (DumpPackage Bool Bool)
+             -> Consumer (DumpPackage Bool Bool Bool)
                          m
-                         (Map PackageName (DumpPackage Bool Bool))
-sinkMatching reqProfiling reqHaddock allowed = do
+                         (Map PackageName (DumpPackage Bool Bool Bool))
+sinkMatching reqProfiling reqHaddock reqSymbols allowed = do
     dps <- CL.filter (\dp -> isAllowed (dpPackageIdent dp) &&
                              (not reqProfiling || dpProfiling dp) &&
-                             (not reqHaddock || dpHaddock dp))
+                             (not reqHaddock || dpHaddock dp) &&
+                             (not reqSymbols || dpSymbols dp))
        =$= CL.consume
     return $ Map.fromList $ map (packageIdentifierName . dpPackageIdent &&& id) $ Map.elems $ pruneDeps
         id
@@ -207,7 +199,7 @@ sinkMatching reqProfiling reqHaddock allowed = do
 -- | Add profiling information to the stream of @DumpPackage@s
 addProfiling :: MonadIO m
              => InstalledCache
-             -> Conduit (DumpPackage a b) m (DumpPackage Bool b)
+             -> Conduit (DumpPackage a b c) m (DumpPackage Bool b c)
 addProfiling (InstalledCache ref) =
     CL.mapM go
   where
@@ -242,7 +234,7 @@ isProfiling content lib =
 -- | Add haddock information to the stream of @DumpPackage@s
 addHaddock :: MonadIO m
            => InstalledCache
-           -> Conduit (DumpPackage a b) m (DumpPackage a Bool)
+           -> Conduit (DumpPackage a b c) m (DumpPackage a Bool c)
 addHaddock (InstalledCache ref) =
     CL.mapM go
   where
@@ -262,10 +254,47 @@ addHaddock (InstalledCache ref) =
                 loop $ dpHaddockInterfaces dp
         return dp { dpHaddock = h }
 
+-- | Add debugging symbol information to the stream of @DumpPackage@s
+addSymbols :: MonadIO m
+           => InstalledCache
+           -> Conduit (DumpPackage a b c) m (DumpPackage a b Bool)
+addSymbols (InstalledCache ref) =
+    CL.mapM go
+  where
+    go dp = do
+        InstalledCacheInner m <- liftIO $ readIORef ref
+        let gid = dpGhcPkgId dp
+        s <- case Map.lookup gid m of
+            Just installed -> return (installedCacheSymbols installed)
+            Nothing | null (dpLibraries dp) -> return True
+            Nothing -> do
+                let lib = T.unpack . head $ dpLibraries dp
+                liftM or . mapM (\dir -> liftIO $ hasDebuggingSymbols dir lib) $ dpLibDirs dp
+        return dp { dpSymbols = s }
+
+hasDebuggingSymbols :: FilePath -- ^ library directory
+                    -> String   -- ^ name of library
+                    -> IO Bool
+hasDebuggingSymbols dir lib = do
+    let path = concat [dir, "/lib", lib, ".a"]
+    exists <- doesFileExist path
+    if not exists then return False
+    else case OS.buildOS of
+        OS.OSX     -> liftM (any (isPrefixOf "0x") . lines) $
+            readProcess "dwarfdump" [path] ""
+        OS.Linux   -> liftM (any (isPrefixOf "Contents") . lines) $
+            readProcess "readelf" ["--debug-dump=info", "--dwarf-depth=1", path] ""
+        OS.FreeBSD -> liftM (any (isPrefixOf "Contents") . lines) $
+            readProcess "readelf" ["--debug-dump=info", "--dwarf-depth=1", path] ""
+        OS.Windows -> return False -- No support, so it can't be there.
+        _          -> return False
+
+
 -- | Dump information for a single package
-data DumpPackage profiling haddock = DumpPackage
+data DumpPackage profiling haddock symbols = DumpPackage
     { dpGhcPkgId :: !GhcPkgId
     , dpPackageIdent :: !PackageIdentifier
+    , dpLicense :: !(Maybe C.License)
     , dpLibDirs :: ![FilePath]
     , dpLibraries :: ![Text]
     , dpHasExposedModules :: !Bool
@@ -274,9 +303,10 @@ data DumpPackage profiling haddock = DumpPackage
     , dpHaddockHtml :: !(Maybe FilePath)
     , dpProfiling :: !profiling
     , dpHaddock :: !haddock
+    , dpSymbols :: !symbols
     , dpIsExposed :: !Bool
     }
-    deriving (Show, Eq, Ord)
+    deriving (Show, Eq)
 
 data PackageDumpException
     = MissingSingleField Text (Map Text [Line])
@@ -284,20 +314,18 @@ data PackageDumpException
     deriving Typeable
 instance Exception PackageDumpException
 instance Show PackageDumpException where
-    show (MissingSingleField name values) = unlines $ concat
-        [ return $ concat
-            [ "Expected single value for field name "
-            , show name
-            , " when parsing ghc-pkg dump output:"
-            ]
-        , map (\(k, v) -> "    " ++ show (k, v)) (Map.toList values)
-        ]
+    show (MissingSingleField name values) = unlines $
+      return (concat
+        [ "Expected single value for field name "
+        , show name
+        , " when parsing ghc-pkg dump output:"
+        ]) ++ map (\(k, v) -> "    " ++ show (k, v)) (Map.toList values)
     show (Couldn'tParseField name ls) =
         "Couldn't parse the field " ++ show name ++ " from lines: " ++ show ls
 
 -- | Convert a stream of bytes into a stream of @DumpPackage@s
 conduitDumpPackage :: MonadThrow m
-                   => Conduit Text m (DumpPackage () ())
+                   => Conduit Text m (DumpPackage () () ())
 conduitDumpPackage = (=$= CL.catMaybes) $ eachSection $ do
     pairs <- eachPair (\k -> (k, ) <$> CL.consume) =$= CL.consume
     let m = Map.fromList pairs
@@ -333,6 +361,10 @@ conduitDumpPackage = (=$= CL.catMaybes) $ eachSection $ do
                 libraries = parseM "hs-libraries"
                 exposedModules = parseM "exposed-modules"
                 exposed = parseM "exposed"
+                license =
+                    case parseM "license" of
+                        [licenseText] -> C.simpleParse (T.unpack licenseText)
+                        _ -> Nothing
             depends <- mapMaybeM parseDepend $ concatMap T.words $ parseM "depends"
 
             let parseQuoted key =
@@ -348,6 +380,7 @@ conduitDumpPackage = (=$= CL.catMaybes) $ eachSection $ do
             return $ Just DumpPackage
                 { dpGhcPkgId = ghcPkgId
                 , dpPackageIdent = PackageIdentifier name version
+                , dpLicense = license
                 , dpLibDirs = libDirPaths
                 , dpLibraries = T.words $ T.unwords libraries
                 , dpHasExposedModules = not (null libraries || null exposedModules)
@@ -356,6 +389,7 @@ conduitDumpPackage = (=$= CL.catMaybes) $ eachSection $ do
                 , dpHaddockHtml = listToMaybe haddockHtml
                 , dpProfiling = ()
                 , dpHaddock = ()
+                , dpSymbols = ()
                 , dpIsExposed = exposed == ["True"]
                 }
 
