@@ -1,3 +1,4 @@
+{-# LANGUAGE ConstraintKinds       #-}
 {-# LANGUAGE FlexibleContexts      #-}
 {-# LANGUAGE MultiParamTypeClasses #-}
 {-# LANGUAGE OverloadedStrings     #-}
@@ -16,70 +17,77 @@ module Stack.Solver
     , parseCabalOutputLine
     ) where
 
-import Prelude ()
-import Prelude.Compat
+import           Prelude ()
+import           Prelude.Compat
 
 import           Control.Applicative
 import           Control.Exception (assert)
-import           Control.Exception.Enclosed  (tryIO)
-import           Control.Monad               (when,void,join,liftM,unless,zipWithM_)
+import           Control.Exception.Safe (tryIO)
+import           Control.Monad (when,void,join,liftM,unless,mapAndUnzipM, zipWithM_)
 import           Control.Monad.Catch
 import           Control.Monad.IO.Class
 import           Control.Monad.Logger
-import           Control.Monad.Reader        (MonadReader, asks)
-import           Control.Monad.Trans.Control
-import           Data.Aeson.Extended         ( WithJSONWarnings(..), object, (.=), toJSON
-                                             , logJSONWarnings)
-import qualified Data.ByteString             as S
-import           Data.Char                   (isSpace)
+import           Data.Aeson.Extended         (object, (.=), toJSON)
+import qualified Data.ByteString as S
+import           Data.Char (isSpace)
 import           Data.Either
-import           Data.Function               (on)
-import qualified Data.HashMap.Strict         as HashMap
+import           Data.Foldable (forM_)
+import           Data.Function (on)
+import qualified Data.HashMap.Strict as HashMap
+import qualified Data.HashSet as HashSet
 import           Data.List                   ( (\\), isSuffixOf, intercalate
                                              , minimumBy, isPrefixOf)
-import           Data.List.Extra             (groupSortOn)
-import           Data.Map                    (Map)
-import qualified Data.Map                    as Map
-import           Data.Maybe                  (catMaybes, isNothing, mapMaybe, fromMaybe)
+import           Data.List.Extra (groupSortOn)
+import           Data.Map (Map)
+import qualified Data.Map as Map
+import           Data.Maybe (catMaybes, isNothing, mapMaybe)
 import           Data.Monoid
-import           Data.Set                    (Set)
-import qualified Data.Set                    as Set
-import           Data.Text                   (Text)
-import qualified Data.Text                   as T
-import           Data.Text.Encoding          (decodeUtf8, encodeUtf8)
-import           Data.Text.Encoding.Error    (lenientDecode)
-import qualified Data.Text.Lazy              as LT
-import           Data.Text.Lazy.Encoding     (decodeUtf8With)
-import           Data.Tuple                  (swap)
-import qualified Data.Yaml                   as Yaml
-import qualified Distribution.Package        as C
+import           Data.Set (Set)
+import qualified Data.Set as Set
+import           Data.Text (Text)
+import qualified Data.Text as T
+import           Data.Text.Encoding (decodeUtf8, encodeUtf8)
+import           Data.Text.Encoding.Error (lenientDecode)
+import           Data.Text.Extra (stripCR)
+import qualified Data.Text.Lazy as LT
+import           Data.Text.Lazy.Encoding (decodeUtf8With)
+import           Data.Tuple (swap)
+import qualified Data.Yaml as Yaml
+import qualified Distribution.Package as C
 import qualified Distribution.PackageDescription as C
-import qualified Distribution.Text           as C
-import           Network.HTTP.Client.Conduit (HasHttpManager)
-import           Text.Regex.Applicative.Text (match, sym, psym, anySym, few)
+import qualified Distribution.Text as C
 import           Path
-import           Path.Find                   (findFiles)
-import           Path.IO                     hiding (findExecutable, findFiles)
+import           Path.Find (findFiles)
+import           Path.IO hiding (findExecutable, findFiles)
 import           Stack.BuildPlan
-import           Stack.Constants             (stackDotYaml)
+import           Stack.Config (getLocalPackages, loadConfigYaml)
+import           Stack.Constants (stackDotYaml, wiredInPackages)
 import           Stack.Package               (printCabalFileWarning
                                              , hpack
                                              , readPackageUnresolved)
+import           Stack.PrettyPrint
 import           Stack.Setup
 import           Stack.Setup.Installed
-import           Stack.Types
-import           Stack.Types.Internal        ( HasTerminal
-                                             , HasReExec
-                                             , HasLogLevel)
-import qualified System.Directory            as D
-import qualified System.FilePath             as FP
+import           Stack.Types.Build
+import           Stack.Types.Compiler
+import           Stack.Types.Config
+import           Stack.Types.FlagName
+import           Stack.Types.PackageIdentifier
+import           Stack.Types.PackageName
+import           Stack.Types.Resolver
+import           Stack.Types.StackT (StackM)
+import           Stack.Types.Version
+import qualified System.Directory as D
+import qualified System.FilePath as FP
 import           System.Process.Read
+import           Text.Regex.Applicative.Text (match, sym, psym, anySym, few)
 
+import qualified Data.Text.Normalize as T ( normalize , NormalizationMode(NFC) )
 
 data ConstraintType = Constraint | Preference deriving (Eq)
 type ConstraintSpec = Map PackageName (Version, Map FlagName Bool)
 
-cabalSolver :: (MonadIO m, MonadLogger m, MonadMask m, MonadBaseControl IO m, MonadReader env m, HasConfig env)
+cabalSolver :: (StackM env m, HasConfig env)
             => EnvOverride
             -> [Path Abs Dir] -- ^ cabal files
             -> ConstraintType
@@ -120,12 +128,13 @@ cabalSolver menv cabalfps constraintType
 
     catch (liftM Right (readProcessStdout (Just tmpdir) menv "cabal" args))
           (\ex -> case ex of
-              ReadProcessException _ _ _ err -> return $ Left err
+              ProcessFailed _ _ _ err -> return $ Left err
               _ -> throwM ex)
     >>= either parseCabalErrors parseCabalOutput
 
   where
     errCheck = T.isInfixOf "Could not resolve dependencies"
+    linesNoCR = map stripCR . T.lines
     cabalBuildErrMsg e =
                ">>>> Cabal errors begin\n"
             <> e
@@ -146,10 +155,10 @@ cabalSolver menv cabalfps constraintType
 
             when (any isNothing mPkgNames) $ do
                   $logInfo $ "*** Only some package names could be parsed: " <>
-                      (T.pack (intercalate ", " (map show pkgNames)))
+                      T.pack (intercalate ", " (map show pkgNames))
                   error $ T.unpack $
                        "*** User packages involved in cabal failure: "
-                       <> (T.intercalate ", " $ parseConflictingPkgs msg)
+                       <> T.intercalate ", " (parseConflictingPkgs msg)
 
             if pkgNames /= [] then do
                   return $ Left pkgNames
@@ -157,30 +166,27 @@ cabalSolver menv cabalfps constraintType
         else errExit msg
 
     parseConflictingPkgs msg =
-        let ls = dropWhile (not . errCheck) $ T.lines msg
-            select s = ((T.isPrefixOf "trying:" s)
-                      || (T.isPrefixOf "next goal:" s))
-                      && (T.isSuffixOf "(user goal)" s)
-            pkgName =   (take 1)
+        let ls = dropWhile (not . errCheck) $ linesNoCR msg
+            select s = (T.isPrefixOf "trying:" s
+                      || T.isPrefixOf "next goal:" s)
+                      && T.isSuffixOf "(user goal)" s
+            pkgName =   take 1
                       . T.words
-                      . (T.drop 1)
-                      . (T.dropWhile (/= ':'))
-        in concat $ map pkgName (filter select ls)
+                      . T.drop 1
+                      . T.dropWhile (/= ':')
+        in concatMap pkgName (filter select ls)
 
     parseCabalOutput bs = do
         let ls = drop 1
                $ dropWhile (not . T.isPrefixOf "In order, ")
-               $ map stripCR
-               $ T.lines
+               $ linesNoCR
                $ decodeUtf8 bs
             (errs, pairs) = partitionEithers $ map parseCabalOutputLine ls
         if null errs
           then return $ Right (Map.fromList pairs)
           else error $ "The following lines from cabal-install output could \
                        \not be parsed: \n"
-                       ++ (T.unpack (T.intercalate "\n" errs))
-
-    stripCR t = fromMaybe t (T.stripSuffix "\r" t)
+                       ++ T.unpack (T.intercalate "\n" errs)
 
     toConstraintArgs userFlagMap =
         [formatFlagConstraint package flag enabled
@@ -228,13 +234,13 @@ parseCabalOutputLine t0 = maybe (Left t0) Right . join .  match re $ t0
 
     lexeme r = some (psym isSpace) *> r
 
-getCabalConfig :: (MonadLogger m, MonadReader env m, HasConfig env, MonadIO m, MonadThrow m)
+getCabalConfig :: (StackM env m, HasConfig env)
                => FilePath -- ^ temp dir
                -> ConstraintType
                -> Map PackageName Version -- ^ constraints
                -> m [Text]
 getCabalConfig dir constraintType constraints = do
-    indices <- asks $ configPackageIndices . getConfig
+    indices <- view $ configL.to configPackageIndices
     remotes <- mapM goIndex indices
     let cache = T.pack $ "remote-repo-cache: " ++ dir
     return $ cache : remotes ++ map goConstraint (Map.toList constraints)
@@ -242,10 +248,15 @@ getCabalConfig dir constraintType constraints = do
     goIndex index = do
         src <- configPackageIndex $ indexName index
         let dstdir = dir FP.</> T.unpack (indexNameText $ indexName index)
-            dst = dstdir FP.</> "00-index.tar"
+            -- NOTE: see https://github.com/commercialhaskell/stack/issues/2888
+            -- for why we are pretending that a 01-index.tar is actually a
+            -- 00-index.tar file.
+            dst0 = dstdir FP.</> "00-index.tar"
+            dst1 = dstdir FP.</> "01-index.tar"
         liftIO $ void $ tryIO $ do
             D.createDirectoryIfMissing True dstdir
-            D.copyFile (toFilePath src) dst
+            D.copyFile (toFilePath src) dst0
+            D.copyFile (toFilePath src) dst1
         return $ T.concat
             [ "remote-repo: "
             , indexNameText $ indexName index
@@ -255,19 +266,17 @@ getCabalConfig dir constraintType constraints = do
     goConstraint (name, version) =
         assert (not . null . versionString $ version) $
             T.concat
-              [ (if constraintType == Constraint
-                 then "constraint: "
-                 else "preference: ")
+              [ if constraintType == Constraint
+                   || name `HashSet.member` wiredInPackages
+                then "constraint: "
+                else "preference: "
               , T.pack $ packageNameString name
               , "=="
               , T.pack $ versionString version
               ]
 
 setupCompiler
-    :: ( MonadBaseControl IO m, MonadIO m, MonadLogger m, MonadMask m
-       , MonadReader env m, HasConfig env , HasGHCVariant env
-       , HasHttpManager env , HasLogLevel env , HasReExec env
-       , HasTerminal env)
+    :: (StackM env m, HasConfig env, HasGHCVariant env)
     => CompilerVersion
     -> m (Maybe ExtraDirs)
 setupCompiler compiler = do
@@ -278,31 +287,27 @@ setupCompiler compiler = do
           , "install the compiler or '--system-ghc' to use a suitable "
           , "compiler available on your PATH." ]
 
-    config <- asks getConfig
-    mpaths <- ensureCompiler SetupOpts
+    config <- view configL
+    (dirs, _, _) <- ensureCompiler SetupOpts
         { soptsInstallIfMissing  = configInstallGHC config
         , soptsUseSystem         = configSystemGHC config
         , soptsWantedCompiler    = compiler
         , soptsCompilerCheck     = configCompilerCheck config
-
         , soptsStackYaml         = Nothing
         , soptsForceReinstall    = False
         , soptsSanityCheck       = False
         , soptsSkipGhcCheck      = False
         , soptsSkipMsys          = configSkipMsys config
-        , soptsUpgradeCabal      = False
+        , soptsUpgradeCabal      = Nothing
         , soptsResolveMissingGHC = msg
-        , soptsStackSetupYaml    = defaultStackSetupYaml
+        , soptsSetupInfoYaml     = defaultSetupInfoYaml
         , soptsGHCBindistURL     = Nothing
+        , soptsGHCJSBootOpts     = ["--clean"]
         }
-
-    return mpaths
+    return dirs
 
 setupCabalEnv
-    :: ( MonadBaseControl IO m, MonadIO m, MonadLogger m, MonadMask m
-       , MonadReader env m, HasConfig env , HasGHCVariant env
-       , HasHttpManager env , HasLogLevel env , HasReExec env
-       , HasTerminal env)
+    :: (StackM env m, HasConfig env, HasGHCVariant env)
     => CompilerVersion
     -> m EnvOverride
 setupCabalEnv compiler = do
@@ -311,13 +316,23 @@ setupCabalEnv compiler = do
     envMap <- removeHaskellEnvVars
               <$> augmentPathMap (maybe [] edBins mpaths)
                                  (unEnvOverride menv0)
-    platform <- asks getPlatform
+    platform <- view platformL
     menv <- mkEnvOverride platform envMap
 
-    mcabal <- findExecutable menv "cabal"
+    mcabal <- getCabalInstallVersion menv
     case mcabal of
         Nothing -> throwM SolverMissingCabalInstall
-        Just _ -> return ()
+        Just version
+            | version < $(mkVersion "1.24") -> $prettyWarn $
+                "Installed version of cabal-install (" <>
+                display version <>
+                ") doesn't support custom-setup clause, and so may not yield correct results." <> line <>
+                "To resolve this, install a newer version via 'stack install cabal-install'." <> line
+            | version >= $(mkVersion "1.25") -> $prettyWarn $
+                "Installed version of cabal-install (" <>
+                display version <>
+                ") is newer than stack has been tested with.  If you run into difficulties, consider downgrading." <> line
+            | otherwise -> return ()
 
     mver <- getSystemCompiler menv (whichCompiler compiler)
     case mver of
@@ -356,10 +371,7 @@ mergeConstraints = Map.mergeWithKey
 -- or the solution in terms of src package flag settings and extra
 -- dependencies.
 solveResolverSpec
-    :: ( MonadBaseControl IO m, MonadIO m, MonadLogger m, MonadMask m
-       , MonadReader env m, HasConfig env , HasGHCVariant env
-       , HasHttpManager env , HasLogLevel env , HasReExec env
-       , HasTerminal env)
+    :: (StackM env m, HasConfig env, HasGHCVariant env)
     => Path Abs File  -- ^ stack.yaml file location
     -> [Path Abs Dir] -- ^ package dirs containing cabal files
     -> ( Resolver
@@ -385,15 +397,14 @@ solveResolverSpec stackYaml cabalDirs
         -- 1. We do not want snapshot versions to override the sources
         -- 2. Sources may have blank versions leading to bad cabal constraints
         depOnlyConstraints = Map.difference depConstraints srcConstraints
-        solver t = cabalSolver menv cabalDirs t
-                               srcConstraints depOnlyConstraints $
-                          ["-v"] -- TODO make it conditional on debug
-                       ++ ["--ghcjs" | (whichCompiler compilerVer) == Ghcjs]
+        solver t = cabalSolver menv cabalDirs t srcConstraints depOnlyConstraints $
+                     "-v" : -- TODO make it conditional on debug
+                     ["--ghcjs" | whichCompiler compilerVer == Ghcjs]
 
-    let srcNames = (T.intercalate " and ") $
+    let srcNames = T.intercalate " and " $
           ["packages from " <> resolverName resolver
               | not (Map.null snapConstraints)] ++
-          [T.pack ((show $ Map.size extraConstraints) <> " external packages")
+          [T.pack (show (Map.size extraConstraints) <> " external packages")
               | not (Map.null extraConstraints)]
 
     $logInfo "Asking cabal to calculate a build plan..."
@@ -432,13 +443,13 @@ solveResolverSpec stackYaml cabalDirs
                 inVers   = fmap fst srcConstraints
                 bothVers = Map.intersectionWith (\v1 v2 -> (v1, v2))
                                                 inVers outVers
-            when (not $ outVers `Map.isSubmapOf` inVers) $ do
+            unless (outVers `Map.isSubmapOf` inVers) $ do
                 let msg = "Error: user package versions returned by cabal \
                           \solver are not the same as the versions in the \
                           \cabal files:\n"
                 -- TODO We can do better in formatting the message
                 error $ T.unpack $ msg
-                        <> (showItems $ map show (Map.toList bothVers))
+                        <> showItems (map show (Map.toList bothVers))
 
             $logInfo $ "Successfully determined a build plan with "
                      <> T.pack (show $ Map.size external)
@@ -467,9 +478,7 @@ solveResolverSpec stackYaml cabalDirs
 -- return the compiler version, package versions and packages flags
 -- for that resolver.
 getResolverConstraints
-    :: ( MonadBaseControl IO m, MonadIO m, MonadLogger m, MonadMask m
-       , MonadReader env m, HasConfig env , HasGHCVariant env
-       , HasHttpManager env)
+    :: (StackM env m, HasConfig env, HasGHCVariant env)
     => Path Abs File
     -> Resolver
     -> m (CompilerVersion,
@@ -488,9 +497,7 @@ getResolverConstraints stackYaml resolver = do
 -- If the package flags are passed as 'Nothing' then flags are chosen
 -- automatically.
 checkResolverSpec
-    :: ( MonadIO m, MonadMask m, MonadLogger m, MonadReader env m
-       , HasHttpManager env, HasConfig env, HasGHCVariant env
-       , MonadBaseControl IO m)
+    :: (StackM env m, HasConfig env, HasGHCVariant env)
     => [C.GenericPackageDescription]
     -> Maybe (Map PackageName (Map FlagName Bool))
     -> Resolver
@@ -533,10 +540,7 @@ ignoredDirs = Set.fromList
 -- pairs as well as any filenames for duplicate packages not included in the
 -- pairs.
 cabalPackagesCheck
-    :: ( MonadBaseControl IO m, MonadIO m, MonadLogger m, MonadMask m
-       , MonadReader env m, HasConfig env , HasGHCVariant env
-       , HasHttpManager env , HasLogLevel env , HasReExec env
-       , HasTerminal env)
+    :: (StackM env m, HasConfig env, HasGHCVariant env)
      => [Path Abs File]
      -> String
      -> Maybe String
@@ -550,7 +554,7 @@ cabalPackagesCheck cabalfps noPkgMsg dupErrMsg = do
     $logInfo $ "Using cabal packages:"
     $logInfo $ T.pack (formatGroup relpaths)
 
-    (warnings, gpds) <- fmap unzip (mapM readPackageUnresolved cabalfps)
+    (warnings, gpds) <- mapAndUnzipM readPackageUnresolved cabalfps
     zipWithM_ (mapM_ . printCabalFileWarning) cabalfps warnings
 
     -- package name cannot be empty or missing otherwise
@@ -559,8 +563,9 @@ cabalPackagesCheck cabalfps noPkgMsg dupErrMsg = do
     -- Just the latter check is enough to cover both the cases
 
     let packages  = zip cabalfps gpds
+        normalizeString = T.unpack . T.normalize T.NFC . T.pack
         getNameMismatchPkg (fp, gpd)
-            | (show . gpdPackageName) gpd /= (FP.takeBaseName . toFilePath) fp
+            | (normalizeString . show . gpdPackageName) gpd /= (normalizeString . FP.takeBaseName . toFilePath) fp
                 = Just fp
             | otherwise = Nothing
         nameMismatchPkgs = mapMaybe getNameMismatchPkg packages
@@ -570,7 +575,7 @@ cabalPackagesCheck cabalfps noPkgMsg dupErrMsg = do
         error $ "Package name as defined in the .cabal file must match the \
                 \.cabal file name.\n\
                 \Please fix the following packages and try again:\n"
-                <> (formatGroup rels)
+                <> formatGroup rels
 
     let dupGroups = filter ((> 1) . length)
                             . groupSortOn (gpdPackageName . snd)
@@ -623,20 +628,18 @@ reportMissingCabalFiles cabalfps includeSubdirs = do
 -- dependencies in an existing stack.yaml and suggest changes in flags or
 -- extra dependencies so that the specified packages can be compiled.
 solveExtraDeps
-    :: ( MonadBaseControl IO m, MonadIO m, MonadLogger m, MonadMask m
-       , MonadReader env m, HasEnvConfig env, HasHttpManager env
-       , HasLogLevel env, HasReExec env, HasTerminal env)
+    :: (StackM env m, HasEnvConfig env)
     => Bool -- ^ modify stack.yaml?
     -> m ()
 solveExtraDeps modStackYaml = do
-    econfig <- asks getEnvConfig
-    bconfig <- asks getBuildConfig
+    bconfig <- view buildConfigL
 
     let stackYaml = bcStackYaml bconfig
-    relStackYaml <- toFilePath <$> makeRelativeToCurrentDir stackYaml
+    relStackYaml <- prettyPath stackYaml
 
     $logInfo $ "Using configuration file: " <> T.pack relStackYaml
-    let cabalDirs = Map.keys $ envConfigPackages econfig
+    packages <- getLocalPackages
+    let cabalDirs = Map.keys packages
         noPkgMsg = "No cabal packages found in " <> relStackYaml <>
                    ". Please add at least one directory containing a .cabal \
                    \file. You can also use 'stack init' to automatically \
@@ -665,7 +668,7 @@ solveExtraDeps modStackYaml = do
     resolverResult <- checkResolverSpec gpds (Just oldSrcFlags) resolver'
     resultSpecs <- case resolverResult of
         BuildPlanCheckOk flags ->
-            return $ Just ((mergeConstraints oldSrcs flags), Map.empty)
+            return $ Just (mergeConstraints oldSrcs flags, Map.empty)
         BuildPlanCheckPartial {} -> do
             eres <- solveResolverSpec stackYaml cabalDirs
                               (resolver', srcConstraints, extraConstraints)
@@ -679,6 +682,8 @@ solveExtraDeps modStackYaml = do
         Nothing -> throwM (SolverGiveUp giveUpMsg)
         Just x -> return x
 
+    mOldResolver <- view $ configL.to (fmap (projectResolver . fst) . configMaybeProject)
+
     let
         flags = removeSrcPkgDefaultFlags gpds (fmap snd (Map.union srcs edeps))
         versions = fmap fst edeps
@@ -695,14 +700,14 @@ solveExtraDeps modStackYaml = do
 
         changed =    any (not . Map.null) [newVersions, goneVersions]
                   || any (not . Map.null) [newFlags, goneFlags]
+                  || any (/= resolver') mOldResolver
 
     if changed then do
         $logInfo ""
         $logInfo $ "The following changes will be made to "
                    <> T.pack relStackYaml <> ":"
 
-        -- TODO print whether resolver changed from previous
-        $logInfo $ "* Resolver is " <> resolverName resolver
+        printResolver mOldResolver resolver'
 
         printFlags newFlags  "* Flags to be added"
         printDeps  newVersions   "* Dependencies to be added"
@@ -721,26 +726,35 @@ solveExtraDeps modStackYaml = do
         $logInfo $ "No changes needed to " <> T.pack relStackYaml
 
     where
-        indent t = T.unlines $ fmap ("    " <>) (T.lines t)
+        indentLines t = T.unlines $ fmap ("    " <>) (T.lines t)
+
+        printResolver mOldRes res = do
+            forM_ mOldRes $ \oldRes ->
+                when (res /= oldRes) $ do
+                    $logInfo $ T.concat
+                        [ "* Resolver changes from "
+                        , resolverName oldRes
+                        , " to "
+                        , resolverName res
+                        ]
 
         printFlags fl msg = do
-            when ((not . Map.null) fl) $ do
+            unless (Map.null fl) $ do
                 $logInfo $ T.pack msg
-                $logInfo $ indent $ decodeUtf8 $ Yaml.encode
-                                  $ object ["flags" .= fl]
+                $logInfo $ indentLines $ decodeUtf8 $ Yaml.encode
+                                       $ object ["flags" .= fl]
 
         printDeps deps msg = do
-            when ((not . Map.null) deps) $ do
+            unless (Map.null deps) $ do
                 $logInfo $ T.pack msg
-                $logInfo $ indent $ decodeUtf8 $ Yaml.encode $ object $
-                        [("extra-deps" .= map fromTuple (Map.toList deps))]
+                $logInfo $ indentLines $ decodeUtf8 $ Yaml.encode $ object
+                        ["extra-deps" .= map fromTuple (Map.toList deps)]
 
         writeStackYaml path res deps fl = do
             let fp = toFilePath path
             obj <- liftIO (Yaml.decodeFileEither fp) >>= either throwM return
-            WithJSONWarnings (ProjectAndConfigMonoid _ _) warnings <-
-                liftIO (Yaml.decodeFileEither fp) >>= either throwM return
-            logJSONWarnings fp warnings
+            -- Check input file and show warnings
+            _ <- loadConfigYaml (parseProjectAndConfigMonoid (parent path)) path
             let obj' =
                     HashMap.insert "extra-deps"
                         (toJSON $ map fromTuple $ Map.toList deps)
@@ -754,6 +768,7 @@ solveExtraDeps modStackYaml = do
             , "        - Remove any unnecessary packages.\n"
             , "        - Add any missing remote packages.\n"
             , "        - Add extra dependencies to guide solver.\n"
+            , "        - Adjust resolver.\n"
             ]
 
 prettyPath
