@@ -5,7 +5,6 @@
 {-# LANGUAGE GeneralizedNewtypeDeriving #-}
 {-# LANGUAGE OverloadedStrings          #-}
 {-# LANGUAGE TemplateHaskell            #-}
-{-# LANGUAGE ViewPatterns               #-}
 
 -- | Build-specific types.
 
@@ -19,6 +18,7 @@ module Stack.Types.Build
     ,Installed(..)
     ,PackageInstallInfo(..)
     ,Task(..)
+    ,taskIsTarget
     ,taskLocation
     ,LocalPackage(..)
     ,BaseConfigOpts(..)
@@ -31,48 +31,61 @@ module Stack.Types.Build
     ,defaultBuildOpts
     ,TaskType(..)
     ,TaskConfigOpts(..)
+    ,BuildCache(..)
+    ,buildCacheVC
     ,ConfigCache(..)
-    ,ConstructPlanException(..)
+    ,configCacheVC
     ,configureOpts
+    ,CachePkgSrc (..)
+    ,toCachePkgSrc
     ,isStackOpt
-    ,BadDependency(..)
     ,wantedLocalPackages
     ,FileCacheInfo (..)
     ,ConfigureOpts (..)
-    ,PrecompiledCache (..))
+    ,PrecompiledCache (..)
+    ,precompiledCacheVC)
     where
 
 import           Control.DeepSeq
 import           Control.Exception
-
-import           Data.Binary (getWord8, putWord8, gput, gget)
-import           Data.Binary.VersionTagged
-import qualified Data.ByteString as S
-import           Data.Char (isSpace)
+import           Data.Binary                     (Binary)
+import           Data.Binary.Tagged              (HasSemanticVersion,
+                                                  HasStructuralInfo)
+import qualified Data.ByteString                 as S
+import           Data.Char                       (isSpace)
 import           Data.Data
 import           Data.Hashable
-import           Data.List (dropWhileEnd, nub, intercalate)
-import qualified Data.Map as Map
-import           Data.Map.Strict (Map)
+import           Data.List.Extra
+import qualified Data.Map                        as Map
+import           Data.Map.Strict                 (Map)
 import           Data.Maybe
 import           Data.Monoid
-import           Data.Set (Set)
-import qualified Data.Set as Set
-import           Data.Text (Text)
-import qualified Data.Text as T
-import           Data.Text.Encoding (decodeUtf8With)
-import           Data.Text.Encoding.Error (lenientDecode)
+import           Data.Set                        (Set)
+import qualified Data.Set                        as Set
+import           Data.Store.Internal             (Store)
+import           Data.Store.Version
+import           Data.Store.VersionTagged
+import           Data.Text                       (Text)
+import qualified Data.Text                       as T
+import           Data.Text.Encoding              (decodeUtf8With)
+import           Data.Text.Encoding.Error        (lenientDecode)
 import           Data.Time.Calendar
 import           Data.Time.Clock
-import           Distribution.System (Arch)
+import           Data.Version                    (showVersion)
 import           Distribution.PackageDescription (TestSuiteInterface)
-import           Distribution.Text (display)
-import           GHC.Generics (Generic, from, to)
-import           Path (Path, Abs, File, Dir, mkRelDir, toFilePath, parseRelDir, (</>))
-import           Path.Extra (toFilePathNoTrailingSep)
+import           Distribution.System             (Arch)
+import qualified Distribution.Text               as C
+import           GHC.Generics                    (Generic)
+import           Path                            (Abs, Dir, File, Path,
+                                                  mkRelDir, parseRelDir,
+                                                  toFilePath, (</>))
+import           Path.Extra                      (toFilePathNoTrailingSep)
+import           Paths_stack                     as Meta
 import           Prelude
-import           Stack.Types.BuildPlan (GitSHA1)
+import           Stack.Constants
+import           Stack.Types.BuildPlan           (GitSHA1)
 import           Stack.Types.Compiler
+import           Stack.Types.CompilerBuild
 import           Stack.Types.Config
 import           Stack.Types.FlagName
 import           Stack.Types.GhcPkgId
@@ -80,22 +93,22 @@ import           Stack.Types.Package
 import           Stack.Types.PackageIdentifier
 import           Stack.Types.PackageName
 import           Stack.Types.Version
-import           System.Exit (ExitCode (ExitFailure))
-import           System.FilePath (pathSeparator)
-import           System.Process.Log (showProcessArgDebug)
+import           System.Exit                     (ExitCode (ExitFailure))
+import           System.FilePath                 (pathSeparator)
+import           System.Process.Log              (showProcessArgDebug)
 
 ----------------------------------------------
 -- Exceptions
 data StackBuildException
   = Couldn'tFindPkgId PackageName
   | CompilerVersionMismatch
-        (Maybe (CompilerVersion, Arch))
-        (CompilerVersion, Arch)
-        GHCVariant
+        (Maybe (CompilerVersion, Arch)) -- found
+        (CompilerVersion, Arch) -- expected
+        GHCVariant -- expected
+        CompilerBuild -- expected
         VersionCheck
-        (Maybe (Path Abs File))
+        (Maybe (Path Abs File)) -- Path to the stack.yaml file
         Text -- recommended resolution
-  -- ^ Path to the stack.yaml file
   | Couldn'tParseTargets [Text]
   | UnknownTargets
     (Set PackageName) -- no known version
@@ -103,9 +116,7 @@ data StackBuildException
     (Path Abs File) -- stack.yaml
   | TestSuiteFailure PackageIdentifier (Map Text (Maybe ExitCode)) (Maybe (Path Abs File)) S.ByteString
   | TestSuiteTypeUnsupported TestSuiteInterface
-  | ConstructPlanExceptions
-        [ConstructPlanException]
-        (Path Abs File) -- stack.yaml
+  | ConstructPlanFailed String
   | CabalExitedUnsuccessfully
         ExitCode
         PackageIdentifier
@@ -127,6 +138,7 @@ data StackBuildException
   | SomeTargetsNotBuildable [(PackageName, NamedComponent)]
   | TestSuiteExeMissing Bool String String String
   | CabalCopyFailed Bool String
+  | LocalPackagesPresent [PackageIdentifier]
   deriving Typeable
 
 data FlagSource = FSCommandLine | FSStackYaml
@@ -139,18 +151,18 @@ data UnusedFlags = UFNoPackage FlagSource PackageName
 
 instance Show StackBuildException where
     show (Couldn'tFindPkgId name) =
-              ("After installing " <> packageNameString name <>
+              "After installing " <> packageNameString name <>
                ", the package id couldn't be found " <> "(via ghc-pkg describe " <>
                packageNameString name <> "). This shouldn't happen, " <>
-               "please report as a bug")
-    show (CompilerVersionMismatch mactual (expected, earch) ghcVariant check mstack resolution) = concat
+               "please report as a bug"
+    show (CompilerVersionMismatch mactual (expected, earch) ghcVariant ghcBuild check mstack resolution) = concat
                 [ case mactual of
                     Nothing -> "No compiler found, expected "
                     Just (actual, arch) -> concat
                         [ "Compiler version mismatched, found "
                         , compilerVersionString actual
                         , " ("
-                        , display arch
+                        , C.display arch
                         , ")"
                         , ", but expected "
                         ]
@@ -160,8 +172,9 @@ instance Show StackBuildException where
                     NewerMinor -> "minor version match or newer with "
                 , compilerVersionString expected
                 , " ("
-                , display earch
+                , C.display earch
                 , ghcVariantSuffix ghcVariant
+                , compilerBuildSuffix ghcBuild
                 , ") (based on "
                 , case mstack of
                     Nothing -> "command line arguments"
@@ -179,7 +192,10 @@ instance Show StackBuildException where
             | Set.null noKnown = []
             | otherwise = return $
                 "The following target packages were not found: " ++
-                intercalate ", " (map packageNameString $ Set.toList noKnown)
+                intercalate ", " (map packageNameString $ Set.toList noKnown) ++
+                "\nSee https://docs.haskellstack.org/en/v"
+                <> showVersion Meta.version <>
+                "/build_command/#target-syntax for details."
         notInSnapshot'
             | Map.null notInSnapshot = []
             | otherwise =
@@ -190,8 +206,8 @@ instance Show StackBuildException where
                 : "but there's no guarantee that they'll build together)."
                 : ""
                 : map
-                    (\(name, version) -> "- " ++ packageIdentifierString
-                        (PackageIdentifier name version))
+                    (\(name, version') -> "- " ++ packageIdentifierString
+                        (PackageIdentifier name version'))
                     (Map.toList notInSnapshot)
     show (TestSuiteFailure ident codes mlogFile bs) = unlines $ concat
         [ ["Test suite failure for package " ++ packageIdentifierString ident]
@@ -215,35 +231,7 @@ instance Show StackBuildException where
           indent = dropWhileEnd isSpace . unlines . fmap (\line -> "  " ++ line) . lines
           doubleIndent = indent . indent
     show (TestSuiteTypeUnsupported interface) =
-              ("Unsupported test suite type: " <> show interface)
-    show (ConstructPlanExceptions exceptions stackYaml) =
-        "While constructing the BuildPlan the following exceptions were encountered:" ++
-        appendExceptions exceptions' ++
-        if Map.null extras then "" else (unlines
-                $ ("\n\nRecommended action: try adding the following to your extra-deps in "
-                    ++ toFilePath stackYaml)
-                : map (\(name, version) -> concat
-                    [ "- "
-                    , packageNameString name
-                    , "-"
-                    , versionString version
-                    ]) (Map.toList extras)
-                    ++ ["", "You may also want to try the 'stack solver' command"]
-                )
-         where
-             exceptions' = removeDuplicates exceptions
-             appendExceptions = foldr (\e -> (++) ("\n\n--" ++ show e)) ""
-             removeDuplicates = nub
-             extras = Map.unions $ map getExtras exceptions'
-
-             getExtras (DependencyCycleDetected _) = Map.empty
-             getExtras (UnknownPackage _) = Map.empty
-             getExtras (DependencyPlanFailures _ m) =
-                Map.unions $ map go $ Map.toList m
-              where
-                go (name, (_range, Just version, NotInBuildPlan)) =
-                    Map.singleton name version
-                go _ = Map.empty
+              "Unsupported test suite type: " <> show interface
      -- Supressing duplicate output
     show (CabalExitedUnsuccessfully exitCode taskProvides' execName fullArgs logFiles bss) =
         let fullCmd = unwords
@@ -353,6 +341,10 @@ instance Show StackBuildException where
             , innerMsg
             , "\n"
             ]
+    show (ConstructPlanFailed msg) = msg
+    show (LocalPackagesPresent locals) = unlines
+      $ "Local packages are not allowed when using the script command. Packages found:"
+      : map (\ident -> "- " ++ packageIdentifierString ident) locals
 
 missingExeError :: Bool -> String -> String
 missingExeError isSimpleBuildType msg =
@@ -370,90 +362,26 @@ missingExeError isSimpleBuildType msg =
 
 instance Exception StackBuildException
 
-data ConstructPlanException
-    = DependencyCycleDetected [PackageName]
-    | DependencyPlanFailures Package (Map PackageName (VersionRange, LatestApplicableVersion, BadDependency))
-    | UnknownPackage PackageName -- TODO perhaps this constructor will be removed, and BadDependency will handle it all
-    -- ^ Recommend adding to extra-deps, give a helpful version number?
-    deriving (Typeable, Eq)
-
--- | For display purposes only, Nothing if package not found
-type LatestApplicableVersion = Maybe Version
-
--- | Reason why a dependency was not used
-data BadDependency
-    = NotInBuildPlan
-    | Couldn'tResolveItsDependencies
-    | DependencyMismatch Version
-    deriving (Typeable, Eq)
-
-instance Show ConstructPlanException where
-  show e =
-    let details = case e of
-         (DependencyCycleDetected pNames) ->
-           "While checking call stack,\n" ++
-           "  dependency cycle detected in packages:" ++ indent (appendLines pNames)
-         (DependencyPlanFailures pkg (Map.toList -> pDeps)) ->
-           "Failure when adding dependencies:" ++ doubleIndent (appendDeps pDeps) ++ "\n" ++
-           "  needed for package " ++ packageIdentifierString (packageIdentifier pkg) ++
-           appendFlags (packageFlags pkg)
-         (UnknownPackage pName) ->
-             "While attempting to add dependency,\n" ++
-             "  Could not find package " ++ show pName  ++ " in known packages"
-    in indent details
-     where
-      appendLines = foldr (\pName-> (++) ("\n" ++ show pName)) ""
-      indent = dropWhileEnd isSpace . unlines . fmap (\line -> "  " ++ line) . lines
-      doubleIndent = indent . indent
-      appendFlags flags =
-          if Map.null flags
-              then ""
-              else " with flags:\n" ++
-                  (doubleIndent . intercalate "\n" . map showFlag . Map.toList) flags
-      showFlag (name, bool) = show name ++ ": " ++ show bool
-      appendDeps = foldr (\dep-> (++) ("\n" ++ showDep dep)) ""
-      showDep (name, (range, mlatestApplicable, badDep)) = concat
-        [ show name
-        , ": needed ("
-        , display range
-        , ")"
-        , ", "
-        , let latestApplicableStr =
-                case mlatestApplicable of
-                    Nothing -> ""
-                    Just la -> " (latest applicable is " ++ versionString la ++ ")"
-           in case badDep of
-                NotInBuildPlan -> "stack configuration has no specified version" ++ latestApplicableStr
-                Couldn'tResolveItsDependencies -> "couldn't resolve its dependencies"
-                DependencyMismatch version ->
-                    case mlatestApplicable of
-                        Just la
-                            | la == version ->
-                                versionString version ++
-                                " found (latest applicable version available)"
-                        _ -> versionString version ++ " found" ++ latestApplicableStr
-        ]
-         {- TODO Perhaps change the showDep function to look more like this:
-          dropQuotes = filter ((/=) '\"')
-         (VersionOutsideRange pName pIdentifier versionRange) ->
-             "Exception: Stack.Build.VersionOutsideRange\n" ++
-             "  While adding dependency for package " ++ show pName ++ ",\n" ++
-             "  " ++ dropQuotes (show pIdentifier) ++ " was found to be outside its allowed version range.\n" ++
-             "  Allowed version range is " ++ display versionRange ++ ",\n" ++
-             "  should you correct the version range for " ++ dropQuotes (show pIdentifier) ++ ", found in [extra-deps] in the project's stack.yaml?"
-             -}
-
-
 ----------------------------------------------
 
-
 -- | Package dependency oracle.
 newtype PkgDepsOracle =
     PkgDeps PackageName
-    deriving (Show,Typeable,Eq,Hashable,Binary,NFData)
+    deriving (Show,Typeable,Eq,Hashable,Store,NFData)
 
--- | Stored on disk to know whether the flags have changed or any
--- files have changed.
+-- | Stored on disk to know whether the files have changed.
+newtype BuildCache = BuildCache
+    { buildCacheTimes :: Map FilePath FileCacheInfo
+      -- ^ Modification times of files.
+    }
+    deriving (Generic, Eq, Show, Data, Typeable)
+instance NFData BuildCache
+instance Store BuildCache
+
+buildCacheVC :: VersionConfig BuildCache
+buildCacheVC = storeVersionConfig "build-v1" "KVUoviSWWAd7tiRRGeWAvd0UIN4="
+
+-- | Stored on disk to know whether the flags have changed.
 data ConfigCache = ConfigCache
     { configCacheOpts :: !ConfigureOpts
       -- ^ All options used for this package.
@@ -468,25 +396,23 @@ data ConfigCache = ConfigCache
       -- is a convenient way to force compilation when the components change.
     , configCacheHaddock :: !Bool
       -- ^ Are haddocks to be built?
+    , configCachePkgSrc :: !CachePkgSrc
     }
-    deriving (Generic,Eq,Show)
-instance Binary ConfigCache where
-    put x = do
-        -- magic string
-        putWord8 1
-        putWord8 3
-        putWord8 4
-        putWord8 8
-        gput $ from x
-    get = do
-        1 <- getWord8
-        3 <- getWord8
-        4 <- getWord8
-        8 <- getWord8
-        fmap to gget
+    deriving (Generic, Eq, Show, Data, Typeable)
+instance Store ConfigCache
 instance NFData ConfigCache
-instance HasStructuralInfo ConfigCache
-instance HasSemanticVersion ConfigCache
+
+data CachePkgSrc = CacheSrcUpstream | CacheSrcLocal FilePath
+    deriving (Generic, Eq, Show, Data, Typeable)
+instance Store CachePkgSrc
+instance NFData CachePkgSrc
+
+toCachePkgSrc :: PackageSource -> CachePkgSrc
+toCachePkgSrc (PSLocal lp) = CacheSrcLocal (toFilePath (lpDir lp))
+toCachePkgSrc PSUpstream{} = CacheSrcUpstream
+
+configCacheVC :: VersionConfig ConfigCache
+configCacheVC = storeVersionConfig "config-v3" "z7N_NxX7Gbz41Gi9AGEa1zoLE-4="
 
 -- | A task to perform when building
 data Task = Task
@@ -499,6 +425,7 @@ data Task = Task
     -- ^ GhcPkgIds of already-installed dependencies
     , taskAllInOne        :: !Bool
     -- ^ indicates that the package can be built in one step
+    , taskCachePkgSrc     :: !CachePkgSrc
     }
     deriving Show
 
@@ -523,6 +450,12 @@ data TaskType = TTLocal LocalPackage
               | TTUpstream Package InstallLocation (Maybe GitSHA1)
     deriving Show
 
+taskIsTarget :: Task -> Bool
+taskIsTarget t =
+    case taskType t of
+        TTLocal lp -> lpWanted lp
+        _ -> False
+
 taskLocation :: Task -> InstallLocation
 taskLocation task =
     case taskType task of
@@ -534,7 +467,7 @@ data Plan = Plan
     { planTasks :: !(Map PackageName Task)
     , planFinals :: !(Map PackageName Task)
     -- ^ Final actions to be taken (test, benchmark, etc)
-    , planUnregisterLocal :: !(Map GhcPkgId (PackageIdentifier, Maybe Text))
+    , planUnregisterLocal :: !(Map GhcPkgId (PackageIdentifier, Text))
     -- ^ Text is reason we're unregistering, for display only
     , planInstallExes :: !(Map Text InstallLocation)
     -- ^ Executables that should be installed after successful building
@@ -549,7 +482,7 @@ data BaseConfigOpts = BaseConfigOpts
     , bcoLocalInstallRoot :: !(Path Abs Dir)
     , bcoBuildOpts :: !BuildOpts
     , bcoBuildOptsCLI :: !BuildOptsCLI
-    , bcoExtraDBs :: ![(Path Abs Dir)]
+    , bcoExtraDBs :: ![Path Abs Dir]
     }
     deriving Show
 
@@ -582,12 +515,14 @@ isStackOpt t = any (`T.isPrefixOf` t)
     , "--haddockdir="
     , "--enable-tests"
     , "--enable-benchmarks"
-    , "--enable-library-profiling"
-    , "--enable-executable-profiling"
     , "--exact-configuration"
-    ] || elem t
-    [ "--user"
-    ]
+    -- Treat these as causing dirtiness, to resolve
+    -- https://github.com/commercialhaskell/stack/issues/2984
+    --
+    -- , "--enable-library-profiling"
+    -- , "--enable-executable-profiling"
+    -- , "--enable-profiling"
+    ] || t == "--user"
 
 configureOptsDirs :: BaseConfigOpts
                   -> InstallLocation
@@ -631,8 +566,12 @@ configureOptsNoDir :: EnvConfig
 configureOptsNoDir econfig bco deps isLocal package = concat
     [ depOptions
     , ["--enable-library-profiling" | boptsLibProfile bopts || boptsExeProfile bopts]
-    , ["--enable-executable-profiling" | boptsExeProfile bopts && isLocal]
+    -- Cabal < 1.21.1 does not support --enable-profiling, use --enable-executable-profiling instead
+    , let profFlag = "--enable-" <> concat ["executable-" | not newerCabal] <> "profiling"
+      in [ profFlag | boptsExeProfile bopts && isLocal]
     , ["--enable-split-objs" | boptsSplitObjs bopts]
+    , ["--disable-library-stripping" | not $ boptsLibStrip bopts || boptsExeStrip bopts]
+    , ["--disable-executable-stripping" | not (boptsExeStrip bopts) && isLocal]
     , map (\(name,enabled) ->
                        "-f" <>
                        (if enabled
@@ -640,16 +579,16 @@ configureOptsNoDir econfig bco deps isLocal package = concat
                            else "-") <>
                        flagNameString name)
                     (Map.toList flags)
-    , concatMap (\x -> ["--ghc-options", T.unpack x]) (packageGhcOptions package)
-    , map (("--extra-include-dirs=" ++) . T.unpack) (Set.toList (configExtraIncludeDirs config))
-    , map (("--extra-lib-dirs=" ++) . T.unpack) (Set.toList (configExtraLibDirs config))
-    , if whichCompiler (envConfigCompilerVersion econfig) == Ghcjs
-        then ["--ghcjs"]
-        else []
-    , if useExactConf then ["--exact-configuration"] else []
+    , concatMap (\x -> [compilerOptionsCabalFlag wc, T.unpack x]) (packageGhcOptions package)
+    , map ("--extra-include-dirs=" ++) (Set.toList (configExtraIncludeDirs config))
+    , map ("--extra-lib-dirs=" ++) (Set.toList (configExtraLibDirs config))
+    , maybe [] (\customGcc -> ["--with-gcc=" ++ toFilePath customGcc]) (configOverrideGccPath config)
+    , ["--ghcjs" | wc == Ghcjs]
+    , ["--exact-configuration" | useExactConf]
     ]
   where
-    config = getConfig econfig
+    wc = view (actualCompilerVersionL.to whichCompiler) econfig
+    config = view configL econfig
     bopts = bcoBuildOpts bco
 
     -- TODO: instead always enable this when the cabal version is new
@@ -657,7 +596,7 @@ configureOptsNoDir econfig bco deps isLocal package = concat
     -- earlier. Cabal also might do less work then.
     useExactConf = configAllowNewer config
 
-    newerCabal = envConfigCabalVersion econfig >= $(mkVersion "1.22")
+    newerCabal = view cabalVersionL econfig >= $(mkVersion "1.22")
 
     -- Unioning atop defaults is needed so that all flags are specified
     -- with --exact-configuration.
@@ -679,10 +618,10 @@ configureOptsNoDir econfig bco deps isLocal package = concat
         [ "--constraint="
         , packageNameString name
         , "=="
-        , versionString version
+        , versionString version'
         ]
       where
-        PackageIdentifier name version = ident
+        PackageIdentifier name version' = ident
 
 -- | Get set of wanted package names from locals.
 wantedLocalPackages :: [LocalPackage] -> Set PackageName
@@ -705,9 +644,8 @@ data ConfigureOpts = ConfigureOpts
     -- if we can use an existing precompiled cache.
     , coNoDirs :: ![String]
     }
-    deriving (Show, Eq, Generic)
-instance Binary ConfigureOpts
-instance HasStructuralInfo ConfigureOpts
+    deriving (Show, Eq, Generic, Data, Typeable)
+instance Store ConfigureOpts
 instance NFData ConfigureOpts
 
 -- | Information on a compiled package: the library conf file (if relevant),
@@ -719,8 +657,12 @@ data PrecompiledCache = PrecompiledCache
     , pcExes    :: ![FilePath]
     -- ^ Full paths to executables
     }
-    deriving (Show, Eq, Generic)
+    deriving (Show, Eq, Generic, Data, Typeable)
 instance Binary PrecompiledCache
 instance HasSemanticVersion PrecompiledCache
 instance HasStructuralInfo PrecompiledCache
+instance Store PrecompiledCache
 instance NFData PrecompiledCache
+
+precompiledCacheVC :: VersionConfig PrecompiledCache
+precompiledCacheVC = storeVersionConfig "precompiled-v1" "eMzSOwaHJMamA5iNKs1A025frlQ="
