@@ -4,6 +4,8 @@
 {-# LANGUAGE OverloadedStrings          #-}
 {-# LANGUAGE RecordWildCards            #-}
 {-# LANGUAGE FlexibleInstances          #-}
+
+{-# LANGUAGE DataKinds                  #-}
 -- | Shared types for various stackage packages.
 module Stack.Types.BuildPlan
     ( -- * Types
@@ -20,6 +22,7 @@ module Stack.Types.BuildPlan
     , Component (..)
     , SnapName (..)
     , MiniBuildPlan (..)
+    , miniBuildPlanVC
     , MiniPackageInfo (..)
     , CabalFileInfo (..)
     , GitSHA1 (..)
@@ -27,39 +30,43 @@ module Stack.Types.BuildPlan
     , parseSnapName
     , SnapshotHash (..)
     , trimmedSnapshotHash
+    , ModuleName (..)
+    , ModuleInfo (..)
+    , moduleInfoVC
     ) where
 
 import           Control.Applicative
-import           Control.Arrow                   ((&&&))
-import           Control.Exception               (Exception)
-import           Control.Monad.Catch             (MonadThrow, throwM)
-import           Data.Aeson                      (FromJSON (..), ToJSON (..),
-                                                  object, withObject, withText,
-                                                  (.!=), (.:), (.:?), (.=))
-import           Data.Binary.VersionTagged
-import           Data.ByteString                 (ByteString)
-import qualified Data.ByteString                 as BS
-import           Data.Hashable                   (Hashable)
-import qualified Data.HashMap.Strict             as HashMap
-import           Data.IntMap                     (IntMap)
-import qualified Data.IntMap                     as IntMap
-import           Data.Map                        (Map)
-import qualified Data.Map                        as Map
-import           Data.Maybe                      (fromMaybe)
+import           Control.Arrow ((&&&))
+import           Control.DeepSeq (NFData)
+import           Control.Exception (Exception)
+import           Control.Monad.Catch (MonadThrow, throwM)
+import           Data.Aeson (FromJSON (..), FromJSONKey(..), ToJSON (..), ToJSONKey (..), object, withObject, withText, (.!=), (.:), (.:?), (.=))
+import           Data.ByteString (ByteString)
+import qualified Data.ByteString as BS
+import           Data.Data
+import qualified Data.HashMap.Strict as HashMap
+import           Data.Hashable (Hashable)
+import           Data.IntMap (IntMap)
+import qualified Data.IntMap as IntMap
+import           Data.Map (Map)
+import qualified Data.Map as Map
+import           Data.Maybe (fromMaybe)
 import           Data.Monoid
-import           Data.Set                        (Set)
-import           Data.String                     (IsString, fromString)
-import           Data.Text                       (Text, pack, unpack)
-import qualified Data.Text                       as T
-import Data.Text.Read (decimal)
-import           Data.Time                       (Day)
-import qualified Data.Traversable                as T
-import           Data.Typeable                   (TypeRep, Typeable, typeOf)
-import           Data.Vector                     (Vector)
-import           Distribution.System             (Arch, OS (..))
-import qualified Distribution.Text               as DT
-import qualified Distribution.Version            as C
-import           GHC.Generics                    (Generic)
+import           Data.Set (Set)
+import           Data.Store (Store)
+import           Data.Store.Version
+import           Data.Store.VersionTagged
+import           Data.String (IsString, fromString)
+import           Data.Text (Text, pack, unpack)
+import qualified Data.Text as T
+import           Data.Text.Read (decimal)
+import           Data.Time (Day)
+import qualified Data.Traversable as T
+import           Data.Vector (Vector)
+import           Distribution.System (Arch, OS (..))
+import qualified Distribution.Text as DT
+import qualified Distribution.Version as C
+import           GHC.Generics (Generic)
 import           Prelude -- Fix AMP warning
 import           Safe (readMay)
 import           Stack.Types.Compiler
@@ -117,7 +124,7 @@ data PackagePlan = PackagePlan
 
 instance ToJSON PackagePlan where
     toJSON PackagePlan {..} = object
-        $ maybe id (\cfi -> (("cabal-file-info" .= cfi):)) ppCabalFileInfo $
+        $ maybe id (\cfi -> (("cabal-file-info" .= cfi):)) ppCabalFileInfo
         [ "version"      .= ppVersion
         , "github-pings" .= ppGithubPings
         , "users"        .= ppUsers
@@ -191,6 +198,7 @@ data PackageConstraints = PackageConstraints
     , pcBuildBenchmarks  :: Bool
     , pcFlagOverrides    :: Map FlagName Bool
     , pcEnableLibProfile :: Bool
+    , pcHide             :: Bool
     }
     deriving (Show, Eq)
 instance ToJSON PackageConstraints where
@@ -201,6 +209,7 @@ instance ToJSON PackageConstraints where
         , "build-benchmarks" .= pcBuildBenchmarks
         , "flags" .= pcFlagOverrides
         , "library-profiling" .= pcEnableLibProfile
+        , "hide" .= pcHide
         ]
       where
         addMaintainer = maybe id (\m -> (("maintainer" .= m):)) pcMaintainer
@@ -214,6 +223,7 @@ instance FromJSON PackageConstraints where
         pcFlagOverrides <- o .: "flags"
         pcMaintainer <- o .:? "maintainer"
         pcEnableLibProfile <- fmap (fromMaybe True) (o .:? "library-profiling")
+        pcHide <- o .:? "hide" .!= False
         return PackageConstraints {..}
 
 data TestState = ExpectSuccess
@@ -277,10 +287,7 @@ newtype Maintainer = Maintainer { unMaintainer :: Text }
 
 -- | Name of an executable.
 newtype ExeName = ExeName { unExeName :: Text }
-    deriving (Show, Eq, Ord, Hashable, IsString, Generic, Binary, NFData)
-instance HasStructuralInfo ExeName
-instance ToJSON ExeName where
-    toJSON = toJSON . unExeName
+    deriving (Show, Eq, Ord, Hashable, IsString, Generic, Store, NFData, Data, Typeable, ToJSON, ToJSONKey, FromJSONKey)
 instance FromJSON ExeName where
     parseJSON = withText "ExeName" $ return . ExeName
 
@@ -399,8 +406,7 @@ data Snapshots = Snapshots
 instance FromJSON Snapshots where
     parseJSON = withObject "Snapshots" $ \o -> Snapshots
         <$> (o .: "nightly" >>= parseNightly)
-        <*> (fmap IntMap.unions
-                $ mapM (parseLTS . snd)
+        <*> fmap IntMap.unions (mapM (parseLTS . snd)
                 $ filter (isLTS . fst)
                 $ HashMap.toList o)
       where
@@ -418,21 +424,17 @@ instance FromJSON Snapshots where
                 Right (LTS x y) -> return $ IntMap.singleton x y
                 Right (Nightly _) -> fail "Unexpected nightly value"
 
-instance ToJSON a => ToJSON (Map ExeName a) where
-  toJSON = toJSON . Map.mapKeysWith const unExeName
-instance FromJSON a => FromJSON (Map ExeName a) where
-    parseJSON = fmap (Map.mapKeysWith const ExeName) . parseJSON
-
 -- | A simplified version of the 'BuildPlan' + cabal file.
 data MiniBuildPlan = MiniBuildPlan
     { mbpCompilerVersion :: !CompilerVersion
     , mbpPackages :: !(Map PackageName MiniPackageInfo)
     }
-    deriving (Generic, Show, Eq)
-instance Binary MiniBuildPlan
+    deriving (Generic, Show, Eq, Data, Typeable)
+instance Store MiniBuildPlan
 instance NFData MiniBuildPlan
-instance HasStructuralInfo MiniBuildPlan
-instance HasSemanticVersion MiniBuildPlan
+
+miniBuildPlanVC :: VersionConfig MiniBuildPlan
+miniBuildPlanVC = storeVersionConfig "mbp-v2" "C8q73RrYq3plf9hDCapjWpnm_yc="
 
 -- | Information on a single package for the 'MiniBuildPlan'.
 data MiniPackageInfo = MiniPackageInfo
@@ -443,7 +445,7 @@ data MiniPackageInfo = MiniPackageInfo
     , mpiToolDeps :: !(Set Text)
     -- ^ Due to ambiguity in Cabal, it is unclear whether this refers to the
     -- executable name, the package name, or something else. We have to guess
-    -- based on what's available, which is why we store this is an unwrapped
+    -- based on what's available, which is why we store this in an unwrapped
     -- 'Text'.
     , mpiExes :: !(Set ExeName)
     -- ^ Executables provided by this package
@@ -452,18 +454,34 @@ data MiniPackageInfo = MiniPackageInfo
     , mpiGitSHA1 :: !(Maybe GitSHA1)
     -- ^ An optional SHA1 representation in hex format of the blob containing
     -- the cabal file contents. Useful for grabbing the correct cabal file
-    -- revision directly from a Git repo
+    -- revision directly from a Git repo or the 01-index.tar file
     }
-    deriving (Generic, Show, Eq)
-instance Binary MiniPackageInfo
-instance HasStructuralInfo MiniPackageInfo
+    deriving (Generic, Show, Eq, Data, Typeable)
+instance Store MiniPackageInfo
 instance NFData MiniPackageInfo
 
+-- | A SHA1 hash, but in Git format. This means that the contents are
+-- prefixed with @blob@ and the size of the payload before hashing, as
+-- Git itself does.
 newtype GitSHA1 = GitSHA1 ByteString
-    deriving (Generic, Show, Eq, NFData, HasStructuralInfo, Binary)
+    deriving (Generic, Show, Eq, NFData, Store, Data, Typeable, Ord, Hashable)
 
 newtype SnapshotHash = SnapshotHash { unShapshotHash :: ByteString }
     deriving (Generic, Show, Eq)
 
 trimmedSnapshotHash :: SnapshotHash -> ByteString
 trimmedSnapshotHash = BS.take 12 . unShapshotHash
+
+newtype ModuleName = ModuleName { unModuleName :: ByteString }
+  deriving (Show, Eq, Ord, Generic, Store, NFData, Typeable, Data)
+
+data ModuleInfo = ModuleInfo
+    { miCorePackages :: !(Set PackageName)
+    , miModules      :: !(Map ModuleName (Set PackageName))
+    }
+  deriving (Show, Eq, Ord, Generic, Typeable, Data)
+instance Store ModuleInfo
+instance NFData ModuleInfo
+
+moduleInfoVC :: VersionConfig ModuleInfo
+moduleInfoVC = storeVersionConfig "mi-v1" "zyCpzzGXA8fTeBmKEWLa_6kF2_s="
