@@ -13,7 +13,8 @@ module Stack.Types.PackageIdentifier
   , parsePackageIdentifierFromString
   , packageIdentifierParser
   , packageIdentifierString
-  , packageIdentifierText )
+  , packageIdentifierText
+  , toCabalPackageIdentifier )
   where
 
 import           Control.Applicative
@@ -22,18 +23,19 @@ import           Control.Exception (Exception)
 import           Control.Monad.Catch (MonadThrow, throwM)
 import           Data.Aeson.Extended
 import           Data.Attoparsec.Text
-import           Data.Binary.VersionTagged (Binary, HasStructuralInfo)
 import           Data.Data
 import           Data.Hashable
+import           Data.Store (Store)
 import           Data.Text (Text)
 import qualified Data.Text as T
+import qualified Distribution.Package as C
 import           GHC.Generics
 import           Prelude hiding (FilePath)
 import           Stack.Types.PackageName
 import           Stack.Types.Version
 
 -- | A parse fail.
-data PackageIdentifierParseFail
+newtype PackageIdentifierParseFail
   = PackageIdentifierParseFail Text
   deriving (Typeable)
 instance Show PackageIdentifierParseFail where
@@ -53,8 +55,7 @@ instance NFData PackageIdentifier where
       seq (rnf p) (rnf v)
 
 instance Hashable PackageIdentifier
-instance Binary PackageIdentifier
-instance HasStructuralInfo PackageIdentifier
+instance Store PackageIdentifier
 
 instance Show PackageIdentifier where
   show = show . packageIdentifierString
@@ -102,3 +103,9 @@ packageIdentifierString (PackageIdentifier n v) = show n ++ "-" ++ show v
 -- | Get a Text representation of the package identifier; name-ver.
 packageIdentifierText :: PackageIdentifier -> Text
 packageIdentifierText = T.pack .  packageIdentifierString
+
+toCabalPackageIdentifier :: PackageIdentifier -> C.PackageIdentifier
+toCabalPackageIdentifier x =
+    C.PackageIdentifier
+        (toCabalPackageName (packageIdentifierName x))
+        (toCabalVersion (packageIdentifierVersion x))
