@@ -27,7 +27,8 @@ module Stack.Types.Version
   ,toMajorVersion
   ,latestApplicableVersion
   ,checkVersion
-  ,nextMajorVersion)
+  ,nextMajorVersion
+  ,UpgradeTo(..))
   where
 
 import           Control.Applicative
@@ -35,20 +36,16 @@ import           Control.DeepSeq
 import           Control.Monad.Catch
 import           Data.Aeson.Extended
 import           Data.Attoparsec.Text
-import           Data.Binary.VersionTagged (Binary, HasStructuralInfo)
 import           Data.Data
 import           Data.Hashable
 import           Data.List
-import           Data.Map (Map)
-import qualified Data.Map as Map
 import           Data.Maybe (listToMaybe)
 import           Data.Monoid
 import           Data.Set (Set)
 import qualified Data.Set as Set
+import           Data.Store (Store)
 import           Data.Text (Text)
 import qualified Data.Text as T
-import           Data.Text.Binary ()
-import           Data.Vector.Binary ()
 import           Data.Vector.Unboxed (Vector)
 import qualified Data.Vector.Unboxed as V
 import           Data.Word
@@ -58,21 +55,24 @@ import           GHC.Generics
 import           Language.Haskell.TH
 import           Language.Haskell.TH.Syntax
 import           Prelude -- Fix warning: Word in Prelude from base-4.8.
+import           Stack.Types.StringError
 import           Text.PrettyPrint (render)
 
 -- | A parse fail.
-data VersionParseFail =
+newtype VersionParseFail =
   VersionParseFail Text
   deriving (Typeable)
 instance Exception VersionParseFail
 instance Show VersionParseFail where
     show (VersionParseFail bs) = "Invalid version: " ++ show bs
 
+-- | A Package upgrade; Latest or a specific version.
+data UpgradeTo = Specific Version | Latest deriving (Show)
+
 -- | A package version.
 newtype Version =
   Version {unVersion :: Vector Word}
-  deriving (Eq,Ord,Typeable,Data,Generic,Binary,NFData)
-instance HasStructuralInfo Version
+  deriving (Eq,Ord,Typeable,Data,Generic,Store,NFData)
 
 instance Hashable Version where
   hashWithSalt i = hashWithSalt i . V.toList . unVersion
@@ -98,14 +98,9 @@ instance FromJSON Version where
          Nothing ->
            fail ("Couldn't parse package version: " ++ s)
          Just ver -> return ver
-instance FromJSON a => FromJSON (Map Version a) where
-    parseJSON val = do
-        m <- parseJSON val
-        fmap Map.fromList $ mapM go $ Map.toList m
-      where
-        go (k, v) = do
-            k' <- either (fail . show) return $ parseVersionFromString k
-            return (k', v)
+instance FromJSONKey Version where
+  fromJSONKey = FromJSONKeyTextParser $ \k ->
+    either (fail . show) return $ parseVersion k
 
 newtype IntersectingVersionRange =
     IntersectingVersionRange { getIntersectingVersionRange :: Cabal.VersionRange }
@@ -119,7 +114,7 @@ instance Monoid IntersectingVersionRange where
 -- | Attoparsec parser for a package version.
 versionParser :: Parser Version
 versionParser =
-  do ls <- ((:) <$> num <*> many num')
+  do ls <- (:) <$> num <*> many num'
      let !v = V.fromList ls
      return (Version v)
   where num = decimal
@@ -167,7 +162,7 @@ fromCabalVersion (Cabal.Version vs _) =
 mkVersion :: String -> Q Exp
 mkVersion s =
   case parseVersionFromString s of
-    Nothing -> error ("Invalid package version: " ++ show s)
+    Nothing -> errorString ("Invalid package version: " ++ show s)
     Just pn -> [|pn|]
 
 -- | Display a version range
