@@ -11,7 +11,9 @@
 
 module System.Process.Read
   (readProcessStdout
+  ,readProcessStderrStdout
   ,tryProcessStdout
+  ,tryProcessStderrStdout
   ,sinkProcessStdout
   ,sinkProcessStderrStdout
   ,sinkProcessStderrStdoutHandle
@@ -28,14 +30,14 @@ module System.Process.Read
   ,envSearchPath
   ,preProcess
   ,readProcessNull
-  ,readInNull
-  ,logProcessRun
   ,ReadProcessException (..)
   ,augmentPath
   ,augmentPathMap
+  ,resetExeCache
   )
   where
 
+import           Control.Applicative
 import           Control.Arrow ((***), first)
 import           Control.Concurrent.Async (concurrently)
 import           Control.Exception hiding (try, catch)
@@ -51,11 +53,10 @@ import           Data.Conduit
 import qualified Data.Conduit.Binary as CB
 import qualified Data.Conduit.List as CL
 import           Data.Conduit.Process hiding (callProcess)
-import           Data.Foldable (forM_)
 import           Data.IORef
 import           Data.Map (Map)
 import qualified Data.Map as Map
-import           Data.Maybe (isJust, maybeToList)
+import           Data.Maybe (isJust, maybeToList, fromMaybe)
 import           Data.Monoid
 import           Data.Text (Text)
 import qualified Data.Text as T
@@ -66,14 +67,16 @@ import           Data.Typeable (Typeable)
 import           Distribution.System (OS (Windows), Platform (Platform))
 import           Language.Haskell.TH as TH (location)
 import           Path
+import           Path.Extra
 import           Path.IO hiding (findExecutable)
 import           Prelude -- Fix AMP warning
 import qualified System.Directory as D
 import           System.Environment (getEnvironment)
 import           System.Exit
 import qualified System.FilePath as FP
-import           System.IO (Handle)
+import           System.IO (Handle, hClose)
 import           System.Process.Log
+import           Prelude () -- Hide post-AMP warnings
 
 -- | Override the environment received by a child process.
 data EnvOverride = EnvOverride
@@ -81,7 +84,7 @@ data EnvOverride = EnvOverride
     , eoStringList :: [(String, String)] -- ^ Environment variables as association list
     , eoPath :: [FilePath] -- ^ List of directories searched for executables (@PATH@)
     , eoExeCache :: IORef (Map FilePath (Either ReadProcessException (Path Abs File)))
-    , eoExeExtension :: String -- ^ @""@ or @".exe"@, depending on the platform
+    , eoExeExtensions :: [String] -- ^ @[""]@ on non-Windows systems, @["", ".exe", ".bat"]@ on Windows
     , eoPlatform :: Platform
     }
 
@@ -112,9 +115,17 @@ mkEnvOverride platform tm' = do
     return EnvOverride
         { eoTextMap = tm
         , eoStringList = map (T.unpack *** T.unpack) $ Map.toList tm
-        , eoPath = maybe [] (FP.splitSearchPath . T.unpack) (Map.lookup "PATH" tm)
+        , eoPath =
+             (if isWindows then (".":) else id)
+             (maybe [] (FP.splitSearchPath . T.unpack) (Map.lookup "PATH" tm))
         , eoExeCache = ref
-        , eoExeExtension = if isWindows then ".exe" else ""
+        , eoExeExtensions =
+            if isWindows
+                then let pathext = fromMaybe
+                           ".COM;.EXE;.BAT;.CMD;.VBS;.VBE;.JS;.JSE;.WSF;.WSH;.MSC"
+                           (Map.lookup "PATHEXT" tm)
+                      in map T.unpack $ "" : T.splitOn ";" pathext
+                else [""]
         , eoPlatform = platform
         }
   where
@@ -135,6 +146,8 @@ envHelper :: EnvOverride -> Maybe [(String, String)]
 envHelper = Just . eoStringList
 
 -- | Read from the process, ignoring any output.
+--
+-- Throws a 'ReadProcessException' exception if the process fails.
 readProcessNull :: (MonadIO m, MonadLogger m, MonadBaseControl IO m, MonadCatch m)
                 => Maybe (Path Abs Dir) -- ^ Optional working directory
                 -> EnvOverride
@@ -144,26 +157,6 @@ readProcessNull :: (MonadIO m, MonadLogger m, MonadBaseControl IO m, MonadCatch
 readProcessNull wd menv name args =
     sinkProcessStdout wd menv name args CL.sinkNull
 
--- | Run the given command in the given directory. If it exits with anything
--- but success, print an error and then call 'exitWith' to exit the program.
-readInNull :: (MonadIO m, MonadLogger m, MonadBaseControl IO m, MonadCatch m)
-           => Path Abs Dir -- ^ Directory to run in
-           -> FilePath -- ^ Command to run
-           -> EnvOverride
-           -> [String] -- ^ Command line arguments
-           -> Maybe Text -- ^ Optional additional error message
-           -> m ()
-readInNull wd cmd menv args errMsg = do
-    result <- try (readProcessNull (Just wd) menv cmd args)
-    case result of
-        Left ex -> do
-            $logError (T.pack (show ex))
-            case ex of
-                ReadProcessException{} -> forM_ errMsg $logError
-                _ -> return ()
-            liftIO exitFailure
-        Right () -> return ()
-
 -- | Try to produce a strict 'S.ByteString' from the stdout of a
 -- process.
 tryProcessStdout :: (MonadIO m, MonadLogger m, MonadBaseControl IO m, MonadCatch m)
@@ -175,9 +168,20 @@ tryProcessStdout :: (MonadIO m, MonadLogger m, MonadBaseControl IO m, MonadCatch
 tryProcessStdout wd menv name args =
     try (readProcessStdout wd menv name args)
 
+-- | Try to produce strict 'S.ByteString's from the stderr and stdout of a
+-- process.
+tryProcessStderrStdout :: (MonadIO m, MonadLogger m, MonadBaseControl IO m, MonadCatch m)
+                       => Maybe (Path Abs Dir) -- ^ Optional directory to run in
+                       -> EnvOverride
+                       -> String -- ^ Command
+                       -> [String] -- ^ Command line arguments
+                       -> m (Either ReadProcessException (S.ByteString, S.ByteString))
+tryProcessStderrStdout wd menv name args =
+    try (readProcessStderrStdout wd menv name args)
+
 -- | Produce a strict 'S.ByteString' from the stdout of a process.
 --
--- Throws a 'ReadProcessException' exception if the  process fails.
+-- Throws a 'ReadProcessException' exception if the process fails.
 readProcessStdout :: (MonadIO m, MonadLogger m, MonadBaseControl IO m, MonadCatch m)
                   => Maybe (Path Abs Dir) -- ^ Optional directory to run in
                   -> EnvOverride
@@ -188,15 +192,29 @@ readProcessStdout wd menv name args =
   sinkProcessStdout wd menv name args CL.consume >>=
   liftIO . evaluate . S.concat
 
+-- | Produce strict 'S.ByteString's from the stderr and stdout of a process.
+--
+-- Throws a 'ReadProcessException' exception if the process fails.
+readProcessStderrStdout :: (MonadIO m, MonadLogger m, MonadBaseControl IO m)
+                        => Maybe (Path Abs Dir) -- ^ Optional directory to run in
+                        -> EnvOverride
+                        -> String -- ^ Command
+                        -> [String] -- ^ Command line arguments
+                        -> m (S.ByteString, S.ByteString)
+readProcessStderrStdout wd menv name args = do
+  (e, o) <- sinkProcessStderrStdout wd menv name args CL.consume CL.consume
+  liftIO $ (,) <$> evaluate (S.concat e) <*> evaluate (S.concat o)
+
 -- | An exception while trying to read from process.
 data ReadProcessException
-    = ReadProcessException CreateProcess ExitCode L.ByteString L.ByteString
+    = ProcessFailed CreateProcess ExitCode L.ByteString L.ByteString
+    -- ^ @'ProcessFailed' createProcess exitCode stdout stderr@
     | NoPathFound
     | ExecutableNotFound String [FilePath]
     | ExecutableNotFoundAt FilePath
     deriving Typeable
 instance Show ReadProcessException where
-    show (ReadProcessException cp ec out err) = concat $
+    show (ProcessFailed cp ec out err) = concat $
         [ "Running "
         , showSpec $ cmdspec cp] ++
         maybe [] (\x -> [" in directory ", x]) (cwd cp) ++
@@ -228,6 +246,8 @@ instance Exception ReadProcessException
 -- If the process fails, spits out stdout and stderr as error log
 -- level. Should not be used for long-running processes or ones with
 -- lots of output; for that use 'sinkProcessStdoutLogStderr'.
+--
+-- Throws a 'ReadProcessException' if unsuccessful.
 sinkProcessStdout
     :: (MonadIO m, MonadLogger m, MonadBaseControl IO m, MonadCatch m)
     => Maybe (Path Abs Dir) -- ^ Optional directory to run in
@@ -252,7 +272,7 @@ sinkProcessStdout wd menv name args sinkStdout = do
           (\(ProcessExitedUnsuccessfully cp ec) ->
                do stderrBuilder <- liftIO (readIORef stderrBuffer)
                   stdoutBuilder <- liftIO (readIORef stdoutBuffer)
-                  throwM $ ReadProcessException
+                  throwM $ ProcessFailed
                     cp
                     ec
                     (toLazyByteString stdoutBuilder)
@@ -271,6 +291,8 @@ logProcessStderrStdout mdir name menv args = liftBaseWith $ \restore -> do
     void $ restore $ sinkProcessStderrStdout mdir menv name args logLines logLines
 
 -- | Consume the stdout and stderr of a process feeding strict 'S.ByteString's to the consumers.
+--
+-- Throws a 'ReadProcessException' if unsuccessful in launching, or 'ProcessExitedUnsuccessfully' if the process itself fails.
 sinkProcessStderrStdout :: forall m e o. (MonadIO m, MonadLogger m)
                         => Maybe (Path Abs Dir) -- ^ Optional directory to run in
                         -> EnvOverride
@@ -280,15 +302,31 @@ sinkProcessStderrStdout :: forall m e o. (MonadIO m, MonadLogger m)
                         -> Sink S.ByteString IO o -- ^ Sink for stdout
                         -> m (e,o)
 sinkProcessStderrStdout wd menv name args sinkStderr sinkStdout = do
-  $logProcessRun name args
   name' <- preProcess wd menv name
-  liftIO $ withCheckedProcess
-      (proc name' args) { env = envHelper menv, cwd = fmap toFilePath wd }
-      (\ClosedStream out err -> f err out)
+  $withProcessTimeLog name' args $
+      liftIO $ withCheckedProcess
+          (proc name' args) { env = envHelper menv, cwd = fmap toFilePath wd }
+          (\ClosedStream out err -> f err out)
   where
-    f :: Source IO S.ByteString -> Source IO S.ByteString -> IO (e, o)
-    f err out = (err $$ sinkStderr) `concurrently` (out $$ sinkStdout)
 
+    -- There is a bug in streaming-commons or conduit-extra which
+    -- leads to a file descriptor leak. Ideally, we should be able to
+    -- simply use the following code. Instead, we're using the code
+    -- below it, which is explicit in closing Handles. When the
+    -- upstream bug is fixed, we can consider moving back to the
+    -- simpler code, though there's really no downside to the more
+    -- complex version used here.
+    --
+    -- f :: Source IO S.ByteString -> Source IO S.ByteString -> IO (e, o)
+    -- f err out = (err $$ sinkStderr) `concurrently` (out $$ sinkStdout)
+
+    f :: Handle -> Handle -> IO (e, o)
+    f err out = ((CB.sourceHandle err $$ sinkStderr) `concurrently` (CB.sourceHandle out $$ sinkStdout))
+        `finally` hClose err `finally` hClose out
+
+-- | Like sinkProcessStderrStdout, but receives Handles for stderr and stdout instead of 'Sink's.
+--
+-- Throws a 'ReadProcessException' if unsuccessful in launching, or 'ProcessExitedUnsuccessfully' if the process itself fails.
 sinkProcessStderrStdoutHandle :: (MonadIO m, MonadLogger m)
                               => Maybe (Path Abs Dir) -- ^ Optional directory to run in
                               -> EnvOverride
@@ -298,19 +336,21 @@ sinkProcessStderrStdoutHandle :: (MonadIO m, MonadLogger m)
                               -> Handle
                               -> m ()
 sinkProcessStderrStdoutHandle wd menv name args err out = do
-  $logProcessRun name args
   name' <- preProcess wd menv name
-  liftIO $ withCheckedProcess
-      (proc name' args)
-          { env = envHelper menv
-          , cwd = fmap toFilePath wd
-          , std_err = UseHandle err
-          , std_out = UseHandle out
-          }
-      (\ClosedStream UseProvidedHandle UseProvidedHandle -> return ())
+  $withProcessTimeLog name' args $
+      liftIO $ withCheckedProcess
+          (proc name' args)
+              { env = envHelper menv
+              , cwd = fmap toFilePath wd
+              , std_err = UseHandle err
+              , std_out = UseHandle out
+              }
+          (\ClosedStream UseProvidedHandle UseProvidedHandle -> return ())
 
 -- | Perform pre-call-process tasks.  Ensure the working directory exists and find the
 -- executable path.
+--
+-- Throws a 'ReadProcessException' if unsuccessful.
 preProcess :: (MonadIO m)
   => Maybe (Path Abs Dir) -- ^ Optional directory to create if necessary
   -> EnvOverride       -- ^ How to override environment
@@ -336,10 +376,7 @@ findExecutable :: (MonadIO m, MonadThrow n)
   -> String            -- ^ Name of executable
   -> m (n (Path Abs File)) -- ^ Full path to that executable on success
 findExecutable eo name0 | any FP.isPathSeparator name0 = do
-    let names0
-            | null (eoExeExtension eo) = [name0]
-            -- Support `stack exec foo/bar.exe` on Windows
-            | otherwise = [name0 ++ eoExeExtension eo, name0]
+    let names0 = map (name0 ++) (eoExeExtensions eo)
         testNames [] = return $ throwM $ ExecutableNotFoundAt name0
         testNames (name:names) = do
             exists <- liftIO $ D.doesFileExist name
@@ -357,14 +394,12 @@ findExecutable eo name = liftIO $ do
             let loop [] = return $ Left $ ExecutableNotFound name (eoPath eo)
                 loop (dir:dirs) = do
                     let fp0 = dir FP.</> name
-                        fps0
-                            | null (eoExeExtension eo) = [fp0]
-                            -- Support `stack exec foo.exe` on Windows
-                            | otherwise = [fp0 ++ eoExeExtension eo, fp0]
+                        fps0 = map (fp0 ++) (eoExeExtensions eo)
                         testFPs [] = loop dirs
                         testFPs (fp:fps) = do
                             exists <- D.doesFileExist fp
-                            if exists
+                            existsExec <- if exists then liftM D.executable $ D.getPermissions fp else return False
+                            if existsExec
                                 then do
                                     fp' <- D.makeAbsolute fp >>= parseAbsFile
                                     return $ return fp'
@@ -376,6 +411,10 @@ findExecutable eo name = liftIO $ do
             return epath
     return $ either throwM return epath
 
+-- | Reset the executable cache.
+resetExeCache :: MonadIO m => EnvOverride -> m ()
+resetExeCache eo = liftIO (atomicModifyIORef (eoExeCache eo) (const mempty))
+
 -- | Load up an 'EnvOverride' from the standard environment.
 getEnvOverride :: MonadIO m => Platform -> m EnvOverride
 getEnvOverride platform =
@@ -384,11 +423,11 @@ getEnvOverride platform =
           mkEnvOverride platform
         . Map.fromList . map (T.pack *** T.pack)
 
-data PathException = PathsInvalidInPath [FilePath]
+newtype InvalidPathException = PathsInvalidInPath [FilePath]
     deriving Typeable
 
-instance Exception PathException
-instance Show PathException where
+instance Exception InvalidPathException
+instance Show InvalidPathException where
     show (PathsInvalidInPath paths) = unlines $
         [ "Would need to add some paths to the PATH environment variable \
           \to continue, but they would be invalid because they contain a "
@@ -397,16 +436,16 @@ instance Show PathException where
         ] ++ paths
 
 -- | Augment the PATH environment variable with the given extra paths.
-augmentPath :: MonadThrow m => [FilePath] -> Maybe Text -> m Text
+augmentPath :: MonadThrow m => [Path Abs Dir] -> Maybe Text -> m Text
 augmentPath dirs mpath =
-  do let illegal = filter (FP.searchPathSeparator `elem`) dirs
+  do let illegal = filter (FP.searchPathSeparator `elem`) (map toFilePath dirs)
      unless (null illegal) (throwM $ PathsInvalidInPath illegal)
      return $ T.intercalate (T.singleton FP.searchPathSeparator)
-            $ map (T.pack . FP.dropTrailingPathSeparator) dirs
+            $ map (T.pack . toFilePathNoTrailingSep) dirs
             ++ maybeToList mpath
 
 -- | Apply 'augmentPath' on the PATH value in the given Map.
-augmentPathMap :: MonadThrow m => [FilePath] -> Map Text Text
+augmentPathMap :: MonadThrow m => [Path Abs Dir] -> Map Text Text
                                -> m (Map Text Text)
 augmentPathMap dirs origEnv =
   do path <- augmentPath dirs mpath
