@@ -4,7 +4,10 @@ module Stack.Build.TargetSpec (main, spec) where
 
 import qualified Data.Text           as T
 import           Stack.Build.Target
-import           Stack.Types
+import           Stack.Types.PackageIdentifier
+import           Stack.Types.PackageName
+import           Stack.Types.Version
+import           Stack.Types.Package
 import           Test.Hspec
 
 main :: IO ()
