@@ -1,23 +1,30 @@
 {-# LANGUAGE OverloadedStrings #-}
-{-# LANGUAGE TemplateHaskell #-}
-{-# LANGUAGE TupleSections #-}
+{-# LANGUAGE TemplateHaskell   #-}
+{-# LANGUAGE TupleSections     #-}
 module Stack.PackageDumpSpec where
 
-import Data.Conduit
-import qualified Data.Conduit.List as CL
-import qualified Data.Conduit.Binary as CB
-import           Data.Conduit.Text (decodeUtf8)
-import Control.Monad.Trans.Resource (runResourceT)
-import Stack.PackageDump
-import Stack.Types
-import Test.Hspec
-import Test.Hspec.QuickCheck
-import System.Process.Read
-import Control.Monad.Logger
-import Distribution.System (buildPlatform)
-import qualified Data.Map as Map
-import Data.Map (Map)
-import qualified Data.Set as Set
+import           Control.Applicative
+import           Control.Monad.Logger
+import           Control.Monad.Trans.Resource  (runResourceT)
+import           Data.Conduit
+import qualified Data.Conduit.Binary           as CB
+import qualified Data.Conduit.List             as CL
+import           Data.Conduit.Text             (decodeUtf8)
+import           Data.Map                      (Map)
+import qualified Data.Map                      as Map
+import qualified Data.Set                      as Set
+import           Distribution.System           (buildPlatform)
+import           Distribution.License          (License(..))
+import           Prelude -- Fix redundant imports warnings
+import           Stack.PackageDump
+import           Stack.Types.Compiler
+import           Stack.Types.GhcPkgId
+import           Stack.Types.PackageIdentifier
+import           Stack.Types.PackageName
+import           Stack.Types.Version
+import           System.Process.Read
+import           Test.Hspec
+import           Test.Hspec.QuickCheck
 
 main :: IO ()
 main = hspec spec
@@ -77,6 +84,7 @@ spec = do
             haskell2010 `shouldBe` DumpPackage
                 { dpGhcPkgId = ghcPkgId
                 , dpPackageIdent = packageIdent
+                , dpLicense = Just BSD3
                 , dpLibDirs = ["/opt/ghc/7.8.4/lib/ghc-7.8.4/haskell2010-1.1.2.0"]
                 , dpDepends = depends
                 , dpLibraries = ["HShaskell2010-1.1.2.0"]
@@ -85,6 +93,7 @@ spec = do
                 , dpHaddockHtml = Just "/opt/ghc/7.8.4/share/doc/ghc/html/libraries/haskell2010-1.1.2.0"
                 , dpProfiling = ()
                 , dpHaddock = ()
+                , dpSymbols = ()
                 , dpIsExposed = False
                 }
 
@@ -115,6 +124,7 @@ spec = do
             haskell2010 `shouldBe` DumpPackage
                 { dpGhcPkgId = ghcPkgId
                 , dpPackageIdent = pkgIdent
+                , dpLicense = Just BSD3
                 , dpLibDirs = ["/opt/ghc/7.10.1/lib/ghc-7.10.1/ghc_EMlWrQ42XY0BNVbSrKixqY"]
                 , dpHaddockInterfaces = ["/opt/ghc/7.10.1/share/doc/ghc/html/libraries/ghc-7.10.1/ghc.haddock"]
                 , dpHaddockHtml = Just "/opt/ghc/7.10.1/share/doc/ghc/html/libraries/ghc-7.10.1"
@@ -123,6 +133,7 @@ spec = do
                 , dpHasExposedModules = True
                 , dpProfiling = ()
                 , dpHaddock = ()
+                , dpSymbols = ()
                 , dpIsExposed = False
                 }
         it "ghc 7.8.4 (osx)" $ do
@@ -146,6 +157,7 @@ spec = do
             hmatrix `shouldBe` DumpPackage
                 { dpGhcPkgId = ghcPkgId
                 , dpPackageIdent = pkgId
+                , dpLicense = Just BSD3
                 , dpLibDirs =
                       [ "/Users/alexbiehl/.stack/snapshots/x86_64-osx/lts-2.13/7.8.4/lib/x86_64-osx-ghc-7.8.4/hmatrix-0.16.1.5"
                       , "/opt/local/lib/"
@@ -158,6 +170,7 @@ spec = do
                 , dpHasExposedModules = True
                 , dpProfiling = ()
                 , dpHaddock = ()
+                , dpSymbols = ()
                 , dpIsExposed = True
                 }
         it "ghc HEAD" $ do
@@ -178,6 +191,7 @@ spec = do
           ghcBoot `shouldBe` DumpPackage
             { dpGhcPkgId = ghcPkgId
             , dpPackageIdent = pkgId
+            , dpLicense = Just BSD3
             , dpLibDirs =
                   ["/opt/ghc/head/lib/ghc-7.11.20151213/ghc-boot-0.0.0.0"]
             , dpHaddockInterfaces = ["/opt/ghc/head/share/doc/ghc/html/libraries/ghc-boot-0.0.0.0/ghc-boot.haddock"]
@@ -187,6 +201,7 @@ spec = do
             , dpHasExposedModules = True
             , dpProfiling = ()
             , dpHaddock = ()
+            , dpSymbols = ()
             , dpIsExposed = True
             }
 
@@ -199,6 +214,7 @@ spec = do
             $  conduitDumpPackage
             =$ addProfiling icache
             =$ addHaddock icache
+            =$ fakeAddSymbols
             =$ CL.sinkNull
 
     it "sinkMatching" $ do
@@ -209,7 +225,8 @@ spec = do
             $  conduitDumpPackage
             =$ addProfiling icache
             =$ addHaddock icache
-            =$ sinkMatching False False (Map.singleton $(mkPackageName "transformers") $(mkVersion "0.0.0.0.0.0.1"))
+            =$ fakeAddSymbols
+            =$ sinkMatching False False False (Map.singleton $(mkPackageName "transformers") $(mkVersion "0.0.0.0.0.0.1"))
         case Map.lookup $(mkPackageName "base") m of
             Nothing -> error "base not present"
             Just _ -> return ()
@@ -226,7 +243,7 @@ spec = do
                     , ((3, 'a'), [(1, 'c')])
                     , ((4, 'a'), [(2, 'a')])
                     ]
-                actual = fmap fst $ pruneDeps fst fst snd bestPrune prunes
+                actual = fst <$> pruneDeps fst fst snd bestPrune prunes
             actual `shouldBe` Map.fromList
                 [ (1, (1, 'b'))
                 , (2, (2, 'a'))
@@ -236,7 +253,7 @@ spec = do
         prop "invariant holds" $ \prunes' ->
             -- Force uniqueness
             let prunes = Map.toList $ Map.fromList prunes'
-             in checkDepsPresent prunes $ fmap fst $ pruneDeps fst fst snd bestPrune prunes
+             in checkDepsPresent prunes $ fst <$> pruneDeps fst fst snd bestPrune prunes
 
 type PruneCheck = ((Int, Char), [(Int, Char)])
 
@@ -256,3 +273,7 @@ checkDepsPresent prunes selected =
         case Map.lookup ident depMap of
             Nothing -> error "checkDepsPresent: missing in depMap"
             Just deps -> Set.null $ Set.difference (Set.fromList deps) allIds
+
+-- addSymbols can't be reasonably tested like this
+fakeAddSymbols :: Monad m => Conduit (DumpPackage a b c) m (DumpPackage a b Bool)
+fakeAddSymbols = CL.map (\dp -> dp { dpSymbols = False })
