@@ -1,27 +1,30 @@
 module StackTest where
 
+import Control.Monad
+import Control.Monad.IO.Class
+import Control.Monad.Trans.Reader
+import Control.Concurrent
 import Control.Exception
 import Data.List (intercalate)
 import System.Environment
 import System.FilePath
 import System.Directory
 import System.IO
+import System.IO.Error
 import System.Process
 import System.Exit
-import System.Info (os)
+import System.Info (arch, os)
 
 run' :: FilePath -> [String] -> IO ExitCode
 run' cmd args = do
-    logInfo $ "Running: " ++ cmd ++ " " ++ intercalate " " (map showProcessArgDebug args)
+    logInfo $ "Running: " ++ cmd ++ " " ++ unwords (map showProcessArgDebug args)
     (Nothing, Nothing, Nothing, ph) <- createProcess (proc cmd args)
     waitForProcess ph
 
 run :: FilePath -> [String] -> IO ()
 run cmd args = do
     ec <- run' cmd args
-    if ec == ExitSuccess
-        then return ()
-        else error $ "Exited with exit code: " ++ show ec
+    unless (ec == ExitSuccess) $ error $ "Exited with exit code: " ++ show ec
 
 stack' :: [String] -> IO ExitCode
 stack' args = do
@@ -31,23 +34,76 @@ stack' args = do
 stack :: [String] -> IO ()
 stack args = do
     ec <- stack' args
-    if ec == ExitSuccess
-        then return ()
-        else error $ "Exited with exit code: " ++ show ec
+    unless (ec == ExitSuccess) $ error $ "Exited with exit code: " ++ show ec
 
 stackErr :: [String] -> IO ()
 stackErr args = do
     ec <- stack' args
-    if ec == ExitSuccess
-        then error "stack was supposed to fail, but didn't"
-        else return ()
+    when (ec == ExitSuccess) $ error "stack was supposed to fail, but didn't"
+
+type Repl = ReaderT ReplConnection IO
+
+data ReplConnection
+  = ReplConnection
+    { replStdin  :: Handle
+    , replStdout :: Handle
+    }
+
+nextPrompt :: Repl ()
+nextPrompt = do
+    (ReplConnection _ handle) <- ask
+    c <- liftIO $ hGetChar handle
+    if c == '>'
+      then do _ <- liftIO $ hGetChar handle
+              return ()
+      else nextPrompt
+
+replCommand :: String -> Repl ()
+replCommand cmd = do
+    (ReplConnection input _) <- ask
+    liftIO $ hPutStrLn input cmd
+
+replGetLine :: Repl String
+replGetLine = fmap replStdout ask >>= liftIO . hGetLine
+
+replGetChar :: Repl Char
+replGetChar = fmap replStdout ask >>= liftIO . hGetChar
+
+runRepl :: FilePath -> [String] -> ReaderT ReplConnection IO () -> IO ExitCode
+runRepl cmd args actions = do
+    logInfo $ "Running: " ++ cmd ++ " " ++ unwords (map showProcessArgDebug args)
+    (Just rStdin, Just rStdout, Just rStderr, ph) <-
+        createProcess (proc cmd args)
+        { std_in = CreatePipe
+        , std_out = CreatePipe
+        , std_err = CreatePipe
+        }
+    hSetBuffering rStdin NoBuffering
+    hSetBuffering rStdout NoBuffering
+    hSetBuffering rStderr NoBuffering
+
+    forkIO $ withFile "/tmp/stderr" WriteMode
+        $ \err -> forever $ catch (hGetChar rStderr >>= hPutChar err)
+                  $ \e -> unless (isEOFError e) $ throw e
+
+    runReaderT (nextPrompt >> actions) (ReplConnection rStdin rStdout)
+    waitForProcess ph
+
+repl :: [String] -> Repl () -> IO ()
+repl args action = do
+    stack <- getEnv "STACK_EXE"
+    ec <- runRepl stack ("repl":args) action
+    unless (ec == ExitSuccess) $ return ()
+        -- TODO: Understand why the exit code is 1 despite running GHCi tests
+        -- successfully.
+        -- else error $ "Exited with exit code: " ++ show ec
 
 -- | Run stack with arguments and apply a check to the resulting
 -- stderr output if the process succeeded.
 stackCheckStderr :: [String] -> (String -> IO ()) -> IO ()
 stackCheckStderr args check = do
     stack <- getEnv "STACK_EXE"
-    logInfo $ "Running: " ++ stack ++ " " ++ intercalate " " (map showProcessArgDebug args)
+    logInfo $ "Running: " ++ stack ++ " " ++ unwords (map showProcessArgDebug args)
     (ec, _, err) <- readProcessWithExitCode stack args ""
     hPutStr stderr err
     if ec /= ExitSuccess
@@ -92,13 +148,8 @@ fileContentsMatch f1 f2 = do
     doesExist f2
     f1Contents <- readFile f1
     f2Contents <- readFile f2
-    if f1Contents == f2Contents
-          then return ()
-          else error
-                   ("contents do not match for " ++
-                    show f1 ++
-                    " " ++
-                    show f2)
+    unless (f1Contents == f2Contents) $
+        error ("contents do not match for " ++ show f1 ++ " " ++ show f2)
 
 logInfo :: String -> IO ()
 logInfo = hPutStrLn stderr
@@ -122,11 +173,16 @@ exeExt = if isWindows then ".exe" else ""
 -- | Is the OS Windows?
 isWindows = os == "mingw32"
 
--- | Is the OS Mac OS X?
-isMacOSX = os == "darwin"
+-- | Is the OS Alpine Linux?
+getIsAlpine = doesFileExist "/etc/alpine-release"
+
+-- | Is the architecture ARM?
+isARM = arch == "arm"
 
 -- | To avoid problems with GHC version mismatch when a new LTS major
 -- version is released, pass this argument to @stack@ when running in
--- a global context.  The LTS major version here should match that of
--- the main @stack.yaml@.
-defaultResolverArg = "--resolver=lts-5"
+-- a global context. The LTS major version here should match that of
+-- the main @stack.yaml@ (and ordinarily be the `.0` minor version).
+--
+-- NOTE: currently using lts-8.22 instead of lts-8.0 because the `cyclic-test-deps` integration test is broken with lts-8.0 because a hackage metadata revision invalidated the snapshot (snapshot has `test-framework-quickcheck2-0.3.0.3` and `QuickCheck-2.9.2`, which used to be fine, but now test-framework-quickcheck2 was revised to have a `QuickCheck < 2.8` constraint).
+defaultResolverArg = "--resolver=lts-8.22"
